/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    PIC24F mTouch CTMU Framework
 *  FileName:        mTouch.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework filtering and decoding module.
 *                   This module provides a group of functions performing the post
 *                   acquisition filtering and decoding for all the sensors.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and INT16ellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/

///////////////////////////////
// INCLUDES 
///////////////////////////////
#include "mTouch.h"
  
///////////////////////////////
// GLOBAL VARIABLES
///////////////////////////////
/* Global Sensor Variables
 * These variables store information about the current state of the sensors.
 * They should only ever be updated by the mTouch framework, but may be read
 * globally so that special actions can be taken based on the sensors' status.
*/ 

volatile UINT8       buttonState   [MTOUCH_NUMBER_SENSORS];    // Tracks the current state of each sensor
volatile UINT16      average       [MTOUCH_NUMBER_SENSORS];    // Tracks environmental changes in the system
volatile INT8        decodeDataIsReady = 0;                    // This flag set to 1 when decoding is completed
volatile SENSOR      sensors       [MTOUCH_NUMBER_SENSORS];    // The array contains configuration data for each sensor.

///////////////////////////////
// LOCAL VARIABLES
///////////////////////////////
/* Local Variables
 * These variables are used INT16ernally by the filtering and decoding framework modules.
 * They should never be accessed outside of the mTouchCVD.c file.
*/ 
#if defined(AVG_ENABLE)
volatile UINT16      averageCount  [MTOUCH_NUMBER_SENSORS];    // Counter variable for updating averages
#endif

#if defined(BUTTON_TIMEOUT)
#if BUTTON_TIMEOUT > 0
volatile UINT8       pressTimer    [MTOUCH_NUMBER_SENSORS];    // Counter variable for button timeout implementation
#endif
#endif

volatile UINT8      debounceCount  [MTOUCH_NUMBER_SENSORS];    // Counter variable for state transition debouncing
volatile UINT16     initCounter;                               // Counter variable for initialization

///////////////////////////////
// LOCAL PROTOTYPES
///////////////////////////////
void    MTouchFilterUpdateAverage(UINT16, UINT8);
void    MTouchFilterInit         (void);


//*************************************************************************
// void MTouchAddSensor(UINT8 sensorNumber, UINT8* tris, UINT8* lat,
//                      UINT8 ioBitNumber, UINT8 channelNumber,
//                      UINT16 pressThreshold, UINT16 releaseThreshold) - mTouch Framework's Routine to Add Sensor
//*************************************************************************
void MTouchAddSensor(UINT8 sensorNumber, UINT16* tris, UINT16* lat, UINT8 ioBitNumber, UINT8 channelNumber, UINT16 pressThreshold, UINT16 releaseThreshold)
{
    sensors[sensorNumber].pressThreshold = pressThreshold;
    sensors[sensorNumber].releaseThreshold = releaseThreshold;
	sensors[sensorNumber].trisReg = tris;
	sensors[sensorNumber].latReg = lat;
    sensors[sensorNumber].ioMask = 1<<ioBitNumber; // calculate the bit mask
	sensors[sensorNumber].channelBits = channelNumber;
}

//*************************************************************************
// void MTouchSetSensorThresholds(UINT8 sensorNumber, UINT16 pressThreshold, UINT16 releaseThreshold) - mTouch Framework's Routine to Change Button Threshold
//*************************************************************************
void MTouchSetSensorThresholds(UINT8 sensorNumber, UINT16 pressThreshold, UINT16 releaseThreshold)
{
    sensors[sensorNumber].pressThreshold = pressThreshold;
    sensors[sensorNumber].releaseThreshold = releaseThreshold;
}

//*************************************************************************
// mTouchMTOUCHInit() - mTouch Framework's Initialization Routine
//*************************************************************************
void MTouchInit(void)
{
UINT8 i;
   
    // 1. Loop through all sensors and initialize array variables

    for (i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
    {
        buttonState     [i] = MTOUCH_INITIALIZING;
        debounceCount   [i] = 0;
        
        #if defined(BUTTON_TIMEOUT)
        #if BUTTON_TIMEOUT > 0
        pressTimer      [i] = 0;
        #endif
        #endif
    }
    
    /// 2. Initialize non-array system variables
    initCounter = POWER_UP_SAMPLES;
    
    /// 3. Initialize the ADC module for correct scanning operation
    MTOUCH_ADC_INIT(); // see mTouchMTOUCH_HardwareProfile... 
    
    /// 4. Initialize the framework's filtering module
    MTouchFilterInit();
    
    /// 5. Initialize the framework's communication module, if enabled by configuration options
    #if defined(MTOUCH_DEBUG)
       MTouchCommInit();          
    #endif

}


//*************************************************************************
// MTouchFilterInit() - Filtering Module's Initialization Routine
//*************************************************************************
void MTouchFilterInit(void)
{
UINT8 i;

    for (i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
    {        
        #if defined(AVG_ENABLE)
        average[i]      = 0;
        averageCount[i] = 0;
        #endif

        oversampleFilterData[i] = 0;
        decimateFilterData[i]   = 0;
        sensorData[i]           = 0;        
    } 
    
}

//*************************************************************************
// MTouchDecode() - Decoding Module's Service Routine
//*************************************************************************
void MTouchDecode(void)
{
UINT8 i;
INT16 delta;
#if defined(MTOUCH_DEBUG)
UINT8 areInitialized;

     areInitialized = 1;
#endif

    if(MTouchIsDataReady() == 0) // These is no new information ready?
        return;

    // This function loops through each sensor and performs the following algorithm:
  
    // For each active sensor...
    for(i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
    {
        /* 
        * 1. Get the current shift amount by comparing the new reading with the average
        * and store it in the variable 'delta'.
        */
        delta   = MTouchGetSensor(i) - MTouchGetAverage(i); 

        if (delta < 0)                         
        {
#ifdef MTOUCH_USE_NEGATIVE_PULSE
            delta = -delta;
#else
            delta = 0;
#endif
        }
        
        /*
        * 2. Based on the sensor's current state, perform one of the following procedures:
        */
        switch(MTouchGetButtonState(i))                           // Begin state machine
        {
            /*
            *
            * This is the default state of all sensors on power up. Once sensors leave the
            * initialization state, they do not return. In this state, it begins by
            * decrementing the initCounter variable. If #initCounter has not yet reached
            * 0, the average is updated to equal the current reading and a flag is set
            * to stop any communication data from being transmitted. If initCounter has
            * reached 0: the sensor transitions to the MTOUCH_RELEASED state, the debounce
            * counter is reset to 0, and the average is updated one final time to be 
            * equal to the current reading's value.
            */
            case MTOUCH_INITIALIZING:    
            {
                if (--initCounter == 0)                         // If initialization delay has elapsed
                {
                    // State Transition -> Released
                    initCounter = 1;                            // Sets the initCounter variable so that
                                                                // all future sensors will be immediately
                                                                // transitioned to 'released'.
                                                                
                    MTouchSetButtonState(MTOUCH_RELEASED,i);         // Sensor is now released and stable                    
                    debounceCount[i]    = 0;                    // Initialize sensor's debounce counter
                    average[i] = MTouchGetSensor(i);               // Initialize sensor's average
                }
                else                                            // If initialization delay is not complete
                {
                    #if defined(MTOUCH_DEBUG)  // (Only if MTOUCH_DEBUG is enabled)
                    areInitialized = 0;                         // Disable data transfer until initialized
                    #endif
                    average[i] = MTouchGetSensor(i);            // Initialize sensor's average
                }
                break;                                          
            }                                                   
            
            /*
            * 
            * This state can be entered from either the MTOUCH_INITIALIZING or MTOUCH_PRESSED 
            * state. The only way to leave this state is by creating enough consecutive
            * positive shifts to overflow the debounceCount[i] variable. Once this is done,
            * the sensor will enter the pressed state.
            * The code for this section begins by checking to see if the latest reading
            * is greater than the average by at least the press threshold. If it is, it
            * performs the debouncing logic. Once the debounce count has been reached, the
            * sensor will change to the MTOUCH_PRESSED state, reset the debounce counter, and
            * initialize the press timer (if enabled).
            * If the threshold has not been crossed, the debounce counter is reset and 
            * the average is updated.
            *
            * Note: This is the only state and condition where the average is able to be 
            * updated, and we only use the latest reading to update the average once we 
            * know it's not crossing any thresholds.
            */
            case MTOUCH_RELEASED:                                          // Sensor is released and stable
            {                  

                if(delta > sensors[i].pressThreshold)                           // If reading has crossed the press threshold
                {
                    if (++debounceCount[i] > DEBOUNCE_COUNT_PRESS)      // Increment the debounce counter
                    {                                                   // and check if it has crossed its limit
                        // State Transition -> Pressed
                        MTouchSetButtonState(MTOUCH_PRESSED, i);             // Sensor is now pressed
                        debounceCount[i]    = DEBOUNCE_COUNT_RELEASE;   // Initialize the pressed state's debounce variable
                        
                        #if defined(BUTTON_TIMEOUT)                     // If the press timer is enabled
                        #if BUTTON_TIMEOUT > 0
                        pressTimer[i]       = BUTTON_TIMEOUT;           // Initialize the press timer counter
                        #endif
                        #endif
                    }
                }
                else                                                    // If the reading has not crossed the press threshold
                {
                    debounceCount[i] = 0;                               // Reset the debounce variable 
                    MTouchFilterUpdateAverage(MTouchGetSensor(i), i);       // Update the average
                }
                break;
                
            }

            /*
            * 
            * A sensor can only enter this state from the MTOUCH_RELEASED state, but there
            * are two ways to leave: First, by being in the state for too INT32 and allowing
            * the press timer to run out. Second, by having a the delta value smaller than
            * the release threshold on enough consecutive scans to cause the debounce
            * counter to reach 0.
            * The code for this section begins by updating and checking the press timer
            * counter variable. If it has reached 0, the sensor transfers to the MTOUCH_RELEASED
            * state, the debounce counter is reset and the average is re-initialized to
            * the current reading. This is done to allow for a quick recovery if some
            * environmental condition caused the sensor to become stuck.
            * Next, the reading is checked against the release threshold. If it has 
            * crossed the threshold, the debouncing and then a state transition to 
            * MTOUCH_RELEASED is performed. If it has not crossed the threshold, the
            * debounce counter is reset.
            * Note: The average is never updated while the sensor is pressed so the system
            * can remember where the unpressed value should end up. Because of this, the
            * average can be better thought of as a released-state average.
            */
            case MTOUCH_PRESSED:
            {

                #if defined(BUTTON_TIMEOUT)                     // (Only if the press timer has been enabled)
                #if BUTTON_TIMEOUT > 0   

                if (--pressTimer[i] <= 0)                       // Decrement the press timer counter
                {                                               // and check if it has reached 0.
                    // State Transition -> Released
                    MTouchSetButtonState(MTOUCH_RELEASED, i);        // Sensor is now released
                    debounceCount[i] = 0;                       // Initialize the debounce counter for the MTOUCH_PRESSED state
                    average[i] = MTouchGetSensor(i);               // Re-initialize the average to the current reading value
                    
                    break;
                }
                
                #endif
                #endif
            
            
                if(delta < sensors[i].releaseThreshold)                 // If the reading has crossed the release threshold
                {
                    if (--debounceCount[i] == 0)                // Decrement the debounce counter
                    {                                           // and check if it has reached 0.
                        MTouchSetButtonState(MTOUCH_RELEASED, i);    // Sensor is now released 
                    }
                }
                else                                            // If the reading has not crossed the release threshold
                {
                    debounceCount[i] = DEBOUNCE_COUNT_RELEASE;  // Reset the debounce counter
                }
                break;
            }
            
            default: break; 
            
        } // End state machine switch command

    } // End for loop iterating over each sensor
    
    /*
    * 3. Now that all state machine decoding is complete, transmit fresh data information.
    */
    #if defined(MTOUCH_DEBUG)                      // (Only if debugging communications are enabled)
    if (areInitialized)                            // Only trasmit data if all sensors are initialized
    {
        decodeDataIsReady = 1;
     }
    #endif
    

    mTouchDataReady = 0; // Clear ready flag
   
} 

//*************************************************************************
// MTouchFilterUpdateAverage() - Filtering Module
// Updates a sensor's average (or baseline) in order to track the
// environmental changes of the system. 
//*************************************************************************

void MTouchFilterUpdateAverage(UINT16 reading, UINT8 index)
{
    /*
    *
    * This function has several possible implementations based on the current 
    * configuration options of the framework. The important configuration
    * options are:
    * NEGATIVE_CAPACITANCE - determines how the average handles a negative reading shift
    * AVG_RATE - acts as a debounce counter for updating only every other scan, etc
    * AVG_UPDATE - determines what weight to give the reading when updating the average
    *
    * The implementation begins by checking if NEGATIVE_CAPACITANCE is set to 1.
    * If so, the average counter variable is set so that the average will perform its
    * update regardless of the previous count. If NEGATIVE_CAPACITANCE is instead 
    * equal to 2, the average is immediately updated to follow the reading and the 
    * function is exited.
    * Next, the average updates the counter variable and compares it with AVG_RATE
    * to see if it should continue or return. If the counter allows it to continue,
    * the average is then updated using one of four possible implementations
    * which is deteremined by the AVG_UPDATE option.
    */
    #if     NEGATIVE_CAPACITANCE == 1
    if ((signed INT16)(reading - average[index]) < 0)     // If there was a negative shift
    {
        averageCount[index] = AVG_RATE;                 // Set the average counter to allow an immediate average update
    }
    #elif   NEGATIVE_CAPACITANCE == 2                       
    if ((signed INT16)(reading - average[index]) < 0)     // If there was a negative shift
    {
        average[index] = reading;                       // Re-initialize the average to the current reading value
        return;                                         // and return from the function - not performing the update algorithm
    }
    #endif

    if (++averageCount[index] >= AVG_RATE)                          // If the average counter has been exceeded
    {
        averageCount[index] = 0;                                    // Reset the average counter

        #if     (AVG_UPDATE == 2)                                   // Calculate average update
        
            // avg = (avg + reading) / 2   
            average[index]  += reading;                             
            average[index] >>= 1;
            
        #elif   (AVG_UPDATE == 4)                                   
        
            // avg = avg - (25% * avg) + (25% * reading)        
            average[index]  -= (UINT16)(average[index] >> 2); 
            average[index]  += (UINT16)(reading >> 2);
            
        #elif   (AVG_UPDATE == 8)                                   
        
            // avg = avg - (12.5% * avg) + (12.5% * reading)   
            average[index]  -= (UINT16)(average[index] >> 3); 
            average[index]  += (UINT16)(reading >> 3);
            
        #elif   (AVG_UPDATE == 16)                                  
        
            // avg = avg - (6.25% * avg) + (6.25% * reading)
            average[index]  -= (UINT16)(average[index] >> 4); 
            average[index]  += (UINT16)(reading >> 4);
            
        #endif
    }
}



