/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    PIC18F mTouch CVD and CTMU Framework
 *  FileName:        mTouchCVD.h
 *  Dependencies:    GenericTypeDefs.h, mTouch_Config.h, mTouch_HardwareProfile.h
 *
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework filtering and decoding module.
 *                   This module provides a group of functions performing the post
 *                   acquisition filtering and decoding for all the sensors.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and INT16ellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/

#ifndef __MTOUCH_H
#define __MTOUCH_H

///////////////////////////////
// INCLUDES 
///////////////////////////////
#include <p24Fxxxx.h>

#include "GenericTypeDefs.h"
#include "mTouch_Config.h"
#include "mTouch_HardwareProfile.h"

///////////////////////////////
//  STRUCTURES
///////////////////////////////
// Contains IO, A2D, decoding thresholds settings for the sensor
typedef struct tag_SENSOR {
UINT16   pressThreshold;   // press detection threshold
UINT16   releaseThreshold; // release detection threshold
UINT16* trisReg;          // TRIS register address
UINT16* latReg;           // LAT register address
UINT16  ioMask;           // sensor bit mask for LAT and TRIS registers
UINT16  channelBits;      // bit pattern to select the sensor A2D channel
} SENSOR;

///////////////////////////////
// GLOBAL VARIABLES
///////////////////////////////
extern     volatile UINT8  mTouchDataReady;                                // Set to 1 when new scan data are ready
extern     volatile SENSOR sensors              [MTOUCH_NUMBER_SENSORS];   // Array of sensors information
extern     volatile UINT8  buttonState          [MTOUCH_NUMBER_SENSORS];   // Tracks the current state of the sensor
extern     volatile UINT16 average              [MTOUCH_NUMBER_SENSORS];   // Tracks environmental changes in the system
extern     volatile UINT16 sensorData           [MTOUCH_NUMBER_SENSORS];   // Latch register storing the latest acquisition data
extern     volatile INT16  decimateFilterData   [MTOUCH_NUMBER_SENSORS];   // Decimate-filtered ADC result
extern     volatile INT16  sampleCounter;                                  // Counter variable for oversampling
extern     volatile UINT32 oversampleFilterData [MTOUCH_NUMBER_SENSORS];   // Accumulator array for sensor oversampling
extern     volatile UINT8  currentSensor;                                  // Current sensor index 
extern     volatile UINT8  previousSensor;                                 // Previous sensor index
extern     volatile INT8   decodeDataIsReady;                              // This flag set to 1 when decoding is completed.


///////////////////////////////
// ENUMERATIONS
///////////////////////////////
/// Sensor states
extern enum  _MTOUCHBUTTONSTATE_ 
             {
                MTOUCH_INITIALIZING,   // Sensor is still initializing
                MTOUCH_RELEASED,       // Sensor is currently released
                MTOUCH_PRESSED         // Sensor is currently pressed
             } MTOUCHBUTTONSTATE;    

///////////////////////////////
// GLOBAL PROTOTYPES
///////////////////////////////

/****************************************************************************
  Function:
    void  MTouchAcquisition(void)

  Summary:
    Gets a new sample.

  Description:
    Gets a new sample (using CVD or CTMU). Contains decimate and oversampling 
    filters. Sets mTouchDataReady flag when oversampling is finished and
    there are new data for decoding (implemented in MTouchDecode() function).
 
  Precondition:
    Before use this function the initialization should be done with MTouchInit()
    and MTouchAddSensor(...) functions. mTouchDataReady flag must be cleared
    by MTouchDecode() function call.

  Parameters:
    None.

  Returns:
    None.

  Remarks:
    This function can be called periodically by timer INT16errupt. Setting of
    mTouchDataReady flag allows decoding with MTouchDecode() function.
  ***************************************************************************/
void  MTouchAcquisition(void);

/****************************************************************************
  Function:
    INT16 ACQ_CVD(void)

  Summary:
    Gets a new sample.

  Description:
    Gets a new sample using CVD.
 
  Precondition:
    None.

  Parameters:
    None.

  Returns:
    Differential CVD value for the current button.

  Remarks:
    This function is called by MTouchAcquisition(). 
  ***************************************************************************/
INT16  ACQ_CVD(void);

/****************************************************************************
  Function:
    INT16 ACQ_CTMU(void)

  Summary:
    Gets a new sample.

  Description:
    Gets a new sample using CTMU.
 
  Precondition:
    None.

  Parameters:
    None.

  Returns:
    CTMU value for the current button.

  Remarks:
    This function is called by MTouchAcquisition(). 
  ***************************************************************************/
INT16  ACQ_CTMU(void);

/****************************************************************************
  Function:
    void MTouchDecode(void)

  Summary:
    Decodes the buttons' states.

  Description:
    Decodes the buttons' states. Clears mTouchDataReady flag when decoding
    is finished.
 
  Precondition:
    Before use this function the initialization should be done with MTouchInit().
    and  MTouchAddSensor(...) functions. mTouchDataReady flag must be set
    by MTouchAcquisition() function calls.

  Parameters:
    None.

  Returns:
    None.

  Remarks:
    Clearing mTouchDataReady flag resumes acquisition (implemented in
    MTouchAcquisition()). 
  ***************************************************************************/
void    MTouchDecode (void);

/****************************************************************************
  Function:
    void MTouchInit(void)

  Summary:
    Initializes mTouch module.

  Description:
    Initializes mTouch module: decimate, oversampling and average filters,
    buttons states.
 
  Precondition:
    None.

  Parameters:
    None.

  Returns:
    None.

  Remarks:
    None.
  ***************************************************************************/
void    MTouchInit (void);

/****************************************************************************
  Function:
    void MTouchAddSensor(UINT8 sensorNumber, UINT8* tris,
                         UINT8* lat, UINT8 ioBitNumber,
                         UINT8 channelNumber, UINT16 pressThreshold,
                         UINT16 releaseThreshold)
  Summary:
    Adds info about sensor.

  Description:
    Adds info about sensor such as IO port, A2D channel, press/release detection
    thresholds.
 
  Precondition:
    None.

  Parameters:
    sensorNumber - sensor number.
    tris - address of TRIS register for the sensor.
    lat - address of LAT register for the sensor.
    ioBitNumber - sensor IO bit number for LAT and TRIS registers.
    channelNumber - A2D analog input number for the sensor.
    pressThreshold - press detection threshold (usually 15-65).
    releaseThreshold - release detection threshold (usually half of
                       press detection threshold).

  Returns:
    None.

  Remarks:
    All sensors must be added before aquisition.
  ***************************************************************************/
void MTouchAddSensor(UINT8 sensorNumber, UINT16* tris, UINT16* lat, UINT8 ioBitNumber,
                     UINT8 channelNumber, UINT16 pressThreshold, UINT16 releaseThreshold);

/****************************************************************************
  Function:
    void MTouchSetSensorThresholds(UINT8 sensorNumber, UINT16 pressThreshold, UINT16 releaseThreshold)

  Summary:
    Changes the press and release detection thresholds.

  Description:
    Changes the press and release detection thresholds for the sensor.
 
  Precondition:
    None.

  Parameters:
    sensorNumber - sensor number.
    pressThreshold - press detection threshold (usually 15-65).
    releaseThreshold - release detection threshold (usually half of
                       press detection threshold).

  Returns:
    None.

  Remarks:
    None.
  ***************************************************************************/
void MTouchSetSensorThresholds(UINT8 sensorNumber, UINT16 pressThreshold, UINT16 releaseThreshold);

///////////////////////////////
// MACROS
///////////////////////////////

/****************************************************************************
  Macros:
    UINT8 MTouchGetButtonState(UINT8 i)

  Summary:
    Returns the current sensor state.

  Description:
    Returns the current sensor state. Valid values are MTOUCH_INITIALIZING,
    MTOUCH_RELEASED, MTOUCH_PRESSED.
 
  Precondition:
    None.

  Parameters:
    i - sensor number.

  Returns:
    Current sensor state.

  Remarks:
    None.
  ***************************************************************************/
#define     MTouchGetButtonState(i)         buttonState[i]

/****************************************************************************
  Macros:
    UINT8 MTouchIsDataReady(void)

  Summary:
    Returns 1 if new data are available for decoding.

  Description:
    Returns 1 if new data are available for decoding. The new data ready flag
    is set by mTouchAcguisition() and cleared by mTouchDecode().
 
  Precondition:
    None.

  Parameters:
    None.

  Returns:
    Returns 1 if new data are available for decoding.

  Remarks:
    None.
  ***************************************************************************/
#define     MTouchIsDataReady()             mTouchDataReady

/****************************************************************************
  Function:
    UINT8 MTouchJitter(void)

  Summary:
    Returns a random number from 0 to 255.

  Description:
    Returns a random number from 0 to 255. This function should be used 
    to create a jitter between acquisitions to improve EMC.
 
  Precondition:
    None.

  Parameters:
    None.

  Returns:
    Random number from 0 to 255.

  Remarks:
    None.
  ***************************************************************************/
UINT8 MTouchJitter(void);

#define     MTouchSetButtonState(state, i) {buttonState[i] = state;}     // Abstraction macro for changing button state
#define     MTouchGetAverage(i)             average[i]                   // Abstraction macro for array access
#define     MTouchGetSensor(i)              sensorData[i]                // Abstraction macro for array access                     

// Communication Functions
#if defined(MTOUCH_DEBUG)
    void    MTouchCommInit     (void);
    void    MTouchSendData     (void);
#endif

#endif
