/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch CVD Framework v1.00.00
 *  FileName:        mTouch_Comm.c
 *  Dependencies:    mTouchCVD.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Communication Module
 *                   - Implements the communication between the mTouch
 *                     PC GUI program and the firmware. May also be used
 *                     to output directly to a PC's terminal.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file mTouch_Comm.c
* @brief Implements the communication function of the framework used for debugging
*/
//*****************************************************************************
// Include and Header files
//*****************************************************************************
#include <stdlib.h>
#include "mTouch.h"

#if defined(MTOUCH_DEBUG)
//*****************************************************************************
// Local Function Prototypes
//*****************************************************************************
    ////////////////////////////////////////////////////
    /* System Support Functions */
    ////////////////////////////////////////////////////
void           MTouchCommInit(void);

    ////////////////////////////////////////////////////
    /* Communication Functions */
    ////////////////////////////////////////////////////
void           MTouchSendData(void);

    ////////////////////////////////////////////////////
    /* Communication Support Functions */
    ////////////////////////////////////////////////////
void           PutChar     (unsigned char);
void           StrInt      (unsigned int);

///////////////////////////////////////////////////////////////////////////////
//*****************************************************************************
// SUPPORT FUNCTIONS
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////

/****************************************************************
* Function: void Init(void)
*
* Overview: This function configures the peripherals and presets
*           the system variables
*
* Input:    None
*
* Output:   None
*
****************************************************************/
/**
*   @brief This function configures the communiation peripherals and presets the system variables
*/
void MTouchCommInit(void)
{

__builtin_write_OSCCONL(OSCCON & 0xbf); //unlock PPS
RPOR8bits.RP16R = 5;
 __builtin_write_OSCCONL(OSCCON | 0x40); //lock   PPS
U2BRG = 34; //115200
U2MODE = 0;
U2MODEbits.BRGH = 1;
U2STA = 0;
U2MODEbits.UARTEN = 1;
U2STAbits.UTXEN = 1;
IFS1bits.U2RXIF = 0;
 
}


///////////////////////////////////////////////////////////////////////////////
//*****************************************************************************
// Communications Functions
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////

/**
*   @brief Performs a loop and outputs data to the PC according to the set protocol
*/
void MTouchSendData(void)
{
unsigned int btnMsk;    
unsigned int bufMsk;
unsigned char i;

    // Sensor state information is sent before outputting the current reading values.
    btnMsk = 0;
    bufMsk = 1;
    for (i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
    {
        if (MTouchGetButtonState(i) < MTOUCH_PRESSED)
        {
            btnMsk |= bufMsk;
        }
        bufMsk <<= 1;
    }
    btnMsk = ~btnMsk;            // Bitwise 'NOT' because bits are inverted 
        
    StrInt(btnMsk);             // Output sensor state data
    
    // For all versions of the GUI, output the current reading values.

    for (i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
    { 
        #if defined(MTOUCH_DEBUG_OUTPUT_RAW)
        StrInt(MTouchGetSensor(i));
        #endif
        #if defined(MTOUCH_DEBUG_OUTPUT_AVG)
        StrInt(MTouchGetAverage(i));
        #endif
    }
    
    // Carriage return, line feed
    PutChar(0x0D);
    PutChar(0x0A);    
}

///////////////////////////////////////////////////////////////////////////////
//*****************************************************************************
// Communications Support functions
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////

/****************************************************************
* Function: void str_Int(unsigned int i)
*
* Overview: This function sends an INT as a space and 5 digits
*
* Input:    Din is the value to be converted
*           base is the radix of the value
*
* Output:   None
*
****************************************************************/
/**
*   @brief Sends an integer through the serial port
*
*   @param i The integer to send
*/
void StrInt(unsigned int i)
{
	unsigned char ctr = 0;
    
    #if defined(MTOUCH_DEBUG_OUTPUT_HEX)
    
    // Hex Output
    ctr = (unsigned char)(i >> 12);
    if (ctr <= 9) { PutChar(ctr+0x30); } else { PutChar(ctr+0x37); }
    ctr = (unsigned char)(i >> 8) & 0x0F;
    if (ctr <= 9) { PutChar(ctr+0x30); } else { PutChar(ctr+0x37); }
    ctr = (unsigned char)(i >> 4) & 0x0F;
    if (ctr <= 9) { PutChar(ctr+0x30); } else { PutChar(ctr+0x37); }
    ctr = (unsigned char)(i & 0x0F);
    if (ctr <= 9) { PutChar(ctr+0x30); } else { PutChar(ctr+0x37); }
    
    #else
    
    // Decimal Output
 	while (i >= 10000)  { i -= 10000; ctr++; } PutChar(ctr+0x30); ctr=0;
 	while (i >=  1000)  { i -=  1000; ctr++; } PutChar(ctr+0x30); ctr=0;
	while (i >=   100)  { i -=   100; ctr++; } PutChar(ctr+0x30); ctr=0;
	while (i >=    10)  { i -=    10; ctr++; } PutChar(ctr+0x30); ctr=0;
	while (i >=     1)  { i -=     1; ctr++; } PutChar(ctr+0x30);	
    
    #endif
    
    // A semi-colon delimiter is used for the GUI. This can be changed if
    // you are not using the GUI to process the output.
    PutChar(';');
}


/**
*   @brief Sends a single character through the serial port.
*
*   @param Dout The character to send
*/
void PutChar(unsigned char dout)
{
    while(U2STAbits.TRMT == 0);
    Nop(); Nop();
    U2TXREG = dout;
}

#endif

