/**
 * @file OnePIC_32bit_timer.c
 * @brief contains all timer related tasks such as:
 *  - Turning ON/OFF the timer
 *  - Setting prescaler
 *  - Loading PR register (rollover limit)
 * @defgroup thirtytwo_bit_timer Timers
 * @ingroup thirtytwo_bit
 * @{
 */
#include <stdint.h>
#include "OnePIC_timer.h"
#include "Compiler.h"

/**
 * @brief selects whether or not the source will be internal or external
 * @param index determines the source of the timer
 * @param source what is to be used?
 */
void EDF_TMR_ClockSourceSelect(TMR_MODULE_ID index, EDF_TMR_CLOCK_SOURCE source)
{
    switch(index) {
        case TMR_1:
            T1CONbits.TCS = source;
            break;
        default:
            break;
    }
}
/**
 * @brief disables the timer source
 * @param index source to disable
 */
void EDF_TMR_StopInIdleDisable(TMR_MODULE_ID index)
{
    switch(index)
    {
        case TMR_1:
            T1CONbits.SIDL = 0;
            break;
        default:
            break;
    }
}
/**
 * @brief disables the timer's gate if used
 * @param index source to disable their gate
 */
void EDF_TMR_GateDisable(TMR_MODULE_ID index)
{
    switch(index)
    {
        case TMR_1:
            T1CONbits.TGATE = 0;
            break;
        default:
            break;
    }
}
/**
 * @brief sets the prescaler for the timer
 * @param index timer to edit
 * @param prescaler sets the prescaler for the respective timer
 */
void EDF_TMR_PrescalerSet(TMR_MODULE_ID index, EDF_TMR_PRESCALE prescaler)
{
    switch(index)
    {
        case TMR_1:
            T1CONbits.TCKPS = prescaler;
            break;
        default:
            break;
    }
}
/**
 * @brief starts te timer
 * @param index timer to start
 */
void EDF_TMR_Start(TMR_MODULE_ID index)
{
    switch(index)
    {
        case TMR_1:
            T1CONbits.ON = 1;
            break;
        default:
            break;
    }
}
/**
 * @brief sets the 16bit timer used for TMR1
 * @param index timer to start
 * @param PeriodValue PR value to assign
 */
void EDF_TMR_Period16BitSet(TMR_MODULE_ID index, uint16_t PeriodValue)
{
    switch(index)
    {
        case TMR_1:
            PR1 = PeriodValue;
            break;
        default:
            break;
    }
}
/**
 * @brief stops the timer
 * @param index timer to stop
 */
void EDF_TMR_Stop(TMR_MODULE_ID index)
{
    switch(index)
    {
        case TMR_1:
            T1CONbits.ON = 0;
            break;
        default:
            break;
    }
}
/**
 * @brief set the 16bit timer (TMR1)
 * @param index timer to stop
 * @param value timer value to assign to TMR1
 */
void EDF_TMR_Counter16BitSet(TMR_MODULE_ID index, uint16_t value)
{
    switch(index)
    {
        case TMR_1:
            EDF_TMR_Stop(index);
            TMR1 = value;
            EDF_TMR_Start(index);
            break;
        default:
            break;
    }
}
