
/**
 * @file OnePIC_8bit_int.c
 * @brief Controls all interrupt related tasks for the 8bit such as:
 *  - Enable interrupt
 *  - Disable interrupt
 *  - Clear interrupt flag
 *  - Set interrupt flag
 *
 * @note Supports only timers 0,1,2,4,6 and SSPIF
 * @defgroup eight_bit 8bit 
 * 
 * @{
 */

#include "Compiler.h"
#include "OnePIC_int.h"

/**
 * @brief enables a specific interrupt
 * @param source the source of the interrupt to enable
 */
void EDF_INT_SourceEnable(EDF_INT_SOURCE source) {
    switch (source) {
        case INT_TMR_0:
            INTCONbits.TMR0IE = 1;
            break;
        case INT_TMR_1:
            PIE1bits.TMR1IE = 1;
            break;
        case INT_TMR_2:
            PIE1bits.TMR2IE = 1;
            break;
        case INT_TMR_4:
            PIE3bits.TMR4IE = 1;
            break;
        case INT_TMR_6:
            PIE3bits.TMR6IE = 1;
            break;
        case INT_SSP:
            PIE1bits.SSPIE = 1;
        default:
            break;
    }
}

/**
 *
 * @param source
 * @return a bool indicating if the interrupt is enabled or not
 */
bool EDF_INT_SourceEnableGet(EDF_INT_SOURCE source) {
    switch (source) {
        case INT_TMR_0:
            return INTCONbits.TMR0IE;
        case INT_TMR_1:
            return PIE1bits.TMR1IE;
        case INT_TMR_2:
            return PIE1bits.TMR2IE;
        case INT_TMR_4:
            return PIE3bits.TMR4IE;
        case INT_TMR_6:
            return PIE3bits.TMR6IE;
        case INT_SSP:
            return PIE1bits.SSPIE;
        default:
            return 0;
    }
}

/**
 * @brief retrieves the interrupt flag
 * @param source source of interrupt flag to get
 * @return bool indicating if flag is set or not
 */
bool EDF_INT_SourceFlagGet(EDF_INT_SOURCE source) {
    switch (source) {
        case INT_TMR_0:
            return INTCONbits.TMR0IF;
        case INT_TMR_1:
            return PIR1bits.TMR1IF;
        case INT_TMR_2:
            return PIR1bits.TMR2IF;
        case INT_TMR_4:
            return PIR3bits.TMR4IF;
        case INT_TMR_6:
            return PIR3bits.TMR6IF;
        case INT_SSP:
            return PIR1bits.SSPIF;
        default:
            return 0;
    }
}

/**
 * @brief disables a specific interrupt
 * @param source interrupt source to disable
 */
void EDF_INT_SourceDisable(EDF_INT_SOURCE source) {
    switch (source) {
        case INT_TMR_0:
            INTCONbits.TMR0IE = 0;
            break;
        case INT_TMR_1:
            PIE1bits.TMR1IE = 0;
            break;
        case INT_TMR_2:
            PIE1bits.TMR2IE = 0;
            break;
        case INT_TMR_4:
            PIE3bits.TMR4IE = 0;
            break;
        case INT_TMR_6:
            PIE3bits.TMR6IE = 0;
            break;
        case INT_SSP:
            PIE1bits.SSPIE = 0;
        default:
            break;
    }
}

/**
 * @brief clears a specific interrupt
 * @param source interrupt flag to clear
 */
void EDF_INT_SourceFlagClear(EDF_INT_SOURCE source) {
    switch (source) {
        case INT_TMR_0:
            INTCONbits.TMR0IF = 0;
            break;
        case INT_TMR_1:
            PIR1bits.TMR1IF = 0;
            break;
        case INT_TMR_2:
            PIR1bits.TMR2IF = 0;
            break;
        case INT_TMR_4:
            PIR3bits.TMR4IF = 0;
            break;
        case INT_TMR_6:
            PIR3bits.TMR6IF = 0;
            break;
        case INT_SSP:
            PIR1bits.SSPIF = 0;
        default:
            break;
    }
}

/**
 * @brief enables all interrupts by enabling GIE
 * @note also enables the periphial interrupt (PEIE)
 */
void EDF_INT_Enable(void) {
    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1;
}

/**
 * @brief disables all interrupts by clearing GIE
 * @note also disables the periphial interrupt (PEIE)
 */
void EDF_INT_Disable(void) {
    INTCONbits.GIE = 0;
    INTCONbits.PEIE = 0;
}
/** @}*/