/**
 * @file OnePIC_8bit_timer.c
 * @brief contains all timer related tasks such as:
 *  - Turning ON/OFF the timer
 *  - Setting prescaler
 *  - Loading PR register (rollover limit)
 * @defgroup eight_bit_timer Timers
 * @ingroup eight_bit
 * @{
 */
#include "OnePIC_timer.h"
#include <stdint.h>
#include "Compiler.h"
#include "OnePIC_int.h"

/**
 * @brief selects whether or not the source will be internal or external 
 * @param index determines the source of the timer
 * @param source what is to be used?
 */
void EDF_TMR_ClockSourceSelect(TMR_MODULE_ID index, EDF_TMR_CLOCK_SOURCE source) {
    switch (index) {
        case TMR_1:
            T1CONbits.TMR1CS = source;
            T1GCONbits.TMR1GE = 0;
            break;
        case TMR_2:
            T2CONbits.TMR2ON = source;
            break;
        default:
            break;
    }
}

/**
 * @brief disables the timer source
 * @param index source to disable
 */
void EDF_TMR_StopInIdleDisable(TMR_MODULE_ID index) {
    switch (index) {
        case TMR_1:
            // Unused in P16
            break;
        default:
            break;
    }
}

/**
 * @brief disables the timer's gate if used
 * @param index source to disable their gate
 */
void EDF_TMR_GateDisable(TMR_MODULE_ID index) {
    switch (index) {
        case TMR_1:
            T1GCONbits.TMR1GE = 0;
            break;
        default:
            break;
    }
}

/**
 * @brief sets the prescaler for the timer
 * @param index timer to edit
 * @param prescaler sets the prescaler for the respective timer
 */
void EDF_TMR_PrescalerSet(TMR_MODULE_ID index, EDF_TMR_PRESCALE prescaler) {
    switch (index) {
        case TMR_0:
            OPTION_REGbits.PS = prescaler;
            break;
        case TMR_1:
            T1CONbits.T1CKPS = prescaler;
            break;
        case TMR_2:
            T2CONbits.T2CKPS = prescaler;
            break;
        case TMR_4:
            T4CONbits.T4CKPS = prescaler;
            break;
        case TMR_6:
            T6CONbits.T6CKPS = prescaler;
            break;
        default:
            break;
    }
}

/**
 * @brief starts te timer
 * @param index timer to start
 */
void EDF_TMR_Start(TMR_MODULE_ID index) {
    switch (index) {
        case TMR_1:
            T1CONbits.TMR1ON = 1;
            break;
        case TMR_2:
            T2CONbits.TMR2ON = 1;
            break;
        case TMR_4:
            T4CONbits.TMR4ON = 1;
            break;
        case TMR_6:
            T6CONbits.TMR6ON = 1;
            break;
        default:
            break;
    }
}

/**
 * @brief stops the timer
 * @param index timer to stop
 */
void EDF_TMR_Stop(TMR_MODULE_ID index) {
    switch (index) {
        case TMR_1:
            T1CONbits.TMR1ON = 0;
            break;
        case TMR_2:
            T2CONbits.TMR2ON = 0;
            break;
        case TMR_4:
            T4CONbits.TMR4ON = 0;
            break;
        case TMR_6:
            T6CONbits.TMR6ON = 0;
            break;
        default:
            break;
    }
}

/**
 * @brief loads the respective period register (PR) register
 * @param index timer to use
 * @param value period register to load
 */
void EDF_TMR_CounterSet(TMR_MODULE_ID index, uint16_t value) {
    switch (index) {
        case TMR_1:
            EDF_TMR_Stop(index);
            TMR1H = (value & 0xFF00) >> 8;
            TMR1L = (value & 0x00FF);
            EDF_TMR_Start(index);
            break;
        case TMR_2:
            EDF_TMR_Stop(index);
            PR2 = value;
            EDF_TMR_Start(index);
            break;
        case TMR_4:
            EDF_TMR_Stop(index);
            PR4 = value;
            EDF_TMR_Start(index);
            break;
        case TMR_6:
            EDF_TMR_Stop(index);
            PR6 = value;
            EDF_TMR_Start(index);
            break;
        default:
            break;
    }
}
/** @}*/
