/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework filtering and decoding modules
 *                   - No application code should be implemented in this
 *                     or any other mTouch framework file. This will allow
 *                     for better customer support and easier upgrades to
 *                     later firmware versions. Use the main.c and user-
 *                     generated files to implement your application.
 *                   - See the documentation located in the docs/ folder
 *                     for a more information about how the framework is
 *                     implemented.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file   mTouch.c
*   @brief  Implements the post-acquisition filtering and decoding of the
*           active/sleep mode sensors.
*/

//========================================================================
// INCLUDES 
//========================================================================
#include "mTouchCVD.h"
  
  
//========================================================================
// GLOBAL VARIABLES
//========================================================================
/** @name   Global Sensor Variables
* 
*   @brief  These variables store information about the current state of 
*           the sensors. They should only ever be updated by the mTouch 
*           framework, but may be read globally so that special actions 
*           can be taken based on the sensors' status.
*           Refer to the mTouch.h file for more information about these 
*           variables.
*/ 
//@{
mTouch_State                    mTouch_state;           // Collection of single-bit flags    
mTouch_StateVariables           mTouch_stateVars;       // Sensor variables and other counters which are Active/Sleep state-specific

#if defined(MTOUCH_ACTIVE_ENABLED)
    uint16_t                    mTouch_average          [MTOUCH_NUMBER_SENSORS];    
#endif

#if defined(MTOUCH_SLEEP_ENABLED)
    uint16_t                    mTouch_sleep_average    [MTOUCH_NUMBER_OF_SLEEP_SENSORS];
#endif

#if defined(MTOUCH_ACTIVE_ENABLED)
    #if defined(MTOUCH_SLIDERS_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY) 
        uint16_t                mTouch_deltas           [MTOUCH_NUMBER_SENSORS];
    #endif
#endif

#if defined(MTOUCH_TOGGLE_ENABLED)
    #if (MTOUCH_NUMBER_SENSORS <= 8)
        uint8_t                 mTouch_toggle;          
    #elif (MTOUCH_NUMBER_SENSORS <= 16)
        uint16_t                mTouch_toggle;
    #elif (MTOUCH_NUMBER_SENSORS <= 24)
        uint24_t                mTouch_toggle;
    #elif (MTOUCH_NUMBER_SENSORS <= 32)
        uint32_t                mTouch_toggle;  
    #endif
#endif
//@}


//========================================================================
// CONSTANT ARRAYS
//========================================================================
/** @name   Constant Threshold Arrays
*
*   @brief  These arrays are stored in program memory at compile time, so 
*           they do not require RAM. The downside to this implementation 
*           is that their values cannot be updated at runtime. If this is 
*           something your application requires, further editing will be 
*           needed.
*/
//@{
#if defined(MTOUCH_ACTIVE_ENABLED)
#if !defined(MTOUCH_EEPROM_ENABLED)
    // If V2 communications are enabled, the thresholds will be stored in EEPROM. Otherwise we need to define a constant array for them.
    const uint16_t  mTouch_pressThreshold        [MTOUCH_NUMBER_SENSORS] = PRESS_THRESHOLD_INIT;     
    const uint16_t  mTouch_releaseThreshold      [MTOUCH_NUMBER_SENSORS] = RELEASE_THRESHOLD_INIT;   
#endif
#endif

#if defined(MTOUCH_SLEEP_ENABLED)
    const uint16_t  mTouch_sleep_pressThreshold  [MTOUCH_NUMBER_OF_SLEEP_SENSORS] = MTOUCH_SLEEP_PTHRESH_INIT;
    #if !defined(MTOUCH_ACTIVE_ENABLED)
    const uint16_t  mTouch_sleep_releaseThreshold[MTOUCH_NUMBER_OF_SLEEP_SENSORS] = MTOUCH_SLEEP_RTHRESH_INIT;
    #endif
#endif
//@}

//========================================================================
// FUNCTION PROTOTYPES                
//========================================================================
void mTouch_Init                    (void);
void mTouch_UpdateAverage           (uint16_t, uint8_t);

#if defined(MTOUCH_ACTIVE_ENABLED)
void mTouch_Decode                  (void);
#endif

#if defined(MTOUCH_SLEEP_ENABLED)
void mTouch_Sleep_Decode            (void);
    
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
    void mTouch_Sleep_UpdateAverage     (uint16_t);
    #else
    void mTouch_Sleep_UpdateAverage     (uint16_t, uint8_t);
    #endif
#endif

#if defined(MTOUCH_ACTIVE_ENABLED)
#if defined(MTOUCH_SLEEP_ENABLED)
void mTouch_ChangeState             (void);
#endif
#endif

void mTouch_ProximityDecode         (void);


//================================================================================================
//          _____                _         ___       _ _   
//  _ __ __|_   _|__  _   _  ___| |__     |_ _|_ __ (_) |_ 
// | '_ ` _ \| |/ _ \| | | |/ __| '_ \     | || '_ \| | __|
// | | | | | | | (_) | |_| | (__| | | |    | || | | | | |_ 
// |_| |_| |_|_|\___/ \__,_|\___|_| |_|___|___|_| |_|_|\__|
//                                   |_____|   
//================================================================================================
/**
*   @brief  Main initialization routine for all mTouch framework modules including the built-in 
*           communication module, if enabled.
*/
void mTouch_Init(void)
{
    #if defined(MTOUCH_SLEEP_ENABLED)
    MTOUCH_WDT_OFF();
    #endif
    
    #if defined(MTOUCH_EEPROM_ENABLED)
    MTOUCH_EEPROM_INIT();
    #endif
    
    /** @section InitImp Framework Initialization Notes
    * 1. Loop through all enabled, active sensors and initialize their variables.
    */  
    #if defined(MTOUCH_ACTIVE_ENABLED)
        #if (MTOUCH_NUMBER_SENSORS == 1)
            #define sensorIndex 0       // If only one sensor is implemented, don't implement
                                        // the 'for' loop, just replace all 'sensorIndex'
                                        // text with the value '0'.
        #else
        // TIP: By declaring the index variable within the 'for' instead of before it, the
        //      compiler is able to recycle that register in memory immediately after the
        //      loop's execution and use it for something else. Limiting the life-span of 
        //      temporary variables such as this helps limit the total amount of temporary 
        //      variable space that is required.
        for (int8_t sensorIndex = MTOUCH_NUMBER_SENSORS-1; sensorIndex >= 0; sensorIndex--)
        {
        #endif
        
            mTouch_AcquisitionData* acqData     = &mTouch_acqData[sensorIndex];
            
            mTouch_stateVars.active.sensor[sensorIndex].state     = MTOUCH_INITIALIZING;
            mTouch_stateVars.active.sensor[sensorIndex].timer     = MTOUCH_POWER_UP_SAMPLES;
            mTouch_stateVars.active.sensor[sensorIndex].avgCount  = MTOUCH_AVGRATE_VALUE;
            #if defined(MTOUCH_STATE_DEBOUNCE)
            mTouch_stateVars.active.sensor[sensorIndex].debounce  = 0;
            #endif
            
            
            mTouch_SetResult(acqData, 0x47F);
            #if defined(MTOUCH_SLIDERS_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY) 
            mTouch_deltas       [sensorIndex] = 0;
            #endif
            mTouch_average      [sensorIndex] = 0;
            
        #if (MTOUCH_NUMBER_SENSORS == 1)
            #undef sensorIndex          // Turn off the text replacement on 'sensorIndex'
        #else
        }
        #endif
        
        #if defined(MTOUCH_TOGGLE_ENABLED)
        mTouch_toggle = 0;
        #endif
    #endif
    
    /// 2. Loop through all enabled sliders and initialize the slider's output value.
    #if defined(MTOUCH_ACTIVE_ENABLED)
    #if defined(MTOUCH_SLIDERS_ENABLED) 
        #if (MTOUCH_NUMBER_OF_SLIDERS == 1)
        mTouch_slider[0] = 0;
        #elif (MTOUCH_NUMBER_OF_SLIDERS == 2)
        mTouch_slider[0] = 0;
        mTouch_slider[1] = 0;
        #else
        for (int8_t i = MTOUCH_NUMBER_OF_SLIDERS-1; i >= 0; i--)
        {
            mTouch_slider       [i] = 0;
        }
        #endif
    #endif
    #endif
    
    /// 3. Loop through all enabled sleep sensors and initialize their variables.
    #if defined(MTOUCH_SLEEP_ENABLED)
        #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
            #define sensorIndex 0
        #else
        for (int8_t sensorIndex = MTOUCH_NUMBER_OF_SLEEP_SENSORS-1; sensorIndex >= 0; sensorIndex--)
        {
        #endif
            mTouch_AcquisitionData*     acqData     = &mTouch_sleep_acqData[sensorIndex];
            
            mTouch_SetResult(acqData, 0x47F);
            mTouch_sleep_average[sensorIndex] = 0;
        #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
            #undef sensorIndex
        #else
        }
        #endif
    #endif
    
    /// 4. Initialize non-array system variables
    mTouch_state.dataReady      = 0;
    mTouch_state.areInitialized = 0;  
    
    #if defined(MTOUCH_ACTIVE_ENABLED)
    mTouch_stateVars.active.sampleCounter   = MTOUCH_SpS_VALUE;
        #if defined(MTOUCH_SLEEP_ENABLED)
        mTouch_state.sleep                  = 0;
        mTouch_state.sleepInitialized       = 0;
        mTouch_state.sleepLockout           = 0;
        #endif
        #if defined(MTOUCH_SLEEP_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
        mTouch_state.allReleased            = 0;
        #endif
    #endif
        
    #if defined(MTOUCH_ACTIVE_RUNS_AS_MAINLOOP) || defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
    mTouch_state.isrServiced    = 0;
    mTouch_state.isRepeatScan   = 0;
    #endif
    
    /// 5. Initialize the DACOUT pin when it is used as a guard ring.
    #if defined(CVD_GUARD_DACOUT_ENABLED)
    CVD_DACOUT_LAT_C    = 0;
    CVD_DACOUT_TRIS_C   = 0;    
    #endif
    
    /// 6. Initialize the ADC module for mTouch operation.
    MTOUCH_ADC_INIT();
    
    /// 7. Initialize the framework's communication module, if enabled.
    #if defined(MTOUCH_V1_COMMS) || defined(MTOUCH_V2_COMMS)
    mTouch_Comm_Init();        
    #endif
    
    /// 6. Initialization complete - enable active-mode mTouch scanning.
    mTouch_EnableScanning();
    /// <br><br>

}


//================================================================================================
//          _____                _         ____                     _      
//  _ __ __|_   _|__  _   _  ___| |__     |  _ \  ___  ___ ___   __| | ___ 
// | '_ ` _ \| |/ _ \| | | |/ __| '_ \    | | | |/ _ \/ __/ _ \ / _` |/ _ \
// | | | | | | | (_) | |_| | (__| | | |   | |_| |  __/ (_| (_) | (_| |  __/
// |_| |_| |_|_|\___/ \__,_|\___|_| |_|___|____/ \___|\___\___/ \__,_|\___|
//                                   |_____|  
//================================================================================================
/**
*   @ingroup Decoding
* 
*   @brief Updates the state of each active-mode sensor using the latest acquisition value
*   
*   This is the mTouch service routine that performs sensor state transitions based on the value
*   of the latest acquisition value as compared to its previous behavior.
*
*/
#if defined(MTOUCH_ACTIVE_ENABLED)
void mTouch_Decode(void)
{

    int16_t delta;
    
    // Flags used to check the state of all sensors at once.
    mTouch_state.areInitialized = 1;
    #if defined(MTOUCH_SLEEP_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
    mTouch_state.allReleased    = 1;
    #endif
    #if defined(MTOUCH_V2_COMMS)
    mTouch_state.buttonStateChange = 0;
    #endif
    
    /** @section DecodeImp Algorithm Implementation Notes
    * 
    *   1. Disable active-mode mTouch scanning.
    */
    mTouch_DisableScanning();
    
    /** 
    *   2. For each active-mode sensor, perform the following:
    */
    #if MTOUCH_NUMBER_SENSORS == 1)
    #define sensorIndex 0               // If only one sensor is implemented, don't implement
                                        // the 'for' loop, just replace all 'sensorIndex'
                                        // text with the value '0'.
    #else
    for(int8_t sensorIndex = MTOUCH_NUMBER_SENSORS-1; sensorIndex >= 0; sensorIndex--)
    {
    #endif
        /** 
        * @li   Store a pointer to this sensor's variable location within the stateVars 
        *       structure.
        */
        mTouch_SensorVariables* sensor  = &mTouch_stateVars.active.sensor[sensorIndex];
        
        /** 
        * @li   Get the current shift amount by comparing the new reading with the average
        *       and store it in the variable 'delta'. If a negative shift is detected, clear
        *       the delta variable. Then, if required by the features that have been enabled,
        *       store the delta value in the mTouch_deltas array for later use.
        */
        delta               = mTouch_GetSensor(sensorIndex) - mTouch_GetAverage(sensorIndex); 

        if (delta < 0)  // Shifts should increase. Setting to 0 helps to prevent triggering
        {               // on negative spikes.
            delta = 0;  
        }

        #if defined(MTOUCH_SLIDERS_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
        mTouch_deltas[sensorIndex] = (uint16_t)delta;
        #endif
        
        /**
        * @li   Use a switch statement to implement a different behavior based on the current
        *       state of each sensor.
        */
        switch((*sensor).state)                                 // Begin state machine
        {
            /** @subsection InitState Initialization State
            *
            *   The default state of all sensors on power up. Once sensors leave this state, 
            *   they do not return. Initialization begins by decrementing the timer variable
            *   for the sensor. 
            *   If #timer has not yet reached 0, the areInitialized and allReleased flags are
            *   cleared to prevent any advanced decoding logic, such as sliders and wheels, 
            *   or communications from occuring. 
            *   Once #timer reaches 0, the sensor transitions to the #MTOUCH_RELEASED state, 
            *   the debounce counter is reset to the value needed to enter the pressed state,
            *   and the average is updated to equal to the current sensor value.
            */
            case MTOUCH_INITIALIZING:    
            {
                if (--((*sensor).timer) <= 0)                       // If initialization delay has elapsed
                {
                    // State Transition -> Released
                    (*sensor).state             = MTOUCH_RELEASED;  // Sensor is now released and stable  
                    
                    mTouch_average[sensorIndex] = mTouch_GetSensor(sensorIndex);  
                    
                    #if defined(MTOUCH_STATE_DEBOUNCE)              // Initialize sensor's debounce counter
                    (*sensor).debounce          = MTOUCH_pDEBOUNCE_VALUE;   
                    #endif
                    
                }
                else                                                // If initialization delay is not complete
                {
                    mTouch_state.areInitialized  = 0;               // Disable data transfer until initialized
                    
                    #if defined(MTOUCH_SLEEP_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
                    mTouch_state.allReleased     = 0;               // Clear the all-sensors-are-released flag.
                    #endif
                    
                    
                }
                break;                                          
            }                                                   
            
            /** @subsection ReleaseState Released State
            * 
            *   This state can be entered from either the #MTOUCH_INITIALIZING or 
            *   #MTOUCH_PRESSED state. The only way to leave this state is by creating enough 
            *   consecutive positive shifts of the reading above the average to overflow the 
            *   #debounce variable for the sensor. Once this is done, the sensor will enter 
            *   the pressed state.<br>
            *   The code for this section begins by checking to see if the latest reading
            *   is greater than the average by at least the press threshold. If it is, the
            *   debounce variable is decremented. Once the debounce value has been reached 0, 
            *   the sensor will change to the #MTOUCH_PRESSED state, reset the debounce 
            *   counter, initialize the press timer (if enabled), toggle the toggle bit,
            *   and clear the 'allReleased' flag.<br>
            *   If the threshold has not been crossed, the debounce counter is reset to the
            *   initial press debounce value and the average is updated.
            *
            *   @note This is the only state and condition where the average is able to be 
            *   updated, and we only use the latest reading to update the average once we 
            *   know it's not crossing any thresholds.
            */
            case MTOUCH_RELEASED:                                           // Sensor is released and stable
            {                  

                if(delta > (int16_t)mTouch_GetPressThreshold(sensorIndex))  // If reading has crossed the press threshold
                {
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    if ((*sensor).debounce == 0)                            // Check the debounce counter
                    {                                                       
                        (*sensor).debounce  = MTOUCH_rDEBOUNCE_VALUE;       // Initialize the pressed state's debounce variable
                    #endif
                    
                        // State Transition -> Pressed
                        (*sensor).state     = MTOUCH_PRESSED;               // Sensor is now pressed
                        
                        #if defined(MTOUCH_BUTTON_TIMEOUT)                  // If the press timer is enabled
                        #if MTOUCH_BUTTON_TIMEOUT > 0
                        (*sensor).timer    = MTOUCH_pTIMEOUT_VALUE;         // Initialize the press timer counter
                        #endif
                        #endif
                        
                        #if defined(MTOUCH_SLEEP_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
                        mTouch_state.allReleased = 0;                       // Clear the all-sensors-are-released flag.
                        #endif
                        
                        #if defined(MTOUCH_TOGGLE_ENABLED)
                        mTouch_ToggleToggle(sensorIndex);                   // Alternate toggle between 'on' and 'off'.
                        #endif
                        
                        #if defined(MTOUCH_V2_COMMS)
                        mTouch_state.buttonStateChange = 1;
                        #endif
                        
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    }
                    else
                    {
                        (*sensor).debounce--;
                    }
                    #endif
                }
                else                                                        // If the reading has not crossed the press threshold
                {
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    (*sensor).debounce      = MTOUCH_pDEBOUNCE_VALUE;       // Reset the debounce variable   
                    #endif
                    
                    // NOTE: This is the only state (MTOUCH_RELEASED) and condition 
                    //       (delta < PressThreshold) where the average is able to
                    //       be updated. Also note how we only use the reading to
                    //       update the average after we have verified it is not
                    //       crossing any thresholds.
                    
                    mTouch_UpdateAverage(mTouch_GetSensor(sensorIndex), sensorIndex);  
                }
                break;
                
            }

            /** @subsection PressState Pressed State
            * 
            *   A sensor can only enter this state from the #MTOUCH_RELEASED state, but there
            *   are two ways to leave: 
            *       First, by being in the state for too long and allowing the press timer to 
            *       run out. 
            *       Second, by having a the delta value smaller than the release threshold on 
            *       enough consecutive scans to cause the debounce counter to reach 0.
            *
            *   The code for this section begins by decrementing and checking the #timer 
            *   variable to see if it has reached 0. If so, the sensor transfers to the 
            *   #MTOUCH_RELEASED state, the debounce counter is reset and the average is 
            *   re-initialized to the sensor's current reading. The average is reset to allow 
            *   for a quick recovery if some environmental condition caused the sensor to 
            *   become stuck. See the #NEGATIVE_CAPACITANCE configuration option for the
            *   different behaviors that can be implemented to quickly allow the average to
            *   re-settle to the unpressed value.
            *
            *   If the timer has not yet reached 0, the reading is checked against the 
            *   release threshold. If the difference between the sensor's reading and the
            *   average (delta) is less than the release threshold, the debounce variable is
            *   decremented in the same manner as the press timer. When it reaches 0, the
            *   sensor enters the #MTOUCH_RELEASED state and the debounce counter is set to
            *   debounce value required to enter the pressed state. If it has not crossed the 
            *   threshold (meaning: the delta value is still large enough to register as a
            *   press), the debounce counter is reset to the release debounce value.
            *
            *   @note The average is never updated while the sensor is pressed so the system
            *   can remember where the unpressed value should end up. Because of this, the
            *   average can be better thought of as a released-state average.
            */
            case MTOUCH_PRESSED:
            {

                #if defined(MTOUCH_BUTTON_TIMEOUT)                  // (Only if the press timer has been enabled)
                #if MTOUCH_BUTTON_TIMEOUT > 0   

                if (--((*sensor).timer) <= 0)                       // Decrement the press timer counter
                {     
                    // State Transition -> Released
                    (*sensor).state     = MTOUCH_RELEASED;          // Sensor is now released
                    
                    mTouch_average[sensorIndex]   = mTouch_GetSensor(sensorIndex);
                    
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    (*sensor).debounce = MTOUCH_pDEBOUNCE_VALUE;    // Initialize the debounce counter for the MTOUCH_PRESSED state
                    #endif                                          // and check if it has reached 0.
                    
                    #if defined(MTOUCH_V2_COMMS)
                    mTouch_state.buttonStateChange = 1;
                    #endif
                    
                    break;
                }
                
                #endif
                #endif
                
                if(delta < (int16_t)mTouch_GetReleaseThreshold(sensorIndex))      
                {                                                   // If the reading has crossed the release threshold
                
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    if ((*sensor).debounce == 0)                    // Check if the debounce counter has reached 0.
                    {                                               
                        (*sensor).debounce = MTOUCH_pDEBOUNCE_VALUE;
                    #endif
                    
                        (*sensor).state = MTOUCH_RELEASED;          // Sensor is now released
                        
                        #if defined(MTOUCH_V2_COMMS)
                        mTouch_state.buttonStateChange = 1;
                        #endif
                        
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    } else {
                    
                        (*sensor).debounce--;
                    
                        #if defined(MTOUCH_SLEEP_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
                        mTouch_state.allReleased = 0;               // Clear the all-sensors-are-released flag.
                        #endif
                        
                    }
                    #endif                    
                }
                else                                                // If the reading has not crossed the release threshold
                {
                
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    (*sensor).debounce  = MTOUCH_rDEBOUNCE_VALUE;   // Reset the debounce counter
                    #endif                    
                    #if defined(MTOUCH_SLEEP_ENABLED) || defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
                    mTouch_state.allReleased = 0;                   // Clear the all-sensors-are-released flag.
                    #endif
                }
                break;
            }
            
            default: break; 
            
        }   // End state machine switch command
        
        
    #if MTOUCH_NUMBER_SENSORS == 1)
    #undef sensorIndex              // Turn off the text replacement on 'sensorIndex'
    #else
    }       // End for loop
    #endif
    
    /**
    * 3. Now that all sensors have had their initial decoding step completed, if sliders 
    *    are implemented in this application, perform decoding on them now.
    */
    #if defined(MTOUCH_SLIDERS_ENABLED)  
    mTouch_SliderDecode();
    #endif
    
    /**
    * 4. If only one sensor is allowed to be pressed at a time due to the 'most pressed'
    *    algorithm being enabled, then perform the logic to check which sensor is showing
    *    the largest shift. 
    *    Once the largest shift has been found, all other sensors are set to the released
    *    state.
    */
    #if defined(MTOUCH_ACTIVE_MOST_PRESSED_ONLY)
    if (!mTouch_state.allReleased && mTouch_state.areInitialized)
    {
        
        #if MTOUCH_NUMBER_SENSORS > 3
        
            // If there are more than 3 sensors, we'll use a 'for' loop to check the states
            // and reset the sensors.
        
            uint16_t    maxValue = 0;
            int8_t      maxIndex = 0;
            
            // TIP: 'for' loops that count down are more efficient than loops that count up 
            //      since the processor is able to use the status register's 'zero' flag or 
            //      the polarity bit of the index to determine when to stop looping.
            
            for (int8_t i = MTOUCH_NUMBER_SENSORS-1; i >= 0; i--)
            {                                                           // Find the maximum delta value and store its sensor index
                if (mTouch_deltas[i] > maxValue)
                {
                    maxValue = mTouch_deltas[i];    // Update maxValue
                    maxIndex = i;                   // Update maxIndex
                }
            }
            for (int8_t i = MTOUCH_NUMBER_SENSORS-1; i >= 0; i--)
            {
                if (i != maxIndex)                                      // Un-press all sensors except the most pressed.
                {
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    mTouch_stateVars.active.sensor[i].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    #endif
                    mTouch_stateVars.active.sensor[i].state     = MTOUCH_RELEASED;
                }
            }
            
            
        #elif MTOUCH_NUMBER_SENSORS == 2
            
            // If we are only comparing two sensors, let's simplify the implementation:
            
            if (mTouch_deltas[0] > mTouch_deltas[1])                    // Sensor 0 is most pressed.
            {
                #if defined(MTOUCH_STATE_DEBOUNCE)
                mTouch_stateVars.active.sensor[1].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                #endif
                mTouch_stateVars.active.sensor[1].state     = MTOUCH_RELEASED;          
            } else {                                                    // Sensor 1 is most pressed.
                #if defined(MTOUCH_STATE_DEBOUNCE)
                mTouch_stateVars.active.sensor[0].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                #endif
                mTouch_stateVars.active.sensor[0].state     = MTOUCH_RELEASED;          
            }        
            
        #elif MTOUCH_NUMBER_SENSORS == 3
        
            // If we are only comparing three sensors, let's simplify the implementation:
        
            if (mTouch_deltas[0] > mTouch_deltas[1])
            {
                if (mTouch_deltas[0] > mTouch_deltas[2])                // Sensor 0 is most pressed.
                {
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    mTouch_stateVars.active.sensor[1].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    mTouch_stateVars.active.sensor[2].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    #endif
                    mTouch_stateVars.active.sensor[1].state     = MTOUCH_RELEASED;
                    mTouch_stateVars.active.sensor[2].state     = MTOUCH_RELEASED;
                } else {   
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    mTouch_stateVars.active.sensor[0].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    mTouch_stateVars.active.sensor[1].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    #endif                                              // Sensor 2 is most pressed.
                    mTouch_stateVars.active.sensor[0].state     = MTOUCH_RELEASED;
                    mTouch_stateVars.active.sensor[1].state     = MTOUCH_RELEASED;
                }   
            } 
            else 
            {
                if (mTouch_deltas[1] > mTouch_deltas[2])                // Sensor 1 is most pressed. 
                {
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    mTouch_stateVars.active.sensor[0].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    mTouch_stateVars.active.sensor[2].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    #endif
                    mTouch_stateVars.active.sensor[0].state     = MTOUCH_RELEASED;
                    mTouch_stateVars.active.sensor[2].state     = MTOUCH_RELEASED;
                } else {                                                // Sensor 2 is most pressed.
                    #if defined(MTOUCH_STATE_DEBOUNCE)
                    mTouch_stateVars.active.sensor[0].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    mTouch_stateVars.active.sensor[1].debounce  = MTOUCH_pDEBOUNCE_VALUE;
                    #endif
                    mTouch_stateVars.active.sensor[0].state     = MTOUCH_RELEASED;
                    mTouch_stateVars.active.sensor[1].state     = MTOUCH_RELEASED;
                }   
            }   
            
        #endif
        
    }  
    #endif
    
    /**
    * 5. Now that all state machine and more advanced decoding is complete,
    *    check to see if the system should enter the sleep state. The conditions
    *    where this happens are:
    *    First, if the sleep sensors have never been initialized (in other words,
    *    the system has just been powered on) and all active sensors are now
    *    initialized.
    *    Second, if the active sensors are all released for 
    *    #MTOUCH_SLEEP_INACTIVE_RESET consecutive decodes in a row.
    */
    #if defined(MTOUCH_SLEEP_ENABLED)
    if (!mTouch_state.sleepInitialized && mTouch_state.areInitialized)  // If sleep mode has not yet been initialized,
    {                                                                   // and we've finished initializing the active
        mTouch_EnterSleepMode();                                        // sensors, go into sleep mode.
    }
    
    if (mTouch_state.allReleased)                   // If all active sensors are currently released,
    {
        if (--mTouch_stateVars.active.inactivity == 0)  // Decrement the inactivity timer. If it equals 0,
        {
            mTouch_EnterSleepMode();                // Go into sleep mode immediately.
        }
    } 
    else                                            // If any of the active sensors are still initializing
    {                                               // or are pressed, reset the inactivity timer.
        mTouch_stateVars.active.inactivity = MTOUCH_SLEEP_INACTIVE_VALUE;
    }
    #endif
    
    /**
    * 6. Finally, if all active sensors are initialized and the system is not
    *    about to enter the sleep state, send communications data.
    */
    #if defined(MTOUCH_V1_COMMS) || defined(MTOUCH_V2_COMMS)    // (Only if debugging communications are enabled)
    if (mTouch_state.areInitialized)                            // Only trasmit data if all sensors are initialized
    {
    #if defined(MTOUCH_SLEEP_ENABLED)
        if (mTouch_state.change == 0)               // If the system is not going to be entering sleep mode right now,
        {
    #endif
    
            mTouch_Comm_SendData();                 // Transmit data packet using mTouch Communication module
            
    #if defined(MTOUCH_SLEEP_ENABLED)
        }
    #endif
    }
    #endif
        
    /**
    * 7. Enable active-mode mTouch scanning if the system is not entering sleep mode.
    */
    #if defined(MTOUCH_SLEEP_ENABLED)
    if (mTouch_state.change == 0)               // If the system is not entering sleep mode,
    {
    #endif
    
        mTouch_EnableScanning();                // Enable the timer's interrupt
        
    #if defined(MTOUCH_SLEEP_ENABLED)
    }
    #endif
    
    /// <br><br><br>
} 
#endif

//================================================================================================
//          _____                _        _   _           _       _          _                                 
//  _ __ __|_   _|__  _   _  ___| |__    | | | |_ __   __| | __ _| |_ ___   / \__   _____ _ __ __ _  __ _  ___ 
// | '_ ` _ \| |/ _ \| | | |/ __| '_ \   | | | | '_ \ / _` |/ _` | __/ _ \ / _ \ \ / / _ \ '__/ _` |/ _` |/ _ \
// | | | | | | | (_) | |_| | (__| | | |  | |_| | |_) | (_| | (_| | ||  __// ___ \ V /  __/ | | (_| | (_| |  __/
// |_| |_| |_|_|\___/ \__,_|\___|_| |_|___\___/| .__/ \__,_|\__,_|\__\___/_/   \_\_/ \___|_|  \__,_|\__, |\___|
//                                   |_____|   |_|                                                  |___/      
//================================================================================================

/**
*   @ingroup    Decoding
*
*   @param[in]  reading     The current reading value of the sensor
*   @param[in]  index       The index of the sensor to update
*
*   @brief      Updates the average for the given sensor with the given value.
*   
*   Updates a sensor's average (or baseline) in order to track the environmental changes of the 
*   system. 
*/
void mTouch_UpdateAverage(uint16_t reading, uint8_t index)
{
    /** @section UAvgImp Algorithm Implementation Notes
    *
    *   This function has several possible implementations based on the current configuration 
    *   options of the framework. The important configuration options are:
    *   @li #MTOUCH_NEGATIVE_CAPACITANCE - determines how the average handles a negative 
    *       reading shift. This is most common after the press timer has caused a reset on the
    *       average. For example, if someone presses on a sensor, the reading will shift high.
    *       After the press timer activates, the average will be reset to the high reading
    *       value. Once the user releases, this configuration option will determine how the
    *       average updates itself to return to the unpressed sensor value.
    *   @li #MTOUCH_AVG_RATE - acts as a debounce counter for updating the average only every 
    *       N scans.
    *   @li #MTOUCH_AVG_UPDATE - determines what weight to give the reading when updating the 
    *       average.
    *
    *   The implementation begins by checking if #MTOUCH_NEGATIVE_CAPACITANCE is set to 1.
    *   If so, the average counter variable is set so that the average will perform its
    *   update regardless of the previous count. 
    *   If #MTOUCH_NEGATIVE_CAPACITANCE is instead equal to 2, the average is immediately 
    *   updated to follow the reading and the function is exited.
    *
    *   Next, the average updates the counter variable and compares it with #MTOUCH_AVG_RATE
    *   to see if it should continue or return. If the counter allows it to continue, the 
    *   average is then updated using one of four possible implementations which is 
    *   determined by the #MTOUCH_AVG_UPDATE option.
    */
    #if defined(MTOUCH_ACTIVE_ENABLED)
    
        #if defined(MTOUCH_EEPROM_ENABLED)
        if (MTOUCH_NEGCAP_VALUE == 1)       // If 'V2' communications are enabled, this configuration
        {                                   // option is changable. So we need to check it each time.
        #endif
    
            #if defined(MTOUCH_EEPROM_ENABLED) || (MTOUCH_NEGATIVE_CAPACITANCE == 1)
            if ((int16_t)(reading - mTouch_average[index]) < 0)     // If there was a negative shift
            {                                                       
                mTouch_stateVars.active.sensor[index].avgCount = 1;    
            }                                                       // Set the average counter to allow 
            #endif                                                  // an immediate average update.
           
        #if defined(MTOUCH_EEPROM_ENABLED)
        }
        else if (MTOUCH_NEGCAP_VALUE == 2)
        {
        #endif
            
            #if defined(MTOUCH_EEPROM_ENABLED) || (MTOUCH_NEGATIVE_CAPACITANCE == 2)
            if ((int16_t)(reading - mTouch_average[index]) < 0)     // If there was a negative shift,
            {                                                       // re-initialize the average to 
                mTouch_average[index] = reading;                    // the current reading value and
                return;                                             // return from the function - not 
            }                                                       // performing the update algorithm.
            #endif

        #if defined(MTOUCH_EEPROM_ENABLED)
        }
        #endif
        
        #if defined(MTOUCH_AVGCOUNT_16BITS_REQUIRED)                // If the average counter has been exceeded
        if ((uint16_t)(--(mTouch_stateVars.active.sensor[index].avgCount)) == 0)    
        #else
        if ((uint8_t)(--(mTouch_stateVars.active.sensor[index].avgCount)) == 0) 
        #endif
        {                                                           // Reset the average counter
            mTouch_stateVars.active.sensor[index].avgCount = MTOUCH_AVGRATE_VALUE;  

            #if defined(MTOUCH_EEPROM_ENABLED)  
            if (MTOUCH_AVGUPDATE_VALUE == 1)    // If 'V2' communications are enabled, this configuration
            {                                   // option is changable. So we need to check it each time.
            #endif
            
                #if defined(MTOUCH_EEPROM_ENABLED) || (MTOUCH_AVG_UPDATE == 1)
                    mTouch_average[index]  += reading;              // Update the average with an equal
                    mTouch_average[index] >>= 1;                    // weight given to both.
                #endif                                              
            
            #if defined(MTOUCH_EEPROM_ENABLED)
            }
            else
            {
            #endif

                #if defined(MTOUCH_EEPROM_ENABLED) || (MTOUCH_AVG_UPDATE != 1) 
                    mTouch_average[index]  -= (uint16_t)(mTouch_average[index] >> MTOUCH_AVGUPDATE_VALUE); 
                    mTouch_average[index]  += (uint16_t)(reading >> MTOUCH_AVGUPDATE_VALUE);
                #endif

            #if defined(MTOUCH_EEPROM_ENABLED)
            }
            #endif
            
        }
        #endif  // End: #if defined(MTOUCH_ACTIVE_ENABLED)
        /// <br><br><br>    
}

#if defined(MTOUCH_SLEEP_ENABLED)
//================================================================================================
//          _____                _         ____  _                    ____                     _      
//  _ __ __|_   _|__  _   _  ___| |__     / ___|| | ___  ___ _ __    |  _ \  ___  ___ ___   __| | ___ 
// | '_ ` _ \| |/ _ \| | | |/ __| '_ \    \___ \| |/ _ \/ _ \ '_ \   | | | |/ _ \/ __/ _ \ / _` |/ _ \
// | | | | | | | (_) | |_| | (__| | | |    ___) | |  __/  __/ |_) |  | |_| |  __/ (_| (_) | (_| |  __/
// |_| |_| |_|_|\___/ \__,_|\___|_| |_|___|____/|_|\___|\___| .__/___|____/ \___|\___\___/ \__,_|\___|
//                                   |_____|                |_| |_____|                               
//================================================================================================
/**
*   @brief Updates the state of each sleep-mode sensor using the latest acquisition value
*   
*   This is the mTouch service routine that performs sensor state transitions based on the value
*   of the latest acquisition value as compared to its previous behavior.
*/
void mTouch_Sleep_Decode(void)
{

    mTouch_state.areInitialized = 1;
    
    /** @section SleepDecodeImp Algorithm Implementation Notes
    * 
    *   1. For each active-mode sensor, perform the following:
    */
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
    #define sensorIndex 0               // If only one sensor is implemented, don't implement
                                        // the 'for' loop, just replace all 'sensorIndex'
                                        // text with the value '0'.
    #else    
    for (uint8_t sensorIndex = 0; sensorIndex < MTOUCH_NUMBER_OF_SLEEP_SENSORS; sensorIndex++)
    {
    #endif
    
        /** 
        * @li   Store a pointer to this sensor's variable location within the stateVars 
        *       structure.
        */
        int16_t delta;
        mTouch_Sleep_SensorVariables*   sensor      = &mTouch_stateVars.sleep.sensor[sensorIndex];
        mTouch_AcquisitionData*         acqData     = &mTouch_sleep_acqData[sensorIndex];
        
        #if defined(MTOUCH_V2_COMMS)
        mTouch_state.buttonStateChange = 0;
        #endif
        /** 
        * @li   Get the current shift amount by comparing the new reading with the average
        *       and store it in the variable 'delta'. If a negative shift is detected, replace
        *       it with the absolute value. 
        */
        delta = mTouch_Sleep_GetSensor() - mTouch_Sleep_GetAverage(sensorIndex);
        if (delta < 0)
        {
            delta = -delta;
        }
        
        
        /**
        * @li   Use a switch statement to implement a different behavior based on the current
        *       state of each sensor.
        */
        switch((*sensor).state)                                         // Begin state machine.
        {
            /** @subsection SleepInitState (Sleep) Initialization State
            *
            *   The default state of all sleep-sensors on power up and on re-entry to the
            *   sleep state. Once sensors leave this state, they do not return until the next
            *   state transition. Initialization begins by decrementing the timer variable
            *   for the sensor. 
            *   If #timer has not yet reached 0, the areInitialized flag is cleared to prevent 
            *   communications from occuring. 
            *   Once #timer reaches 0, the sensor transitions to the #MTOUCH_SLEEP_RELEASED 
            *   state, and the average is updated to equal to the current sensor value.
            */
            case MTOUCH_SLEEP_INITIALIZING:
            
                mTouch_state.areInitialized = 0;
                if (--(*sensor).timer == 0)
                {
                    mTouch_sleep_average[sensorIndex] = mTouch_Sleep_GetSensor();
                    
                    (*sensor).state = MTOUCH_SLEEP_RELEASED;
                }  
                break;
                
            /** @subsection SleepReleaseState (Sleep) Released State
            *
            *   This state can be entered from either the #MTOUCH_SLEEP_INITIALIZING or 
            *   #MTOUCH_SLEEP_PRESSED state.
            *
            *   If active sensors are also enabled in the framework, any sleep sensor
            *   that crosses its threshold will cause the system to enter the active
            *   state and reset all sleep sensors to the initializing state.
            *
            *   If only sleep sensors are enabled, a sleep sensor crossing its threshold
            *   will only cause that sensor to enter the #MTOUCH_SLEEP_PRESSED state. All
            *   other sensors will act independently.
            *
            *   @note This is the only state and condition where the average is able to be 
            *   updated, and we only use the latest reading to update the average once we 
            *   know it's not crossing any thresholds.
            */
            case MTOUCH_SLEEP_RELEASED:
            
                if ((uint16_t)(delta) > mTouch_Sleep_GetPressThreshold(sensorIndex))
                {
                    #if defined(MTOUCH_ACTIVE_ENABLED) && defined(MTOUCH_SLEEP_INACTIVE_RESET)
                    
                        mTouch_ExitSleepMode();         // Set flag to exit sleep mode, enter active mode.
                        
                    #else
                    
                        (*sensor).state = MTOUCH_SLEEP_PRESSED;
                        (*sensor).timer = MTOUCH_SLEEP_pTIMEOUT_VALUE;
                        
                        #if defined(MTOUCH_V2_COMMS)
                        mTouch_state.buttonStateChange = 1;
                        #endif
                        
                    #endif
                }
                else
                {
                    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
                    mTouch_Sleep_UpdateAverage(mTouch_Sleep_GetSensor());
                    #else
                    mTouch_Sleep_UpdateAverage(mTouch_Sleep_GetSensor(), sensorIndex);
                    #endif
                }
                
                break;
                
            /** @subsection SleepReleaseState (Sleep) Pressed State
            *
            *   This state can only be entered from the #MTOUCH_SLEEP_RELEASED state and
            *   is only implemented if no active sensors are enabled for the framework.
            *
            *   If only sleep sensors are enabled, the pressed state will implement a press
            *   timer to limit the number of consecutive decodes in which the sensor can
            *   remain in the pressed state.
            */
            #if !defined(MTOUCH_ACTIVE_ENABLED)
            case MTOUCH_SLEEP_PRESSED:
            
                if (--((*sensor).timer) == 0)
                {
                    (*sensor).state = MTOUCH_SLEEP_RELEASED;
                    
                    mTouch_sleep_average[sensorIndex] = mTouch_Sleep_GetSensor();
                    
                    #if defined(MTOUCH_V2_COMMS)
                    mTouch_state.buttonStateChange = 1;
                    #endif
                    break;
                }
                
                if ((uint16_t)(delta) < mTouch_Sleep_GetReleaseThreshold(sensorIndex))
                {
                    (*sensor).state = MTOUCH_SLEEP_RELEASED;
                    
                    #if defined(MTOUCH_V2_COMMS)
                    mTouch_state.buttonStateChange = 1;
                    #endif
                }
                break;
            #endif
        }

    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
    #undef sensorIndex                                  // Turn off the text replacement on 'sensorIndex'
    #else  
    }
    #endif 
    
    
    
    /**
    *   2. Now that the sensor decoding is complete, if all of the sensors are initialized
    *      and we are not changing to active mode, send communications data.
    */
    if (mTouch_state.areInitialized)                    // Only trasmit data if all sensors are initialized
    {
        mTouch_state.sleepInitialized = 1;
        
        #if defined(MTOUCH_V1_COMMS) || defined(MTOUCH_V2_COMMS)    // (Only if debugging communications are enabled)
        #if defined(MTOUCH_ACTIVE_ENABLED)
        if (mTouch_state.change == 0)
        {
        #endif
        
            mTouch_Comm_SendData();                     // Transmit data packet using mTouch Communication module
        
        #if defined(MTOUCH_ACTIVE_ENABLED)
        }
        #endif
        #endif
    }
}

//================================================================================================
//          _____                _         ____  _                   _   _           _       _          _                                 
//  _ __ __|_   _|__  _   _  ___| |__     / ___|| | ___  ___ _ __   | | | |_ __   __| | __ _| |_ ___   / \__   _____ _ __ __ _  __ _  ___ 
// | '_ ` _ \| |/ _ \| | | |/ __| '_ \    \___ \| |/ _ \/ _ \ '_ \  | | | | '_ \ / _` |/ _` | __/ _ \ / _ \ \ / / _ \ '__/ _` |/ _` |/ _ \
// | | | | | | | (_) | |_| | (__| | | |    ___) | |  __/  __/ |_) | | |_| | |_) | (_| | (_| | ||  __// ___ \ V /  __/ | | (_| | (_| |  __/
// |_| |_| |_|_|\___/ \__,_|\___|_| |_|___|____/|_|\___|\___| .__/___\___/| .__/ \__,_|\__,_|\__\___/_/   \_\_/ \___|_|  \__,_|\__, |\___|
//                                   |_____|                |_| |_____|   |_|                                                  |___/      
//================================================================================================

/**
*   @param[in]  reading     The current reading value of the sleep-mode sensor
*   @param[in]  sensorIndex The index of the sleep-mode sensor to update
*
*   @brief      Updates the average for the given sleep-mode sensor with the given value.
*/
#if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
void mTouch_Sleep_UpdateAverage(uint16_t reading)
#define sensorIndex 0   // If only one sensor is implemented, make the code more efficient
                        // by putting a constant value in for the index.
#else
void mTouch_Sleep_UpdateAverage(uint16_t reading, uint8_t sensorIndex)
#endif
{
    if ((--mTouch_stateVars.sleep.sensor[sensorIndex].avgCount) == 0)
    {
        mTouch_stateVars.sleep.sensor[sensorIndex].avgCount = MTOUCH_SLEEP_AVGRATE_VALUE;
        
        #if defined(MTOUCH_EEPROM_ENABLED)
            mTouch_sleep_average[sensorIndex]  -= (uint16_t)(mTouch_sleep_average[sensorIndex] >> MTOUCH_SLEEP_AVGUPDATE_VALUE); 
            mTouch_sleep_average[sensorIndex]  += (uint16_t)(reading >> MTOUCH_SLEEP_AVGUPDATE_VALUE);
        #else
            #if MTOUCH_SLEEP_AVG_UPDATE == 1
                mTouch_sleep_average[sensorIndex]  += reading;                             
                mTouch_sleep_average[sensorIndex] >>= 1;
            #elif MTOUCH_SLEEP_AVG_UPDATE == 2
                mTouch_sleep_average[sensorIndex]  -= (uint16_t)(mTouch_sleep_average[sensorIndex] >> 2); 
                mTouch_sleep_average[sensorIndex]  += (uint16_t)(reading >> 2);
            #elif MTOUCH_SLEEP_AVG_UPDATE == 3
                mTouch_sleep_average[sensorIndex]  -= (uint16_t)(mTouch_sleep_average[sensorIndex] >> 3); 
                mTouch_sleep_average[sensorIndex]  += (uint16_t)(reading >> 3);
            #elif MTOUCH_SLEEP_AVG_UPDATE == 4
                mTouch_sleep_average[sensorIndex]  -= (uint16_t)(mTouch_sleep_average[sensorIndex] >> 4); 
                mTouch_sleep_average[sensorIndex]  += (uint16_t)(reading >> 4);
            #endif
        #endif
    }
    return;
}
#if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
#undef sensorIndex      // Turn off the text replacement on 'sensorIndex'
#endif


#endif  // end - #if defined(MTOUCH_SLEEP_ENABLED)

//================================================================================================
//          _____                _         ____ _                            ____  _        _       
//  _ __ __|_   _|__  _   _  ___| |__     / ___| |__   __ _ _ __   __ _  ___/ ___|| |_ __ _| |_ ___ 
// | '_ ` _ \| |/ _ \| | | |/ __| '_ \   | |   | '_ \ / _` | '_ \ / _` |/ _ \___ \| __/ _` | __/ _ \
// | | | | | | | (_) | |_| | (__| | | |  | |___| | | | (_| | | | | (_| |  __/___) | || (_| | ||  __/
// |_| |_| |_|_|\___/ \__,_|\___|_| |_|___\____|_| |_|\__,_|_| |_|\__, |\___|____/ \__\__,_|\__\___|
//                                   |_____|                      |___/              
//================================================================================================
#if defined(MTOUCH_ACTIVE_ENABLED)
#if defined(MTOUCH_SLEEP_ENABLED)
/**
*   @brief      Performs the state conversion between active and sleep mode.
*
*   This routine will initialize the shared variable space for the new mode
*   and enable/disable the mTouch interrupt associated with the active mode
*   scanning - if used.
*
*   This is called automatically by the mTouch framework as part of the 
*   mTouch_Service() API call. Do not call this function directly.
*/
void mTouch_ChangeState(void)
{
    mTouch_state.change = 0;    // Reset the 'change mode' flag
    
    
    if (mTouch_state.sleep)     // If currently in sleep mode, ENTER ACTIVE.
    {
        mTouch_AcquisitionData* data    = &mTouch_acqData[0];                       // Clear sensor 0 accumulator location
        mTouch_ClearAccumulator(data);   
        
        mTouch_state.sleep = 0; // Active mode        
        
        // Initialize State-Volatile Variable Space
        #if (MTOUCH_NUMBER_SENSORS > 1)
        mTouch_stateVars.active.sampleCounter   = MTOUCH_SpS_VALUE;                 // Reset the active-mode sample counter
        #endif
        #if defined(MTOUCH_SLEEP_INACTIVE_RESET)
        mTouch_stateVars.active.inactivity      = MTOUCH_SLEEP_INACTIVE_VALUE;      // Reset the 'inactivity' counter
        #endif
        
        #if (MTOUCH_NUMBER_SENSORS == 1)
            mTouch_stateVars.active.sensor[0].state     = MTOUCH_RELEASED;          // Set all sensors to 'released' state
            mTouch_stateVars.active.sensor[0].avgCount  = MTOUCH_AVGRATE_VALUE;     // Reset the average counters
            #if defined(MTOUCH_STATE_DEBOUNCE)
            mTouch_stateVars.active.sensor[0].debounce  = MTOUCH_pDEBOUNCE_VALUE;   // Reset the debounce counters
            #endif   
        #elif (MTOUCH_NUMBER_SENSORS == 2)
            mTouch_stateVars.active.sensor[0].state     = MTOUCH_RELEASED;
            mTouch_stateVars.active.sensor[0].avgCount  = MTOUCH_AVGRATE_VALUE;
            #if defined(MTOUCH_STATE_DEBOUNCE)
            mTouch_stateVars.active.sensor[0].debounce  = MTOUCH_pDEBOUNCE_VALUE;  
            #endif   
            mTouch_stateVars.active.sensor[1].state     = MTOUCH_RELEASED;
            mTouch_stateVars.active.sensor[1].avgCount  = MTOUCH_AVGRATE_VALUE;
            #if defined(MTOUCH_STATE_DEBOUNCE)
            mTouch_stateVars.active.sensor[1].debounce  = MTOUCH_pDEBOUNCE_VALUE;   
            #endif   
        #else
        for (int8_t i = MTOUCH_NUMBER_SENSORS-1; i >= 0; i--)
        {
            mTouch_stateVars.active.sensor[i].state     = MTOUCH_RELEASED;
            mTouch_stateVars.active.sensor[i].avgCount  = MTOUCH_AVGRATE_VALUE;
            #if defined(MTOUCH_STATE_DEBOUNCE)                                       
            mTouch_stateVars.active.sensor[i].debounce  = MTOUCH_pDEBOUNCE_VALUE;   
            #endif                                                                  
        }
        #endif
        
        #if defined(MTOUCH_V2_COMMS)    // If 'V2' communications are enabled,
        mTouch_Comm_SendMode();         // send an update to the PC about the mode change.
        #endif
        
        
        #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR) || (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)
        MTOUCH_ISR_TMRxIE = 1;          // Enable active mode scanning
        #endif
    }
    else    // If currently in active mode, ENTER SLEEP.
    {
        #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR) || (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)
        MTOUCH_ISR_TMRxIE = 0;          // Disable active mode scanning
        #endif
        
        if (mTouch_state.sleepLockout)  // If the application is requesting the framework not enter sleep mode yet,
        {                               // reset the inactivity timer to activate next decode, re-enable interrupts
                                        // if needed, and exit before changing states.
            mTouch_stateVars.active.inactivity      = 1;
        
            #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR) || (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)
            MTOUCH_ISR_TMRxIE = 1;      // Enable active mode scanning
            #endif
            
            return;
        }
        
        mTouch_state.sleep = 1;         // Sleep mode        
        
        // Initialize State-Volatile Variable Space
        #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
            mTouch_stateVars.sleep.sensor[0].state      = MTOUCH_SLEEP_INITIALIZING;    // Sleep sensors are reset to the initializing state
            mTouch_stateVars.sleep.sensor[0].avgCount   = MTOUCH_SLEEP_AVGRATE_VALUE;   
            if (mTouch_state.sleepInitialized)  {   mTouch_stateVars.sleep.sensor[0].timer  = MTOUCH_SLEEP_REINIT_VALUE; } 
            else                                {   mTouch_stateVars.sleep.sensor[0].timer  = MTOUCH_SLEEP_INIT_VALUE;   }
        #elif (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 2)
            mTouch_stateVars.sleep.sensor[0].state      = MTOUCH_SLEEP_INITIALIZING;
            mTouch_stateVars.sleep.sensor[1].state      = MTOUCH_SLEEP_INITIALIZING;
            mTouch_stateVars.sleep.sensor[0].avgCount   = MTOUCH_SLEEP_AVGRATE_VALUE;
            mTouch_stateVars.sleep.sensor[1].avgCount   = MTOUCH_SLEEP_AVGRATE_VALUE;
            if (mTouch_state.sleepInitialized)  {   mTouch_stateVars.sleep.sensor[0].timer  = MTOUCH_SLEEP_REINIT_VALUE;
                                                    mTouch_stateVars.sleep.sensor[1].timer  = MTOUCH_SLEEP_REINIT_VALUE; } 
            else                                {   mTouch_stateVars.sleep.sensor[0].timer  = MTOUCH_SLEEP_INIT_VALUE;   
                                                    mTouch_stateVars.sleep.sensor[1].timer  = MTOUCH_SLEEP_INIT_VALUE;   }
        #else
        for (int8_t i = MTOUCH_NUMBER_OF_SLEEP_SENSORS-1; i >= 0; i--)
        {
            mTouch_stateVars.sleep.sensor[i].state      = MTOUCH_SLEEP_INITIALIZING;
            mTouch_stateVars.sleep.sensor[i].avgCount   = MTOUCH_SLEEP_AVGRATE_VALUE;
            if (mTouch_state.sleepInitialized)  {   mTouch_stateVars.sleep.sensor[i].timer  = MTOUCH_SLEEP_REINIT_VALUE; } 
            else                                {   mTouch_stateVars.sleep.sensor[i].timer  = MTOUCH_SLEEP_INIT_VALUE;   }
        }
        #endif

    }
}
#endif 
#endif

#if defined(MTOUCH_PROX_ENABLED)
void mTouch_ProximityDecode(void)
{
    uint16_t* proxArray = &mTouch_stateVars.active.proxBuffer[0][0];    // Point to the proximity buffer
    uint16_t  sortArray[MTOUCH_PROX_BUFFER_SIZE];                       // Declare the temporary sort buffer
    uint16_t  mean = 0;                                                 // Stores the mean reading of the prox sensor
    
    // Update the proximity buffer with the latest value.
    for (int8_t i = MTOUCH_PROX_BUFFER_SIZE-1; i > 0; i--)
    {
        proxArray[i] = proxArray[i-1];
    }
    proxArray[0] = mTouch_GetSensor(MTOUCH_PROX_SLEEP_SENSOR0);
    
    // Copy the proximity buffer to the sort buffer.
    for (int8_t i = MTOUCH_PROX_BUFFER_SIZE-1; i >= 0; i--)
    {
        sortArray[i] = proxArray[i];
    }
    
    #if MTOUCH_PROXIMITY_SORT == MTOUCH_PROX_FAST
    
        // Fast implementation:
        //  Remove the extreme min/max values from the array.
        //  The number of min/max pairs to remove depends on the
        //  MTOUCH_PROX_REMOVE_EXTREME value.
        for (uint8_t i = MTOUCH_PROX_REMOVE_EXTREME; i != 0; i--)
        {
            uint16_t    min = 0xFFFF;
            uint16_t    max = 0x0000;
            uint8_t     minIndex, maxIndex;
            
            for (int8_t j = MTOUCH_PROX_BUFFER_SIZE-1; j >= 0; j--)
            {
                uint8_t value = sortArray[j];
                if (value != 0)
                {
                    if (value < min)
                    {
                        minIndex = j;
                        min = value;
                    } 
                    if (value > max)
                    {
                        maxIndex = j;
                        max = value;
                    }  
                }      
            }    
            sortArray[maxIndex] = 0;
            sortArray[minIndex] = 0;
        }
        
        // Sum all remaining non-extreme values.
        for (int8_t i = MTOUCH_PROX_BUFFER_SIZE-1; i >= 0; i--)
        {
            mean += sortArray[i];
        }  
        
    #elif MTOUCH_PROXIMITY_SORT == MTOUCH_PROX_EFFICIENT
    
        // Efficient Implementation:
        //  Sort the array using 'insertion sort' which requires
        //  only 4 bytes of RAM. This can take up to twice as long
        //  but is a good option for limiting the RAM requirements.
        for (uint8_t i = 1; i <= MTOUCH_PROX_BUFFER_SIZE-1; i++)
        {
            int8_t   j      = i-1;
            uint16_t value  = sortArray[i];
            
            while ((sortArray[j] > value) && (j >= 0))
            {
                sortArray[j+1] = sortArray[j];
                j--;  
            }    
            sortArray[j+1] = value;
        }
        
        // Sum the median values
        //
        //                      MTOUCH_PROX_REMOVE_EXTREME value vs. which array locations
        //                      are used to calculate the mean of the proximity sensor.
        //                      ==========================================================
        //  Buffer Index Value |0   1   2   3   4   5   6   7   8   9   10  11  12  13  14|
        //                     |==========================================================|
        //  Size            5  |1   2   -   2   1|XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX|
        //     of           9  |1   2   3   4   -   4   3   2   1|XXXXXXXXXXXXXXXXXXXXXXXX|
        //      Buffer     15  |1   2   3   4   5   6   7   -   7   6   5   4   3   2   1 |
        //                      ==========================================================
        //
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 2)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  9) && (MTOUCH_PROX_REMOVE_EXTREME < 2)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  5) && (MTOUCH_PROX_REMOVE_EXTREME < 2))
        mean += sortArray[1];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 3)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  9) && (MTOUCH_PROX_REMOVE_EXTREME < 3)) || \
             (MTOUCH_PROX_BUFFER_SIZE ==  5)
        mean += sortArray[2];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 4)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  9) && (MTOUCH_PROX_REMOVE_EXTREME < 4)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  5) && (MTOUCH_PROX_REMOVE_EXTREME < 2))
        mean += sortArray[3];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 5)) || \
             (MTOUCH_PROX_BUFFER_SIZE ==  9)
        mean += sortArray[4];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 6)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  9) && (MTOUCH_PROX_REMOVE_EXTREME < 4))
        mean += sortArray[5];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 7)) || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  9) && (MTOUCH_PROX_REMOVE_EXTREME < 3))
        mean += sortArray[6];
        #endif
        #if  (MTOUCH_PROX_BUFFER_SIZE == 15)                                      || \
            ((MTOUCH_PROX_BUFFER_SIZE ==  9) && (MTOUCH_PROX_REMOVE_EXTREME < 2))
        mean += sortArray[7];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 7))
        mean += sortArray[8];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 6))
        mean += sortArray[9];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 5))
        mean += sortArray[10];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 4))
        mean += sortArray[11];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 3))
        mean += sortArray[12];
        #endif
        #if ((MTOUCH_PROX_BUFFER_SIZE == 15) && (MTOUCH_PROX_REMOVE_EXTREME < 2))
        mean += sortArray[13];
        #endif
        
    #endif
 
    
    // Now, with the new mean value, perform proximity decoding.
    
    // ...
}
#endif