/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_Acquisition.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework Acquisition Module
 *                   - Implements the mTouch acquisition process. 
 *                     Additional ISR functions should be carefully 
 *                     designed to not interfere with the mTouch interrupt 
 *                     service.
 *                   - See the documentation for more information about
 *                     implementing the framework with your application.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
 /****************************************************************************
 * NOTE:  This is partially written in assembly to guarantee proper timing 
 *        control for noise robustness. PORTING THIS CODE TO 'C' COULD MAKE IT 
 *        LESS ROBUST IN NOISY ENVIRONMENTS! We highly suggest you do not 
 *        modify the macros used to implement these scans.
 *
 * Revision History
 * Date		Rev	 Author  Comments
 * 09.20.10    1.00         Public Release
 ****************************************************************************/
/** 
* @file     mTouch_Acquistion.c
* @brief    Implements scanning procedure for the framework which performs the 
*           mTouch acquisition and provides an oversampled, scaled integer
*           value representing the relative capacitance of the sensor.
*/
#include "mTouchCVD.h"

/** @name mTouch Acquisition ISR Variables
* These variables are used by the ISR of the mTouch Framework and should not be directly
* accessed by the main loop application.
*/
//@{
#if defined(MTOUCH_ACTIVE_ENABLED)
  mTouch_AcquisitionData        mTouch_acqData      [MTOUCH_NUMBER_SENSORS];    
                uint16_t        mTouch_sensorData   [MTOUCH_NUMBER_SENSORS];    
                uint16_t        mTouch_lastResult;                                      ///< Stores the last active-mode ADC result for differential calculation        @ingroup Acquisition
#endif

#if defined(MTOUCH_SLEEP_ENABLED)
  mTouch_AcquisitionData        mTouch_sleep_acqData[MTOUCH_NUMBER_OF_SLEEP_SENSORS];   
                uint16_t        mTouch_sleep_lastResult;                                ///< Stores the last sleep-mode ADC result for differential calculation         @ingroup Acquisition
#endif

#if defined(MTOUCH_JITTER_ENABLE)
                 uint8_t        mTouch_jitter           = 0x55;                         ///< Stores the current random seed value for jittering                         @ingroup Acquisition
    #if defined(MTOUCH_V2_COMMS)
           const uint8_t        mTouch_jitter_mask[9]   = MTOUCH_JITTER_MASK_ARRAY; 
    #endif
#endif
#if (MTOUCH_NUMBER_SENSORS > 1)
                 uint8_t        mTouch_currentSensor;                                   ///< Currently scanned sensor index value                                       @ingroup Acquisition
#endif


#if defined(CVD_DELAY_VARIABLE_REQUIRED)
    // Depending on the current configuration of the framework, this variable may or may not
    // be required to generate delays for the scan routines. If it's not required, it is not
    // included so as to save as much RAM space as possible.
                 uint8_t        mTouch_delayCount;                                      ///< Delay counter variable used in CVD scan sequence                           @ingroup Acquisition
#endif

#if defined(_PIC14)
    // These state-saving variables are only required in non-enhanced core PIC microcontrollers. 
    // They are automatically omitted if not needed.
    near    uint8_t     int_w;              
    near    uint8_t     int_status;         
    near    uint8_t     int_fsr;            
    near    uint8_t     int_pclath;         
    // TIP: The 'near' qualifier tells the compiler to place these variables in 'common' memory so 
    //      they may be accessed from any bank. On the enhanced core products, this typically 
    //      refers to the registers: 0x70 to 0x7F.
#elif defined(_PIC18)
            uint8_t     int_w;
            uint8_t     int_status;
            uint8_t     int_bsr;
// TIP: The 'near' qualifier is not needed in this case due to the PIC18's bank access features.
#endif
//@}


/**
*   @ingroup    Acquisition
*   @brief      Implements the framework's active-mode acquisition routine. 
*               Sometimes implemented as an ISR, sometimes as a normal function.
*
*   Implements the acquisition process for the mTouch Framework. 
*   Please do not modify.
*
*/
#if defined(MTOUCH_ACTIVE_ENABLED)
#if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR)
void interrupt  mTouch_Scan(void)
#elif (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)
void            mTouch_Scan(void)
#elif (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
uint8_t         mTouch_Scan(void)
#endif
{
    #asm
    #ifndef MTOUCH_ACQ_CAS_INCLUDE
    #define MTOUCH_ACQ_CAS_INCLUDE
        #if defined(_12F617) && (_HTC_VER_MAJOR_ == 9 && _HTC_VER_MINOR_ == 80)
            #include <cas12F617.h>      // HiTech's 9.80 compiler's caspic.h includes the 617 header file twice, causing compiling 
                                        // errors. This will by-pass the mistake by directly including the correct file only once.
                                        // This issue has been fixed as of the 9.81 release.
        #elif (_PIC18)
            //#include <caspic.h>
        #else
            #include <caspic.h>
        #endif
    #endif

mTouch_Scan:
    #endasm 
    
    /**
    * @section AcqImp Algorithm Implementation Notes
    *
    * 1. If required, save the current state of the main loop application.
    */
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR)
    SAVE_STATE();
    #endif
    
    /**
    * 2. Initialize temporary variables with this sensor's data.
    */
    #if MTOUCH_NUMBER_SENSORS > 1
    mTouch_AcquisitionData* data    = &mTouch_acqData[mTouch_currentSensor];
    #else
    mTouch_AcquisitionData* data    = &mTouch_acqData[0];
    #endif
    uint16_t                result  = mTouch_GetResult(data);
        
    /**
    * 3. If running as the only interrupt, check and clear the TMR0 interrupt
    *    flag. If it is not set, leave the ISR by jumping to END_MTOUCH_SCAN.
    */
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR)
    CHECK_AND_CLEAR_TMRxIF();
    #elif (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    if (mTouch_state.scanningEnabled == 0)
    {
        return 0;
    }    
    #endif
    
    /**
    * 4. If running out of the main loop, implement a random delay here to
    *    help vary the sampling rate.
    */
    #if defined(MTOUCH_EEPROM_ENABLE) && (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    if (MTOUCH_JITTER_VALUE != 0)
    {
    #endif
    
        #if defined(MTOUCH_EEPROM_ENABLE) || defined(MTOUCH_JITTER_ENABLE)
            #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)

                JITTER_MAIN_LOOP(); 
                    
            #endif
        #endif
        
    #if defined(MTOUCH_EEPROM_ENABLE) && (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    }
    #endif
    
    //
    //  SCANNING LOGIC
    //
    //  1. Scan A
    //  2. Math on Index-1 results
    //  3. Scan B
    //  4. 
    //      A: Index increment and END_MTOUCH_SCAN jump
    //      B: Index reset, sample counter decrement
    //          * END_MTOUCH_SCAN jump
    //          * Sample counter reset and fall-through to storage
    //
    
    
    /**
    * 5. Perform mTouch sensor scanning:
    */
    /**
    * <b>=== START: If only scanning one sensor per function call</b>
    */
    #if (MTOUCH_SCAN_FUNCTIONALITY == MTOUCH_SCANS_ONE_SENSOR)
        /**
        * A. If only one sensor is being scanned per function call and the framework
        *    has control of the ADC at all times, scan B's conversion will not be
        *    stored until the next sensor scan's ISR call in order to minimize the
        *    total ISR time. In this case, we need to check the GO/DONE bit and store
        *    the result of scan B.
        */
        #if (MTOUCH_ADC_CONTROL == MTOUCH_ALWAYS_CONTROLS_ADC)
            WAIT_FOR_GODONE_BIT();
            MTOUCH_STORE_SCAN_B();
        #endif
        
        /**
        * B. Implement and use a jump table based on which sensor should be
        *    scanned next.
        */ 
        // Only supported in PICC 9.81 and later
        #if (_HTC_VER_MAJOR_ > 9) || ((_HTC_VER_MAJOR_ == 9) && (_HTC_VER_MINOR_ > 80))
        #pragma switch rangetable       
        #endif
        // TIP: The 'switch rangetable' option tells the compiler to implement
        //      the following switch statement as a jump table. It is still not as
        //      efficient as hand-coded assembly, however.
        GENERATE_JUMP_TABLE();   

        /**
        * C. Generate the correct macro functions to implement the mTouch CVD
        *    scanning algorithms. This step uses the number of sensors to implement
        *    a 'previous sensor used as the reference sensor' configuration. The
        *    last sensor uses the EXIT_LAST_SENSOR() macro instead of the EXIT_SENSOR()
        *    macro. This allows the storage function to be performed instead of
        *    immediately moving to the END_MTOUCH_SCAN label.
        */
        #if MTOUCH_NUMBER_SENSORS > 0
            #asm
            sensor_0:                           // This label is used by the jump table, above.
            #endasm
                MTOUCH_PERFORM_SCAN(0);         
                MTOUCH_EXIT_SCAN_FUNCTION(0);   // Exits the ISR
        #endif
        #if MTOUCH_NUMBER_SENSORS > 1
            #asm
            sensor_1:
            #endasm
                MTOUCH_PERFORM_SCAN(1);
                MTOUCH_EXIT_SCAN_FUNCTION(1);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 2
            #asm
            sensor_2:
            #endasm
                MTOUCH_PERFORM_SCAN(2);
                MTOUCH_EXIT_SCAN_FUNCTION(2);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 3
            #asm
            sensor_3:
            #endasm
                MTOUCH_PERFORM_SCAN(3);
                MTOUCH_EXIT_SCAN_FUNCTION(3);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 4
            #asm
            sensor_4:
            #endasm
                MTOUCH_PERFORM_SCAN(4);
                MTOUCH_EXIT_SCAN_FUNCTION(4);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 5
            #asm
            sensor_5:
            #endasm
                MTOUCH_PERFORM_SCAN(5);
                MTOUCH_EXIT_SCAN_FUNCTION(5);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 6
            #asm
            sensor_6:
            #endasm
                MTOUCH_PERFORM_SCAN(6);
                MTOUCH_EXIT_SCAN_FUNCTION(6);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 7
            #asm
            sensor_7:
            #endasm
                MTOUCH_PERFORM_SCAN(7);
                MTOUCH_EXIT_SCAN_FUNCTION(7);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 8
            #asm
            sensor_8:
            #endasm
                MTOUCH_PERFORM_SCAN(8);
                MTOUCH_EXIT_SCAN_FUNCTION(8);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 9
            #asm
            sensor_9:
            #endasm
                MTOUCH_PERFORM_SCAN(9);
                MTOUCH_EXIT_SCAN_FUNCTION(9);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 10
            #asm
            sensor_10:
            #endasm
                MTOUCH_PERFORM_SCAN(10);
                MTOUCH_EXIT_SCAN_FUNCTION(10);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 11
            #asm
            sensor_11:
            #endasm
                MTOUCH_PERFORM_SCAN(11);
                MTOUCH_EXIT_SCAN_FUNCTION(11);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 12
            #asm
            sensor_12:
            #endasm
                MTOUCH_PERFORM_SCAN(12);
                MTOUCH_EXIT_SCAN_FUNCTION(12);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 13
            #asm
            sensor_13:
            #endasm
                MTOUCH_PERFORM_SCAN(13);
                MTOUCH_EXIT_SCAN_FUNCTION(13);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 14
            #asm
            sensor_14:
            #endasm
                MTOUCH_PERFORM_SCAN(14);
                MTOUCH_EXIT_SCAN_FUNCTION(14);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 15
            #asm
            sensor_15:
            #endasm
                MTOUCH_PERFORM_SCAN(15);
                MTOUCH_EXIT_SCAN_FUNCTION(15);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 16
            #asm
            sensor_16:
            #endasm
                MTOUCH_PERFORM_SCAN(16);
                MTOUCH_EXIT_SCAN_FUNCTION(16);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 17
            #asm
            sensor_17:
            #endasm
                MTOUCH_PERFORM_SCAN(17);
                MTOUCH_EXIT_SCAN_FUNCTION(17);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 18
            #asm
            sensor_18:
            #endasm
                MTOUCH_PERFORM_SCAN(18);
                MTOUCH_EXIT_SCAN_FUNCTION(18);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 19
            #asm
            sensor_19:
            #endasm
                MTOUCH_PERFORM_SCAN(19);
                MTOUCH_EXIT_SCAN_FUNCTION(19);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 20
            #asm
            sensor_20:
            #endasm
                MTOUCH_PERFORM_SCAN(20);
                MTOUCH_EXIT_SCAN_FUNCTION(20);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 21
            #asm
            sensor_21:
            #endasm
                MTOUCH_PERFORM_SCAN(21);
                MTOUCH_EXIT_SCAN_FUNCTION(21);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 22
            #asm
            sensor_22:
            #endasm
                MTOUCH_PERFORM_SCAN(22);
                MTOUCH_EXIT_SCAN_FUNCTION(22);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 23
            #asm
            sensor_23:
            #endasm
                MTOUCH_PERFORM_SCAN(23);
                MTOUCH_EXIT_SCAN_FUNCTION(23);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 24
            #asm
            sensor_24:
            #endasm
                MTOUCH_PERFORM_SCAN(24);
                MTOUCH_EXIT_SCAN_FUNCTION(24);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 25
            #asm
            sensor_25:
            #endasm
                MTOUCH_PERFORM_SCAN(25);
                MTOUCH_EXIT_SCAN_FUNCTION(25);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 26
            #asm
            sensor_26:
            #endasm
                MTOUCH_PERFORM_SCAN(26);
                MTOUCH_EXIT_SCAN_FUNCTION(26);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 27
            #asm
            sensor_27:
            #endasm
                MTOUCH_PERFORM_SCAN(27);
                MTOUCH_EXIT_SCAN_FUNCTION(27);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 28
            #asm
            sensor_28:
            #endasm
                MTOUCH_PERFORM_SCAN(28);
                MTOUCH_EXIT_SCAN_FUNCTION(28);
        #endif
        #if MTOUCH_NUMBER_SENSORS > 29
            #asm
            sensor_29:
            #endasm
                MTOUCH_PERFORM_SCAN(29);
                MTOUCH_EXIT_SCAN_FUNCTION(29);
        #endif 

        // End (MTOUCH_SCAN_FUNCTIONALITY == MTOUCH_SCANS_ONE_SENSOR)
    
    
    /**
    * <b>=== END: If only scanning one sensor per function call</b>
    */
    /**
    * <b>=== START: If scanning all sensors each function call</b>
    */
    #elif (MTOUCH_SCAN_FUNCTIONALITY == MTOUCH_SCANS_ALL_SENSORS)
        
        /**
        * A. Generate the correct macro functions to scan all sensors at once. After a
        *    sensor has been scanned, the index value is incremented and the next sensor
        *    scan begins. At the end, the routine will wait on the last ADC conversion
        *    to complete before returning.
        */
        #if MTOUCH_NUMBER_SENSORS == 1
        
            #asm
            sensor_0:
            #endasm            
                MTOUCH_PERFORM_SCAN(0);
                MTOUCH_EXIT_SCAN_FUNCTION(0);
            
        #else
        
                mTouch_currentSensor = 0;
            #asm
            sensor_0:
            #endasm   
                MTOUCH_PERFORM_SCAN(0);
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_1:
            #endasm            
                MTOUCH_PERFORM_SCAN(1);
                #if MTOUCH_NUMBER_SENSORS > 2
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_2:
            #endasm            
                MTOUCH_PERFORM_SCAN(2);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 3
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_3:
            #endasm            
                MTOUCH_PERFORM_SCAN(3);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 4
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_4:
            #endasm            
                MTOUCH_PERFORM_SCAN(4);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 5
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_5:
            #endasm            
                MTOUCH_PERFORM_SCAN(5);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 6
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_6:
            #endasm            
                MTOUCH_PERFORM_SCAN(6);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 7
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_7:
            #endasm            
                MTOUCH_PERFORM_SCAN(7);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 8
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_8:
            #endasm            
                MTOUCH_PERFORM_SCAN(8);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 9
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_9:
            #endasm            
                MTOUCH_PERFORM_SCAN(9);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 10
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_10:
            #endasm            
                MTOUCH_PERFORM_SCAN(10);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 11
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_11:
            #endasm            
                MTOUCH_PERFORM_SCAN(11);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 12
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_12:
            #endasm            
                MTOUCH_PERFORM_SCAN(12);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 13
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_13:
            #endasm            
                MTOUCH_PERFORM_SCAN(13);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 14
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_14:
            #endasm            
                MTOUCH_PERFORM_SCAN(14);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 15
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_15:
            #endasm            
                MTOUCH_PERFORM_SCAN(15);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 16
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_16:
            #endasm            
                MTOUCH_PERFORM_SCAN(16);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 17
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_17:
            #endasm            
                MTOUCH_PERFORM_SCAN(17);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 18
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_18:
            #endasm            
                MTOUCH_PERFORM_SCAN(18);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 19
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_19:
            #endasm            
                MTOUCH_PERFORM_SCAN(19);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 20
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_20:
            #endasm            
                MTOUCH_PERFORM_SCAN(20);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 21
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_21:
            #endasm            
                MTOUCH_PERFORM_SCAN(21);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 22
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_22:
            #endasm            
                MTOUCH_PERFORM_SCAN(22);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 23
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_23:
            #endasm            
                MTOUCH_PERFORM_SCAN(23);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 24
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_24:
            #endasm            
                MTOUCH_PERFORM_SCAN(24);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 25
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_25:
            #endasm            
                MTOUCH_PERFORM_SCAN(25);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 26
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_26:
            #endasm            
                MTOUCH_PERFORM_SCAN(26);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 27
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_27:
            #endasm            
                MTOUCH_PERFORM_SCAN(27);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 28
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_28:
            #endasm            
                MTOUCH_PERFORM_SCAN(28);
                #endif
                #if MTOUCH_NUMBER_SENSORS > 29
                MTOUCH_INC_INDEX_AND_STORE_RESULT();
            #asm
            sensor_29:
            #endasm            
                MTOUCH_PERFORM_SCAN(29);
                #endif
                
            // This code segment will wait for the last ADC scan to complete
            // and store the result before continuing to the next step.
            mTouch_currentSensor = 0;
            WAIT_FOR_GODONE_BIT();
            MTOUCH_STORE_SCAN_B();
            EXIT_LAST_SENSOR_MAIN();
        
        #endif
    #endif  // End (MTOUCH_SCAN_FUNCTIONALITY == MTOUCH_SCANS_ALL_SENSORS)
    /**
    * <b>=== END: If scanning all sensors each function call</b>
    */
    
    /**
    * 6. Uses the number of sensors to generate a storage macro for each 
    *    sensor. This step is only reached after a decrementing counter,
    *    initialized to #MTOUCH_SAMPLES_PER_SCAN, reaches 0.
    */    
    GENERATE_STORAGE_FUNCTION();    
    
    /**
    * 7. Sets the mTouch_state.dataReady flag so the main application knows
    *    to process the new data.
    */   
    SET_DATA_READY_FLAG();
    
    /// <br><br><br>
    
    asm("goto END_MTOUCH_SCAN");
    
    /**
    * @section AcqDec Decimation Helper Function
    *
    * The decimate function is split into two sequences: SCAN_A_PROCESSING and SCAN_B_PROCESSING.
    * As the name implies, these routines will be called while the respective ADC conversions are
    * completing.
    */    
    /// <br><br><br>
    #asm
    SCAN_A_PROCESSING:
    #endasm

    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    if (!mTouch_state.isRepeatScan)
    {
    #endif
    
        #if defined(MTOUCH_V2_COMMS)
            #if MTOUCH_DECIMATION_MAX_STEP > 255
            uint16_t    decMaxStep = MTOUCH_DECIMATION_MAX_VALUE;
            #else
            uint8_t     decMaxStep = MTOUCH_DECIMATION_MAX_VALUE;
            #endif
            
            #undef MTOUCH_DECIMATION_MAX_VALUE
            #define MTOUCH_DECIMATION_MAX_VALUE decMaxStep      // Replace the EEPROM access code with our temporary variable
        #endif

        #if defined(MTOUCH_DECIMATION_MAX_STEP) || defined(MTOUCH_V2_COMMS)
            #if (MTOUCH_DECIMATION_MAX_STEP != 0) || defined(MTOUCH_V2_COMMS)
                if ((uint16_t)(mTouch_lastResult) > (uint16_t)(result + MTOUCH_DECIMATION_MAX_VALUE))
                {                                                                                           
                    result += MTOUCH_DECIMATION_MAX_VALUE;      // Latest conversion is very large. Increment by max step size only.
                }                                                                         
                else if (result > MTOUCH_DECIMATION_MAX_VALUE)                                     
                {                                                                               
                    if ((uint16_t)(mTouch_lastResult) < result - MTOUCH_DECIMATION_MAX_VALUE)  
                    {                                                                           
                        result -= MTOUCH_DECIMATION_MAX_VALUE;  // Latest conversion is very small. Decrement by max step size only.
                    } else {          
                        result = mTouch_lastResult;             // Latest conversion is similar in value to decimated value.
                    }                                                                               
                }                                                                                   
                else                                            // Latest conversion is less than (decimated value + STEP_SIZE)
                {                                               // AND the decimated value is in danger of underflowing.                               
                    result = mTouch_lastResult; 
                } 
            #else                                           
                result = mTouch_lastResult;                     // Decimation disabled.
            #endif
        #else                                               
            result = mTouch_lastResult;                         // Decimation disabled.
        #endif
        
        
        mTouch_SetResult(data, result);                         // Store the new decimated value into memory for later use.
        
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    }
    #endif   
    
    asm("return");
    
    
    /// <br><br><br>
    
    #asm
    SCAN_B_PROCESSING:
    #endasm
    
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    if (mTouch_state.isRepeatScan)                              
    {
        mTouch_state.isRepeatScan = 0;              // If we are repeating a scan due to being interrupted by
                                                    // the ISR, do not process the previous sensor's data
                                                    // twice.
    } else {
    #endif
    
        mTouch_UpdateAccumulator(data);             // Take the result from SCAN_A_PROCESSING and update the
                                                    // accumulator value for the previous sensor.
    
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
    }
    #endif
    
    asm("return");

    /// <br><br><br>
    
    /**
    * @section AcqEnd END_MTOUCH_SCAN
    *
    * This section determines the actions performed when exiting the mTouch scanning routine.
    */    
    /// <br><br><br>
        
    #asm
    END_MTOUCH_SCAN:
    #endasm
    
    /**
    * 1. If running the mTouch scanning process from a timer's interrupt, reload the timer's 
    *    register to prepare for the next mTouch scan after this one.
    */
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR) || (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)

        #if defined(MTOUCH_EEPROM_ENABLE)
        if (MTOUCH_JITTER_VALUE != 0)
        {
        #endif
        
            #if defined(MTOUCH_EEPROM_ENABLE) || defined(MTOUCH_JITTER_ENABLE)
                JITTER_START_TIME(); 
            #endif
            
        #if defined(MTOUCH_EEPROM_ENABLE)
        }
        #endif
        
        MTOUCH_ISR_TMRxIF = 0;          // Clear TMRxIF to avoid immediate ISR re-entry upon leaving.
        
    #endif
    
    /**
    * 2. If required, restore the state of the main loop application. Then either 
    *    return from the interrupt vector if running as the only interrupt, return
    *    to the main interrupt function, or return with the 'dataReady' flag value
    *    if running out of the main loop.
    */
    #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR)
        RESTORE_STATE();
        asm("retfie");
    #elif (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)
        asm("return");
    #elif (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_MAINLOOP)
        return mTouch_state.dataReady;
    #endif

}
#endif  // End: #if defined(MTOUCH_ACTIVE_ENABLED)


#if defined(MTOUCH_SLEEP_ENABLED)
/**
*   @ingroup    Acquisition
*   @brief      Implements the framework's sleep-mode acquisition routine. 
*               
*   Implements the acquisition process for the mTouch Framework. 
*   Please do not modify.
*
*/
uint8_t mTouch_Sleep_Scan(void)
{
    #ifndef MTOUCH_ACQ_CAS_INCLUDE
    #define MTOUCH_ACQ_CAS_INCLUDE
        #if defined(_12F617) && (_HTC_VER_MAJOR_ == 9 && _HTC_VER_MINOR_ == 80)
            #include <cas12F617.h>      // HiTech's 9.80 compiler's caspic.h includes the 617 header file twice, causing compiling 
                                        // errors. This will by-pass the mistake by directly including the correct file only once.
                                        // This issue has been fixed as of the 9.81 release.
        #elif (_PIC18)
            #include <caspic18.h>
            #warning PIC18 sleep mode is still under development and may not be fully functional.
        #else
            #include <caspic.h>
        #endif
    #endif

    mTouch_AcquisitionData* data;           // Used to access sensor information
    uint16_t                result;         // Used to store the decimated value of a sensor.
    
    #if MTOUCH_SLEEP_SAMPLES_PER_SCAN < 256
    uint8_t                 counter;        // Used to count the number of times each sensor has been scanned.
    #else
    uint16_t                counter;
    #endif
    
    #if defined(MTOUCH_ACTIVE_ENABLED)
    if (!mTouch_state.sleep)
    {
    
        #if (MTOUCH_INTEGRATION_TYPE == MTOUCH_CONTROLS_ISR) || (MTOUCH_INTEGRATION_TYPE == MTOUCH_CALLED_FROM_ISR)
            return mTouch_state.dataReady;              
        #else
            return mTouch_Scan();                       // Returns the updated value of mTouch_state.dataReady
        #endif
        
    }
    #endif
    
    
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 0) 
        data = &mTouch_sleep_acqData[0];                // Clear all accumulation registers in preparation for
        mTouch_ClearAccumulator(data);                  // the upcoming round of scans.
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 1) 
        data = &mTouch_sleep_acqData[1];
        mTouch_ClearAccumulator(data);
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 2) 
        data = &mTouch_sleep_acqData[2];
        mTouch_ClearAccumulator(data);
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 3) 
        data = &mTouch_sleep_acqData[3];
        mTouch_ClearAccumulator(data);
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 4) 
        data = &mTouch_sleep_acqData[4];
        mTouch_ClearAccumulator(data);
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 5) 
        data = &mTouch_sleep_acqData[5];
        mTouch_ClearAccumulator(data);
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 6) 
        data = &mTouch_sleep_acqData[6];
        mTouch_ClearAccumulator(data);
    #endif
    #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 7) 
        data = &mTouch_sleep_acqData[7];
        mTouch_ClearAccumulator(data);
    #endif
    
    MTOUCH_SLEEP_STABILIZE_OSC();                       // If necessary, wait until the processor's internal
                                                        // oscillator has sufficiently stabilized.
    
    #if defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
    mTouch_state.isRepeatScan = 1;                      // If the 'isRepeatScan' bit is implemented, set it
                                                        // now to avoid doing math on the first conversion.
                                                        // Not mandatory but if it's there, we'll use it.
    #endif     
    
    result  = mTouch_GetResult(data);                   // Prepare the result variable for math processing.
    counter = MTOUCH_SLEEP_SAMPLES_PER_SCAN;            // Initialize the sample counter
    do 
    {
        MTOUCH_SLEEP_ASM_GLOBAL_LABELS();               // Generates a list of global labels for the assembler
                                                        // to be able to find 'sleep_sensor_0' and the others.
        
            #asm
            sleep_sensor_0:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(0, MTOUCH_SLEEP_INDEXMINUSONE_0);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(0);
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 1
            #asm
            sleep_sensor_1:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(1, 0);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(1);
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 2
            #asm
            sleep_sensor_2:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(2, 1);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(2);
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 3
            #asm
            sleep_sensor_3:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(3, 2);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(3);
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 4
            #asm
            sleep_sensor_4:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(4, 3);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(4);
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 5
            #asm
            sleep_sensor_5:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(5, 4);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(5);
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 6
            #asm
            sleep_sensor_6:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(6, 5);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(6);
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 7
            #asm
            sleep_sensor_7:
            #endasm
                MTOUCH_SLEEP_READ_SENSOR(7, 6);
                MTOUCH_SLEEP_INCINDEX_STORE_RESULT(7);
        #endif
        
    } while (--counter != 0);    
    
    asm("call SLEEP_SCAN_A_PROCESSING");    // Finish math on the last scan.
    asm("call SLEEP_SCAN_B_PROCESSING");
    asm("ljmp END_MTOUCH_SLEEP_SCAN");
    
    #asm
    SLEEP_SCAN_A_PROCESSING:
    #endasm
        #if defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
        if (!mTouch_state.isRepeatScan)
        {
        #endif
        #if (defined(MTOUCH_SLEEP_DECIMATION_STEP) && (MTOUCH_SLEEP_DECIMATION_STEP != 0)) || defined(MTOUCH_V2_COMMS)
                if ((uint16_t)(mTouch_sleep_lastResult) > (uint16_t)(result + MTOUCH_SLEEP_DEC_MAX_VALUE))           
                {                                                                                           
                    result += MTOUCH_SLEEP_DEC_MAX_VALUE;  
                }                                                                         
                else if (result > MTOUCH_SLEEP_DEC_MAX_VALUE)                                     
                {                                                                               
                    if ((uint16_t)(mTouch_sleep_lastResult) < result - MTOUCH_SLEEP_DEC_MAX_VALUE)  
                    {                                                                           
                        result -= MTOUCH_SLEEP_DEC_MAX_VALUE;                                     
                    } else {          
                        result = mTouch_sleep_lastResult;                                             
                    }                                                                               
                }                                                                                   
                else                                        
                {                                                              
                    result = mTouch_sleep_lastResult; 
                } 
        #else                                               
            result = mTouch_sleep_lastResult;               
        #endif
            mTouch_SetResult(data, result);
            
        #if defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
        }
        #endif    
        asm("return");
        
        
    #asm
    SLEEP_SCAN_B_PROCESSING:
    #endasm    
        #if defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
        if (mTouch_state.isRepeatScan)
        {
            mTouch_state.isRepeatScan = 0;
        } else {
        #endif
        
            mTouch_UpdateAccumulator(data);  
        
        #if defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
        }
        #endif
        asm("return");
    
    #asm
    END_MTOUCH_SLEEP_SCAN:
    #endasm
        return 1;       // Returns '1' because data is always ready at this point.
}
#endif
