/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_Comm.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework Communication Module
 *                   - Implements the communication between the mTouch
 *                     PC GUI program and the firmware. May also be used
 *                     to output directly to a PC's terminal.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file   mTouch_Comm.c
*   @brief  Implements the mTouch framework's communication functions for
*           both ASCII data and the 'V2' two-way protocol for debugging
*           with the mTouch Framework GUI.
*/
//*****************************************************************************
// Include and Header files
//*****************************************************************************
#include <stdlib.h>
#include "mTouchCVD.h"
#include "mTouch_comm.h"

#if defined(MTOUCH_V1_COMMS) || defined(MTOUCH_V2_COMMS)
    
    #if defined(MTOUCH_V2_COMMS)
        uint8_t     mTouch_comm_state;
        uint8_t     mTouch_comm_bufferCounter;
        uint8_t     mTouch_comm_bufferIndex;
        uint8_t     mTouch_comm_checksum;
                
        #if (MTOUCH_NUMBER_SENSORS <= 8)
        uint8_t     mTouch_comm_outputMask;   
        #elif (MTOUCH_NUMBER_SENSORS <= 16)
        uint16_t    mTouch_comm_outputMask;    
        #elif (MTOUCH_NUMBER_SENSORS <= 24)
        uint24_t    mTouch_comm_outputMask;    
        #elif (MTOUCH_NUMBER_SENSORS <= 32)
        uint32_t    mTouch_comm_outputMask;    
        #endif
        
        uint8_t     mTouch_comm_buffer[MTOUCH_V2_COMMS_BUFFER_SIZE];
    #endif
    
    #if defined(MTOUCH_V1_COMM_BITBANGED)
    bank0 uint8_t   mTouch_comm_shiftReg;
    bank0 uint8_t   mTouch_comm_bitCount;
    bank0 uint8_t   mTouch_comm_delayCount;
    #endif
    
//*****************************************************************************
// Function Prototypes
//*****************************************************************************
    ////////////////////////////////////////////////////
    /*  Globally Accessible System Support Functions  */
    ////////////////////////////////////////////////////
    void    mTouch_Comm_Init        (void);
    void    mTouch_Comm_SendData    (void);
    
    #if defined(MTOUCH_V2_COMMS)
    void    mTouch_Comm_ReceiveData (void);
    void    mTouch_Comm_ProcessData (void);
    void    mTouch_Comm_SendMode    (void);
    uint8_t v2Comm_putChar          (uint8_t);
    uint8_t v2Comm_sendOpcode       (uint8_t, uint8_t);
    #endif

    ////////////////////////////////////////////////////
    /*      Locally Accessible Support Functions      */
    ////////////////////////////////////////////////////
    void    put_Char     (uint8_t);
    void    str_Int      (uint16_t);


/****************************************************************
* Function: void mTouch_Comm_Init(void)
*
* Overview: 
*
* Input:    None
*
* Output:   None
*
****************************************************************/
/**
*   @brief This function configures the communiation peripherals and presets the system variables
*/
void mTouch_Comm_Init(void)
{
    // Turn on the Hardware UART - this is defined in each HardwareProfile header file
    #if defined(MTOUCH_V1_COMM_BITBANGED)
    MTOUCH_COMM_232_INIT();
    #else
    MTOUCH_DEBUG_COMM_INIT();
    #endif
    
    #if defined(MTOUCH_V2_COMMS)
    mTouch_comm_state               = MTOUCH_COMM_IDLE;
    mTouch_comm_bufferCounter       = 0;
    mTouch_comm_bufferIndex         = 0;
    mTouch_comm_checksum            = 0;
    mTouch_comm_outputMask          = 0;
    mTouch_state.commInvalid        = 0;
    mTouch_state.commNewCommand     = 0;
    mTouch_state.commStreaming      = 0;
        
    for (int8_t i = MTOUCH_V2_COMMS_BUFFER_SIZE-1; i >= 0; i--)
    {
        mTouch_comm_buffer[i] = 0;
    }
    
    RCIE    = 1;
    PEIE    = 1;
    #endif
    
}


///////////////////////////////////////////////////////////////////////////////
//*****************************************************************************
// Communications Functions
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////

/**
*   @brief Performs a loop and outputs data to the PC according to the set protocol
*/
void mTouch_Comm_SendData(void)
{   
    #if defined(MTOUCH_V2_COMMS)
        uint8_t     checksum    = 0;        
    
        mTouch_state.commStreaming      = 1;    
           
        
        #if (MTOUCH_NUMBER_SENSORS > 2)
        #if (MTOUCH_NUMBER_SENSORS <= 8)
        uint8_t     bitCheck = 1;   
        #elif (MTOUCH_NUMBER_SENSORS <= 16)
        uint16_t    bitCheck = 1;    
        #elif (MTOUCH_NUMBER_SENSORS <= 24)
        uint24_t    bitCheck = 1;    
        #elif (MTOUCH_NUMBER_SENSORS <= 32)
        uint32_t    bitCheck = 1;    
        #endif
        #endif
    
        #if defined(MTOUCH_ACTIVE_ENABLED)
        if ((mTouch_comm_outputMask != 0) || (mTouch_state.commButtonStateReq && mTouch_state.buttonStateChange))
        {
            uint8_t     size = 6;
                            
            #if (MTOUCH_NUMBER_SENSORS <= 8)
            uint8_t     buttonMask  = 0;      
            #elif (MTOUCH_NUMBER_SENSORS <= 16)
            uint16_t    buttonMask  = 0;      
            #elif (MTOUCH_NUMBER_SENSORS <= 24)
            uint24_t    buttonMask  = 0;       
            #elif (MTOUCH_NUMBER_SENSORS <= 32)
            uint32_t    buttonMask  = 0;   
            #endif
            
            
            #if (MTOUCH_NUMBER_SENSORS == 1)
            if (mTouch_GetButtonState(0) == MTOUCH_PRESSED) { buttonMask = 1;  }
            if (mTouch_comm_outputMask)                     { size       = 6 + MTOUCH_V2_STREAMBYTESperSENSOR; }
            #elif (MTOUCH_NUMBER_SENSORS == 2)
            if (mTouch_GetButtonState(0) == MTOUCH_PRESSED) 
            {
                if (mTouch_GetButtonState(1) == MTOUCH_PRESSED) {    buttonMask = 3;   } else {    buttonMask = 1;   }
            } else {
                if (mTouch_GetButtonState(1) == MTOUCH_PRESSED) {    buttonMask = 2;   } else {    buttonMask = 0;   }
            }
            if (mTouch_comm_outputMask & 0x01) size += MTOUCH_V2_STREAMBYTESperSENSOR;
            if (mTouch_comm_outputMask & 0x02) size += MTOUCH_V2_STREAMBYTESperSENSOR;                
            #else
            for (uint8_t i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
            {
                if (mTouch_GetButtonState(i) == MTOUCH_PRESSED)     // Create button state output, regardless of 
                {                                                   // current outputMask value.
                    buttonMask |= bitCheck;     
                }
                
                if ((mTouch_comm_outputMask & bitCheck) != 0)       // Calculate the size of the output packet
                {   
                    size += MTOUCH_V2_STREAMBYTESperSENSOR;
                }
                
                bitCheck <<= 1;
            }
            #endif
            
            
            // Stream Frame Format
            //   START                                                                                                                    END
            //   break  | Byte Count | Opcode | Button State | Sensor0 Raw | Sensor0 Average | ... | SensorN Raw | SensorN Average | Checksum
            // 13 bits        1 byte   1 byte        4 bytes       2 bytes           2 bytes             2 bytes           2 bytes     1 byte
            
            
            if (mTouch_comm_outputMask)
            {
                MTOUCH_COMM_SEND_OP_MAINLOOP(MTOUCH_COMM_STREAM_ACTIVE,size);       // Output break character + op code + byte count
            }
            else
            {
                MTOUCH_COMM_SEND_OP_MAINLOOP(MTOUCH_COMM_STREAM_MASK_ACTIVE,size);  // Output break character + op code + byte count
            }    
            
            MTOUCH_COMM_PUT_CHAR((uint8_t) buttonMask);                      // Output button state data
            #if (MTOUCH_NUMBER_SENSORS > 8)
                MTOUCH_COMM_PUT_CHAR((uint8_t)((uint16_t)(buttonMask) >> 8));
                #if (MTOUCH_NUMBER_SENSORS > 16)
                    MTOUCH_COMM_PUT_CHAR((uint8_t)((uint24_t)(buttonMask) >> 16));
                    #if (MTOUCH_NUMBER_SENSORS > 24)
                        MTOUCH_COMM_PUT_CHAR((uint8_t)((uint32_t)(buttonMask) >> 24));
                    #else
                        MTOUCH_COMM_PUT_CHAR(0x00);
                    #endif
                #else
                    MTOUCH_COMM_PUT_CHAR(0x00);
                    MTOUCH_COMM_PUT_CHAR(0x00);
                #endif
            #else
                MTOUCH_COMM_PUT_CHAR(0x00);
                MTOUCH_COMM_PUT_CHAR(0x00);
                MTOUCH_COMM_PUT_CHAR(0x00);
            #endif
        }
        
        if (mTouch_comm_outputMask)
        {
            #if (MTOUCH_NUMBER_SENSORS == 1)
                MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetSensor(0));            // Output raw data, low byte
                MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetSensor(0) >> 8));      // Output raw data, high byte
                MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetAverage(0));           // Output average,  low byte
                MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetAverage(0) >> 8));     // Output average,  high byte
            #elif (MTOUCH_NUMBER_SENSORS == 2)
                if (mTouch_comm_outputMask & 0x01)
                {
                    MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetSensor(0));            // Output raw data, low byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetSensor(0) >> 8));      // Output raw data, high byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetAverage(0));           // Output average,  low byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetAverage(0) >> 8));     // Output average,  high byte
                }
                if (mTouch_comm_outputMask & 0x02)
                {
                    MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetSensor(1));            // Output raw data, low byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetSensor(1) >> 8));      // Output raw data, high byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetAverage(1));           // Output average,  low byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetAverage(1) >> 8));     // Output average,  high byte
                }
            #else
            bitCheck = 1;
            for (uint8_t i = 0; i < MTOUCH_NUMBER_SENSORS; i++)             // Output raw and average data
            {  
                if((mTouch_comm_outputMask & bitCheck) != 0)
                {
                    MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetSensor(i));            // Output raw data, low byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetSensor(i) >> 8));      // Output raw data, high byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t) mTouch_GetAverage(i));           // Output average,  low byte
                    MTOUCH_COMM_PUT_CHAR((uint8_t)(mTouch_GetAverage(i) >> 8));     // Output average,  high byte
                }
                bitCheck <<= 1;
            }
            #endif
        }
        
        if (checksum != 0)
        {
            if (!mTouch_state.commNewCommand)
            {
                MTOUCH_COMM_SEND_CHECKSUM();
            }
            else
            {
                mTouch_state.commNewCommand = 0;
                return;
            }
        }
        #endif
        
        mTouch_state.commStreaming      = 0;
    #else
    
    
        #if defined(MTOUCH_SLEEP_ENABLED) && defined(MTOUCH_ACTIVE_ENABLED)
        if (mTouch_state.sleep == 0)
        {
        #endif
        
            #if defined(MTOUCH_ACTIVE_ENABLED)

                #if (MTOUCH_NUMBER_SENSORS > 8)
                uint16_t buttonMask = 0;
                uint16_t bitCheck = 1;
                #elif (MTOUCH_NUMBER_SENSORS > 2)
                 uint8_t buttonMask = 0;
                 uint8_t bitCheck = 1;
                #endif
                
                
                #if (MTOUCH_NUMBER_SENSORS == 1)
                if (mTouch_GetButtonState(0) == MTOUCH_PRESSED)     {    str_Int(0x0001);   } else {    str_Int(0x0000);   }
                #elif (MTOUCH_NUMBER_SENSORS == 2)
                if (mTouch_GetButtonState(0) == MTOUCH_PRESSED)
                {
                    if (mTouch_GetButtonState(1) == MTOUCH_PRESSED) {    str_Int(0x0003);   } else {    str_Int(0x0001);   }
                } else {
                    if (mTouch_GetButtonState(1) == MTOUCH_PRESSED) {    str_Int(0x0002);   } else {    str_Int(0x0000);   }
                }
                #else
                for (uint8_t i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
                {
                    if (mTouch_GetButtonState(i) == MTOUCH_PRESSED)
                    {
                        buttonMask |= bitCheck;
                    }
                    bitCheck <<= 1;
                }
                str_Int((uint16_t)buttonMask);  // Output sensor state data
                #endif
                
            
                #if (MTOUCH_NUMBER_SENSORS == 1)
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    str_Int(mTouch_GetSensor(0));
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_GetAverage(0));
                    #endif
                #elif (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 2)
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    str_Int(mTouch_GetSensor(0));
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_GetAverage(0));
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    str_Int(mTouch_GetSensor(1));
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_GetAverage(1));
                    #endif
                #else
                // For all versions of the GUI, output the current reading values.
                for (uint8_t i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
                { 
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    str_Int(mTouch_GetSensor(i));
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_GetAverage(i));
                    #endif
                    
                }
                #endif
            
                #if defined(MTOUCH_SLIDERS_ENABLED) && defined(MTOUCH_V1_COMM_OUTPUT_SLIDERS)
                    #if (MTOUCH_NUMBER_OF_SLIDERS == 1)
                        str_Int(mTouch_slider[0]);
                    #elif (MTOUCH_NUMBER_OF_SLIDERS == 2)
                        str_Int(mTouch_slider[0]);
                        str_Int(mTouch_slider[1]);
                    #else                
                    for (uint8_t i = 0; i < MTOUCH_NUMBER_OF_SLIDERS; i++)
                    { 
                        str_Int(mTouch_slider[i]);
                    }
                    #endif
                    
                    
                    // #if (MTOUCH_NUMBER_SENSORS == 1)
                        // str_Int(mTouch_deltas[0]);
                    // #elif (MTOUCH_NUMBER_SENSORS == 2)
                        // str_Int(mTouch_deltas[0]);
                        // str_Int(mTouch_deltas[1]);
                    // #else    
                    // for (uint8_t i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
                    // { 
                        // str_Int(mTouch_deltas[i]);
                    // }
                    // #endif
                #endif
                
            #endif
            
        #if defined(MTOUCH_SLEEP_ENABLED) && defined(MTOUCH_ACTIVE_ENABLED)
        } else {
        #endif
        
            #if defined(MTOUCH_SLEEP_ENABLED)
            
                #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                mTouch_AcquisitionData* acqData;
                #endif
        
                #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 8)
                uint16_t buttonMask = 0;
                uint16_t bitCheck = 1;
                #elif (MTOUCH_NUMBER_OF_SLEEP_SENSORS > 2)
                 uint8_t buttonMask = 0;
                 uint8_t bitCheck = 1;
                #endif
                
                #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
                if (mTouch_Sleep_GetButtonState(0) > MTOUCH_SLEEP_RELEASED)     {    str_Int(0x0001);   } else {    str_Int(0x0000);   }
                #elif (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 2)
                if (mTouch_Sleep_GetButtonState(0) > MTOUCH_SLEEP_RELEASED)
                {
                    if (mTouch_Sleep_GetButtonState(1) > MTOUCH_SLEEP_RELEASED) {    str_Int(0x0003);   } else {    str_Int(0x0001);   }
                } else {
                    if (mTouch_Sleep_GetButtonState(1) > MTOUCH_SLEEP_RELEASED) {    str_Int(0x0002);   } else {    str_Int(0x0000);   }
                }
                #else
                for (uint8_t i = 0; i < MTOUCH_NUMBER_OF_SLEEP_SENSORS; i++)
                {
                    if (mTouch_Sleep_GetButtonState(i) > MTOUCH_SLEEP_RELEASED)
                    {
                        buttonMask |= bitCheck;
                    }
                    bitCheck <<= 1;
                }
                str_Int((uint16_t)buttonMask);  // Output sensor state data
                #endif
                
                // For all versions of the GUI, output the current reading values.
                #if (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 1)
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    acqData    = &mTouch_sleep_acqData[0];
                    str_Int(mTouch_Sleep_GetSensor());
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_Sleep_GetAverage(0));
                    #endif
                #elif (MTOUCH_NUMBER_OF_SLEEP_SENSORS == 2)
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    acqData    = &mTouch_sleep_acqData[0];
                    str_Int(mTouch_Sleep_GetSensor());
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_Sleep_GetAverage(0));
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    acqData    = &mTouch_sleep_acqData[1];
                    str_Int(mTouch_Sleep_GetSensor());
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_Sleep_GetAverage(1));
                    #endif
                #else
                for (uint8_t i = 0; i < MTOUCH_NUMBER_OF_SLEEP_SENSORS; i++)
                { 
                    
                    #if defined(MTOUCH_V1_COMM_OUTPUT_RAW)
                    acqData = &mTouch_sleep_acqData[i];
                    str_Int(mTouch_Sleep_GetSensor());
                    #endif
                    #if defined(MTOUCH_V1_COMM_OUTPUT_AVG)
                    str_Int(mTouch_Sleep_GetAverage(i));
                    #endif
                }
                #endif
                
            #endif
            
        #if defined(MTOUCH_SLEEP_ENABLED) && defined(MTOUCH_ACTIVE_ENABLED)
        }
        #endif
        // Carriage return, line feed
        put_Char(0x0D);
        put_Char(0x0A);
    #endif // end: !defined(MTOUCH_V2_COMMS)
}


#if defined(MTOUCH_V2_COMMS)
void mTouch_Comm_ReceiveData (void)
{
    uint8_t data = RCREG;
    
    if ((RCREG == 0x00) && (FERR == 1))
    {
        // Break Character Received. Start of new packet.
        mTouch_DisableScanning();
        
        mTouch_comm_state           = MTOUCH_COMM_START;
        mTouch_state.commInvalid         = 0;        
        return;
    }
    
    switch(mTouch_comm_state)
    {

        /**************************************************************
        * 1. Get the byte count to determine the expected packet length.
        **************************************************************/
    
        case MTOUCH_COMM_START:
        
            mTouch_comm_checksum        = 0;
            mTouch_comm_bufferIndex     = 0;
            mTouch_comm_bufferCounter   = data;                 // Store packet length
            mTouch_comm_state           = MTOUCH_COMM_BUFFER;   // Next state: Store incoming data in buffer
            
            break;
           
           
        /**************************************************************
        * 2. Knowing the packet length, buffer all incoming data.
        **************************************************************/
        
        case MTOUCH_COMM_BUFFER:
        
            mTouch_comm_buffer[mTouch_comm_bufferIndex] = data;
            mTouch_comm_bufferCounter--;
            
            if (OERR)       // If UART overrun has occurred, restart the UART and this state machine
            { 
                mTouch_state.commInvalid = 1;
                CREN = 0;
                asm("NOP");
                asm("NOP");
                CREN = 1;
                break;
            }
            else            // Everything good so far. 
            {
                if (mTouch_comm_bufferCounter == 0)
                {
                    mTouch_comm_bufferIndex = 0;
                    mTouch_comm_state       = MTOUCH_COMM_PROCESS_DATA;
                } 
                else
                {  
                    mTouch_comm_bufferIndex++;
                }
                mTouch_comm_checksum ^= data;
            }
            break;
        
        
        /**************************************************************
        * Something has gone wrong if we end up here...
        **************************************************************/

        case MTOUCH_COMM_IDLE:      // Break character not received first. Invalid data being received.
        default:                    // State variable invalid. Something has gone wrong.
        
            mTouch_state.commInvalid = 1; 
            break;
    }
    
    if (mTouch_state.commInvalid)
    {
        // Invalid packet received. Reset this state machine in the Idle state and re-enable mTouch scanning.
        mTouch_comm_bufferIndex = 0;
        mTouch_comm_state = MTOUCH_COMM_IDLE;
        mTouch_EnableScanning();
    }
    
    if (mTouch_comm_state == MTOUCH_COMM_PROCESS_DATA)
    {
        mTouch_Comm_ProcessData();
    }
}

void mTouch_Comm_ProcessData(void)
{
    uint8_t     checksum = 0;   // Used to calculate the checksum we will be sending to the PC.
    uint8_t     temp     = 0;   // Temporary variable to help with EEPROM manipulation.
    uint16_t    temp2    = 0;  

    __delay_ms(2);          // Delay required to ensure the PC's DLL starts a new packet
                            //  in case we interrupted a previous packet with this one.
    
    
    /**************************************************************
    * 3. Use the buffered data to evaluate the checksum
    **************************************************************/
    if (mTouch_comm_checksum != 0x00)
    {
        mTouch_state.commInvalid = 1;
    }
    else
    {
        /**************************************************************
        * 4. Pull all data from the buffer and execute the command.
        **************************************************************/
        
        switch(mTouch_comm_buffer[0] >> 4)      // Most significant nibble of the op code
        {                                       // (Minimizes execution time by reducing the number of required comparisons)
            
            // "A" OP codes: System Configuration Commands
            case MTOUCH_COMM_OPCODE_A:
            
                switch(mTouch_comm_buffer[0])
                {
                    // Send system configuration packet
                    case MTOUCH_COMM_READ_CONFIG:
                    
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_CONFIGURATION,8);  // OP code, Packet Length
                        MTOUCH_COMM_SEND_BYTE  (MTOUCH_FRAME_REVISION);
                        MTOUCH_COMM_SEND_BYTE  (MTOUCH_BOARD_REVISION);
                        MTOUCH_COMM_SEND_BYTE  (MTOUCH_SOFTWARE_REVISION);
                        MTOUCH_COMM_SEND_BYTE  (MTOUCH_NUMBER_SENSORS);             // Number of active mode sensors
                        MTOUCH_COMM_SEND_BYTE  (MTOUCH_NUMBER_OF_SLEEP_SENSORS);    // Number of sleep mode sensors (Not yet implemented)
                        MTOUCH_COMM_SEND_BYTE  (0);                                 // Algorithm id number (Not yet implemented)
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                        
                    // Restore the framework's configuration file settings
                    case MTOUCH_COMM_RESTORE_FAC_SETTINGS:
                    
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_START_ADR,                   MTOUCH_EEPROM_INIT_VALUE);               // Store 'initialized' byte
                        MTOUCH_EEACTIVE_INIT();
                        
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_ACK,2);    // Send 'acknowledge' packet to PC to verify the restore is complete.
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    default:
                        mTouch_state.commInvalid = 1;                    // Invalid op code.
                        break;
                }
                break;
                
            // "C" OP codes: Configure the active mode output stream
            case MTOUCH_COMM_OPCODE_C:
            
                switch(mTouch_comm_buffer[0])
                {
                
                    // Configure which sensors have their data output by the constant active stream.
                    case MTOUCH_COMM_CONFIG_RAW_ACTIVE:
                    
                        #if (MTOUCH_NUMBER_SENSORS <= 8)
                        mTouch_comm_outputMask = (uint8_t) mTouch_comm_buffer[1];   
                        #elif (MTOUCH_NUMBER_SENSORS <= 16)
                        mTouch_comm_outputMask = (uint16_t)mTouch_comm_buffer[1] | (uint16_t)(mTouch_comm_buffer[2] << 8);   
                        #elif (MTOUCH_NUMBER_SENSORS <= 24)
                        mTouch_comm_outputMask = (uint24_t)mTouch_comm_buffer[1] | (uint24_t)(mTouch_comm_buffer[2] << 8) | (uint24_t)(mTouch_comm_buffer[3] << 16);    
                        #elif (MTOUCH_NUMBER_SENSORS <= 32)
                        mTouch_comm_outputMask = (uint32_t)mTouch_comm_buffer[1] | (uint32_t)(mTouch_comm_buffer[2] << 8) | (uint32_t)(mTouch_comm_buffer[3] << 16) | (uint32_t)(mTouch_comm_buffer[4] << 24);   
                        #endif
                        
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_ACK,2);     // Send 'acknowledge' packet to PC to verify new output mask is now active.
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    case MTOUCH_COMM_CONFIG_MASK_ACTIVE:
                    
                        if (mTouch_comm_buffer[1] == 0x00)
                        {
                            mTouch_state.commButtonStateReq = 0;
                        }
                        else
                        {
                            mTouch_state.commButtonStateReq = 1;
                        }
                    
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_ACK,2);     // Send 'acknowledge' packet to PC to verify new output mask is now active.
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    default:
                        mTouch_state.commInvalid = 1;                    // Invalid op code.
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_NACK,2);    // Send 'NOT acknowledged' packet to PC.
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                }
                break;
                
               
            // "5" OP Codes: Write to an indexed value
            case MTOUCH_COMM_OPCODE_5:
            
                switch (mTouch_comm_buffer[0])
                {
                    case MTOUCH_COMM_WRITE_pTHRESHOLD:  
                        temp2 = ((uint16_t)(mTouch_comm_buffer[3] << 8) + mTouch_comm_buffer[2]);
                        temp2 *= MTOUCH_EEPROM_read(MTOUCH_EEPROM_rTHRESHper_ADR);
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTHRESH_ADRL + (mTouch_comm_buffer[1] * MTOUCH_EEPROM_SENS_SIZE), mTouch_comm_buffer[2]);
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTHRESH_ADRH + (mTouch_comm_buffer[1] * MTOUCH_EEPROM_SENS_SIZE), mTouch_comm_buffer[3]);
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESH_ADRL + (mTouch_comm_buffer[1] * MTOUCH_EEPROM_SENS_SIZE), (uint8_t)(temp2 & 0xFF));
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESH_ADRH + (mTouch_comm_buffer[1] * MTOUCH_EEPROM_SENS_SIZE), (uint8_t)(temp2 >> 8));
                        mTouch_state.commInvalid = 0; 
                        break;

                    default:    
                        mTouch_state.commInvalid = 1;    
                        break;
                }
                
                if (mTouch_state.commInvalid == 0)
                {
                    MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_ACK,2);     // Send 'acknowledge' packet verifying that we have saved the new indexed value.
                    MTOUCH_COMM_SEND_CHECKSUM();
                }
                else
                {
                    MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_NACK,2);    // Send 'NOT acknowledge' packet saying no change occurred.
                    MTOUCH_COMM_SEND_CHECKSUM();
                }

                break;
                
            // "6" OP Codes: Write to a non-indexed Value
            case MTOUCH_COMM_OPCODE_6:
            
                switch (mTouch_comm_buffer[0])
                {
                    case MTOUCH_COMM_WRITE_rTHRESHOLD_PER:
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESHper_ADR,  mTouch_comm_buffer[1]); 
                        // Update all sensors' released thresholds to match new release factor.  
                        for (int8_t i = MTOUCH_NUMBER_SENSORS-1; i >= 0; i--)
                        {
                            temp2 = mTouch_GetPressThreshold(i) * mTouch_comm_buffer[1];
                            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESH_ADRL + (i * MTOUCH_EEPROM_SENS_SIZE), (uint8_t)(temp2 & 0xFF));
                            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESH_ADRH + (i * MTOUCH_EEPROM_SENS_SIZE), (uint8_t)(temp2 >> 8));
                        }
                        break;
                        
                    case MTOUCH_COMM_WRITE_pDEBOUNCE: 
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_pDEB_ADRL,        mTouch_comm_buffer[1]); // No break - storing two bytes
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_pDEB_ADRH,        mTouch_comm_buffer[2]); 
                        break;
                        
                    case MTOUCH_COMM_WRITE_rDEBOUNCE: 
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_rDEB_ADRL,        mTouch_comm_buffer[1]); // No break - storing two bytes
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_rDEB_ADRH,        mTouch_comm_buffer[2]);
                        break;       
                        
                    case MTOUCH_COMM_WRITE_SAMPLESperSCAN:  
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_SpS_ADRL,         mTouch_comm_buffer[1]); // No break - storing two bytes
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_SpS_ADRH,         mTouch_comm_buffer[2]);
                        break;  
                        
                    case MTOUCH_COMM_WRITE_SCALING_FACTOR:   
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_SCALE_ADR,        mTouch_comm_buffer[1]);
                        break;     
                        
                    case MTOUCH_COMM_WRITE_pTIMEOUT:    
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTIMEOUT_ADRL,    mTouch_comm_buffer[1]); 
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTIMEOUT_ADRH,    mTouch_comm_buffer[2]);
                        break;
                        
                    case MTOUCH_COMM_WRITE_AVG_UPDATE:  
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_AVGUPDATE_ADR,    mTouch_comm_buffer[1]);
                        break;
                        
                    case MTOUCH_COMM_WRITE_AVG_RATE:     
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_AVGRATE_ADRL,     mTouch_comm_buffer[1]);
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_AVGRATE_ADRH,     mTouch_comm_buffer[2]);
                        break;         

                    case MTOUCH_COMM_WRITE_JITTER:              
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_JITTER_ADR,      mTouch_jitter_mask[mTouch_comm_buffer[1]]); 
                        break;
                        
                    case MTOUCH_COMM_WRITE_NEG_CAPACITANCE:     
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_NEGCAP_ADR,      mTouch_comm_buffer[1]); 
                        break;
                        
                    case MTOUCH_COMM_WRITE_BOARD_ID:     
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_BOARDID_ADR,     mTouch_comm_buffer[1]); 
                        break;
                        
                    case MTOUCH_COMM_WRITE_DEC_STEPSIZE:      
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_DECSTEP_ADRL,    mTouch_comm_buffer[1]); 
                        MTOUCH_EEPROM_write(MTOUCH_EEPROM_DECSTEP_ADRH,    mTouch_comm_buffer[2]);
                        break;            
                    
                    default: 
                        mTouch_state.commInvalid = 1;
                        break;
                }
                
                if (mTouch_state.commInvalid == 0)
                {
                    MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_ACK,2);     // Send 'acknowledge' packet verifying that we have saved the new indexed value.
                    MTOUCH_COMM_SEND_CHECKSUM();
                }
                else
                {
                    MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_NACK,2);    // Send 'NOT acknowledge' packet saying no change occurred.
                    MTOUCH_COMM_SEND_CHECKSUM();
                }
                break;
                
            // "8" OP Codes: Read indexed value
            case MTOUCH_COMM_OPCODE_8:
            
                MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);
                switch (mTouch_comm_buffer[0])
                {
                    case MTOUCH_COMM_READ_pTHRESHOLD:       
                        MTOUCH_COMM_READandSEND_EE(MTOUCH_EEPROM_pTHRESH_ADRL + (mTouch_comm_buffer[1] * MTOUCH_EEPROM_SENS_SIZE));
                        MTOUCH_COMM_READandSEND_EE(MTOUCH_EEPROM_pTHRESH_ADRH + (mTouch_comm_buffer[1] * MTOUCH_EEPROM_SENS_SIZE));
                        break;   
                        
                    default: break;
                }
                MTOUCH_COMM_SEND_CHECKSUM();
                break;
                
            // "9" OP Codes: Read non-indexed value
            case MTOUCH_COMM_OPCODE_9:
            
                switch (mTouch_comm_buffer[0])
                {
                    case MTOUCH_COMM_READ_rTHRESHOLD_PER: 
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_rTHRESHper_ADR);
                        MTOUCH_COMM_SEND_BYTE       (0x00);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    case MTOUCH_COMM_READ_pDEBOUNCE:
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);    
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_pDEB_ADRL);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_pDEB_ADRH);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    case MTOUCH_COMM_READ_rDEBOUNCE:   
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);        
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_rDEB_ADRL);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_rDEB_ADRH);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;   
                        
                    case MTOUCH_COMM_READ_SAMPLESperSCAN:  
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);         
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_SpS_ADRL);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_SpS_ADRH);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    case MTOUCH_COMM_READ_SCALING_FACTOR:  
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);       
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_SCALE_ADR);
                        MTOUCH_COMM_SEND_BYTE       (0x00);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    case MTOUCH_COMM_READ_pTIMEOUT:    
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);     
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_pTIMEOUT_ADRL);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_pTIMEOUT_ADRH);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;   
                        
                    case MTOUCH_COMM_READ_AVG_UPDATE:      
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);     
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_AVGUPDATE_ADR);
                        MTOUCH_COMM_SEND_BYTE       (0x00); 
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;   
                        
                    case MTOUCH_COMM_READ_AVG_RATE:  
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);            
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_AVGRATE_ADRL);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_AVGRATE_ADRH);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                        
                    case MTOUCH_COMM_READ_JITTER:    
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);    
                    
                        temp    = MTOUCH_EEPROM_read(MTOUCH_EEPROM_JITTER_ADR);
                        if (temp & 0xF0)
                        {
                                 if (temp == 0x3F)  {   temp = 6;   }
                            else if (temp == 0x1F)  {   temp = 5;   }
                            else if (temp == 0x7F)  {   temp = 7;   }
                            else                    {   temp = 8;   }
                        } else {
                                 if (temp == 0x03)  {   temp = 2;   }
                            else if (temp == 0x01)  {   temp = 1;   }
                            else if (temp == 0x07)  {   temp = 3;   }
                            else if (temp == 0x0F)  {   temp = 4;   }
                            else                    {   temp = 0;   }
                        }
                        checksum ^= temp;
                        put_Char(temp);
                        MTOUCH_COMM_SEND_BYTE(0x00);  
                        MTOUCH_COMM_SEND_CHECKSUM();                           
                        break;
                        
                    case MTOUCH_COMM_READ_NEG_CAPACITANCE: 
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);     
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_NEGCAP_ADR);
                        MTOUCH_COMM_SEND_BYTE       (0x00); 
                        MTOUCH_COMM_SEND_CHECKSUM();                           
                        break;
                    
                    case MTOUCH_COMM_READ_BOARD_ID:  
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);    
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_BOARDID_ADR);
                        MTOUCH_COMM_SEND_BYTE       (0x00); 
                        MTOUCH_COMM_SEND_CHECKSUM();                           
                        break;
                        
                    case MTOUCH_COMM_READ_DEC_STEPSIZE: 
                        MTOUCH_COMM_SEND_OPCODE(MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE,4);            
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_DECSTEP_ADRL);
                        MTOUCH_COMM_READandSEND_EE  (MTOUCH_EEPROM_DECSTEP_ADRH);
                        MTOUCH_COMM_SEND_CHECKSUM();
                        break;
                           
                    default: break;
                }
                break;
                
                
            default: break;
        }
    }
    
    if (mTouch_state.commStreaming)
    {
        mTouch_state.commNewCommand     = 1;
    }
    mTouch_comm_state               = MTOUCH_COMM_IDLE;
    __delay_ms(1);                                                      // Why?
    mTouch_EnableScanning();
    
    
}

#endif // end: MTOUCH_V2_COMMS


///////////////////////////////////////////////////////////////////////////////
//*****************************************************************************
// Communications Support functions
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////

/****************************************************************
* Function: void str_Int(uint16_t i)
*
* Overview: This function sends an INT as a space and 5 digits
*
* Input:    Din is the value to be converted
*           base is the radix of the value
*
* Output:   None
*
****************************************************************/
/**
*   @brief Sends an integer through the serial port
*
*   @param i The integer to send
*/
void str_Int(uint16_t i)
{
	uint8_t ctr = 0;
    
    #if defined(MTOUCH_DEBUG_OUTPUT_HEX)
    
        // Hex Output
        ctr = (uint8_t)(i >> 12);       if (ctr <= 9) { put_Char(ctr + 0x30); } else { put_Char(ctr + 0x37); }
        ctr = (uint8_t)(i >> 8) & 0x0F; if (ctr <= 9) { put_Char(ctr + 0x30); } else { put_Char(ctr + 0x37); }
        ctr = (uint8_t)(i >> 4) & 0x0F; if (ctr <= 9) { put_Char(ctr + 0x30); } else { put_Char(ctr + 0x37); }
        ctr = (uint8_t) i       & 0x0F; if (ctr <= 9) { put_Char(ctr + 0x30); } else { put_Char(ctr + 0x37); }
    
    #else
    
        // Decimal Output
        while (i >= 10000)  { i -= 10000; ctr++; } put_Char(ctr + 0x30); ctr = 0;
        while (i >=  1000)  { i -=  1000; ctr++; } put_Char(ctr + 0x30); ctr = 0;
        while (i >=   100)  { i -=   100; ctr++; } put_Char(ctr + 0x30); ctr = 0;
        while (i >=    10)  { i -=    10; ctr++; } put_Char(ctr + 0x30); ctr = 0;
        while (i >=     1)  { i -=     1; ctr++; } put_Char(ctr + 0x30);
    
    #endif
    
    // A semi-colon delimiter is used for the GUI. This can be changed if
    // you are not using the GUI to process the output.
    put_Char(MTOUCH_V1_COMM_DELIMITER);
}


/**
*   @brief Sends a single character through the serial port or bit-bangs
*          a UART implementation if no UART is available.
*
*   @param output The character to send
*/
void put_Char(uint8_t output)
{
    #if !defined(MTOUCH_V1_COMM_BITBANGED)
        while(MTOUCH_DEBUG_COMM_TXIF == 0);     // Hardware RS-232 Implementation
        MTOUCH_DEBUG_COMM_TXREG = output;
    #else
        MTOUCH_COMM_232_TRANSMIT();             // Software RS-232 Implementation :: "includes/mTouch_comm.h"
        MTOUCH_COMM_232_BITDELAY();
    #endif
}

#if defined(MTOUCH_V2_COMMS)
uint8_t v2Comm_putChar(uint8_t output)
{
    if(!mTouch_state.commNewCommand)                
    {                                               
        while(MTOUCH_DEBUG_COMM_TXIF == 0);
        MTOUCH_DEBUG_COMM_TXREG = output;                          
        return output;                          
    }                                               
    else                                            
    {                                               
        mTouch_state.commNewCommand = 0;      
        return 0;
    }                                               
}

uint8_t v2Comm_sendOpcode(uint8_t value, uint8_t size)
{
    SENDB   = 1;                                    
    TXREG   = size;                                 
    while(SENDB);                                   
    put_Char(size);                                 
    put_Char(value);                                
    return value;                              
}
#endif
#endif