/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_macroLibrary_common.h
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Hardware Profile Header File
 *                   - Editting this file will make customer support for
 *                     your application extremely difficult. Be careful,
 *                     as it can also have a negative impact on your
 *                     system's noise susceptibility.
 *************************************************************************/
 /***********************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file   mTouch_macroLibrary_common.h
*   @brief  Implements non-core-specific acquisition macros
*/

/**
* @def      MASKBANK(var,bank)
* @param    var     the variable address to mask
* @param    bank    the bank location of the variable
* @brief    Masks the bank out of the variable address for ASM code usage. 
*           Requires that the actual bank bits being masked equal the bank 
*           value provided.
* @hideinitializer
*/
#define MASKBANK(var,bank)      ((var)^((bank)*80h))
/**
* @def      NOBANK(var)
* @param    var     the variable address to mask
* @brief    Masks the bank out of the variable address for ASM code usage.
* @hideinitializer
*/
#define NOBANK(var)             (var & 0x7F)

/** @name Housekeeping Macros
*
*   These macros perform basic ISR and timer module functions such as checking
*   that the interrupt flag is set and reloading the TMR0 counter.
*/
//@{


/**
* @def          SAVE_STATE()
* @brief        Executes any required actions to save the current main-loop 
*               process
* @hideinitializer
*/
#define SAVE_STATE()                    \
    do {                                \
        asm("movwf  _int_w");           \
        asm("swapf  _int_w, F");        \
        asm("movf   _STATUS, W");       \
        asm("clrf   _STATUS");          \
        asm("movwf  _int_status");      \
        asm("movf   _PCLATH, W");       \
        asm("clrf   _PCLATH");          \
        asm("movwf  _int_pclath");      \
        asm("movf   _FSR, W");          \
        asm("movwf  _int_fsr");         \
    } while (0)
// The Enhanced Midrange Core does not require saving/restoring state on entering
// the ISR, so these macros will be undefined later for those parts. 
// They remain implemented in the non-enhanced core version.

/**
* @def          RESTORE_STATE()
* @brief        Executes any required actions to save the current main-loop process
* @hideinitializer
*/
#define RESTORE_STATE()                 \
    do {                                \
        asm("movf   _int_fsr,W");       \
        asm("movwf  _FSR");             \
        asm("movf   _int_pclath,W");    \
        asm("movwf  _PCLATH");          \
        asm("movf   _int_status,W");    \
        asm("movwf  _STATUS");          \
        asm("swapf  _int_w, W");        \
    } while (0)
// The Enhanced Midrange Core does not require saving/restoring state on entering
// the ISR, so these macros will be undefined later for those parts. 
// They remain implemented in the non-enhanced core version.

/**
* @def          CHECK_AND_CLEAR_TMRxIF()
* @brief        Checks the assigned mTouch ISR timer's interrupt flag and determines 
*               whether the mTouch ISR is executed
* @hideinitializer
*/
#define CHECK_AND_CLEAR_TMRxIF()        \
do {                                    \
    if (MTOUCH_ISR_TMRxIF == 0)         \
    {                                   \
        asm("ljmp END_MTOUCH_SCAN");    \
    } else {                            \
        MTOUCH_ISR_TMRxIF = 0;          \
    }                                   \
} while (0)

/**
* @def          MTOUCH_SLEEP_STABILIZE_OSC()
* @brief        Causes the system to wait until the oscillator's frequency has stabilized
* @hideinitializer
*/
#define MTOUCH_SLEEP_STABILIZE_OSC()            \
    do {                                        \
        while (!HFIOFL);                        \
    } while(0)


#if defined(MTOUCH_JITTER_ENABLE)

    #if !defined(MTOUCH_V2_COMMS)
        /**
        * @def      JITTER_START_TIME()
        * @brief    Calculates a random value and uses it to seed the mTouch interrupt timer
        *
        *   Implements a linear feedback shift register algorithm to increase the randomness 
        *   of the jitter function. This implementation costs one byte of RAM.
        *
        * @hideinitializer
        */
        #define JITTER_START_TIME()                                                                                             \
            do {                                                                                                                \
                asm("BANKSEL    _mTouch_jitter"                     );  /* Make sure we're starting in the correct bank     */  \
                asm("bcf    "   ___mkstr(_STATUS)           ",0"    );  /* Clear the carry bit                              */  \
                asm("rrf    "   ___mkstr(_mTouch_jitter)    ",W"    );  /* Right shift the current jitter seed value        */  \
                asm("btfsc  "   ___mkstr(_STATUS)           ",0"    );  /* Check the carry bit - if set, perform XOR        */  \
                asm("xorlw      0xB4"                               );  /* (cond) XOR the jitter seed with 0xB4             */  \
                asm("movwf  "   ___mkstr(_mTouch_jitter)            );  /* Store the result as the new jitter seed value    */  \
                asm("andlw  "   ___mkstr(MTOUCH_JITTER_MASK)        );  /* Mask the seed value to limit the number of bits  */  \
                asm("BANKSEL "  ___mkstr(MTOUCH_ISR_TMR_ASM)        );  /* Move to Bank 0 to access timer SFR               */  \
                asm("clrf   "   ___mkstr(NOBANK(MTOUCH_ISR_TMR_ASM)));  /* Clear timer                                      */  \
                asm("addwf  "   ___mkstr(NOBANK(MTOUCH_ISR_TMR_ASM)));  /* Add the masked LFSR value as an offset to timer  */  \
            } while (0)

            
        /**
        * @def      JITTER_MAIN_LOOP() 
        * @brief    Calculates a random value and uses it to jitter the mTouch scanning 
        *           routine when operating out of the main loop.
        *
        *   Implements a linear feedback shift register algorithm to increase the randomness 
        *   of the jitter function. This implementation costs one byte of RAM.
        *
        * @hideinitializer
        */
        #define JITTER_MAIN_LOOP()                                                                                          \
            do {                                                                                                            \
                asm("BANKSEL    _mTouch_jitter"                 );  /* Make sure we're starting in the correct bank     */  \
                asm("bcf    "   ___mkstr(_STATUS)           ",0");  /* Clear the carry bit                              */  \
                asm("rrf    "   ___mkstr(_mTouch_jitter)    ",W");  /* Right shift the current jitter seed value        */  \
                asm("btfsc  "   ___mkstr(_STATUS)           ",0");  /* Check the carry bit - if set, perform XOR        */  \
                asm("xorlw      0xB4"                           );  /* (cond) XOR the jitter seed with 0xB4             */  \
                asm("movwf  "   ___mkstr(_mTouch_jitter)        );  /* Store the result as the new jitter seed value    */  \
                for (uint8_t i = (mTouch_jitter & MTOUCH_JITTER_MASK); i > 0; i--); /* Delay loop                       */  \
            } while (0)
    
    #else
    
        #define JITTER_START_TIME()                                                                 \
            do {                                                                                    \
                if (mTouch_jitter & 0x01) {                                                         \
                    mTouch_jitter = (uint8_t)(mTouch_jitter >> 1) ^ 0xB4;                           \
                } else {                                                                            \
                    mTouch_jitter = (uint8_t)(mTouch_jitter >> 1);                                  \
                }                                                                                   \
                MTOUCH_ISR_TMR_C = mTouch_jitter & MTOUCH_EEPROM_read(MTOUCH_EEPROM_JITTER_ADR);    \
            } while(0)
            
        #define JITTER_MAIN_LOOP()                                          \
            do {                                                            \
                if (mTouch_jitter & 0x01) {                                 \
                    mTouch_jitter = (uint8_t)(mTouch_jitter >> 1) ^ 0xB4;   \
                } else {                                                    \
                    mTouch_jitter = (uint8_t)(mTouch_jitter >> 1);          \
                }                                                           \
                for (uint8_t i = mTouch_jitter & MTOUCH_EEPROM_read(MTOUCH_EEPROM_JITTER_ADR); i > 0; i--); \
            } while(0)
            
    #endif
        
#else                                                       // Do not implement the jittering function
    #define JITTER_START_TIME()     \
        do {                        \
            MTOUCH_ISR_TMR_C = 0;   \
        } while(0) 
    #define JITTER_MAIN_LOOP()
#endif

/**
* @def          WAIT_FOR_GODONE_BIT()
* @brief        Implements a while loop waiting on the ADC's GO/DONE bit to be cleared by hardware.
* @hideinitializer
*/
#define WAIT_FOR_GODONE_BIT()           while (GO_nDONE);  


/**
* @def      MTOUCH_STORE_SCAN_A()
* @brief    Stores the result of the active-mode 'scan a' in the mTouch_lastResult variable.
* @hideinitializer
**/
/**
* @def      MTOUCH_STORE_SCAN_B()
* @brief    Stores the result of the active-mode 'scan b' in the mTouch_lastResult variable.
* @hideinitializer
*/
/**
* @def      MTOUCH_SLEEP_STORE_SCAN_A()
* @brief    Stores the result of the sleep-mode 'scan a' in the mTouch_sleep_lastResult variable.
* @hideinitializer
**/
/**
* @def      MTOUCH_SLEEP_STORE_SCAN_B()
* @brief    Stores the result of the sleep-mode 'scan b' in the mTouch_sleep_lastResult variable.
* @hideinitializer
*/
#if defined(MTOUCH_ADC_BITS) && (MTOUCH_ADC_BITS == 8)
    #define MTOUCH_STORE_SCAN_A()           mTouch_lastResult =            ADRES;
    #define MTOUCH_STORE_SCAN_B()           mTouch_lastResult = (uint16_t)(ADRES | 0x0100) - mTouch_lastResult;
    
    #define MTOUCH_SLEEP_STORE_SCAN_A()     mTouch_sleep_lastResult =            ADRES;
    #define MTOUCH_SLEEP_STORE_SCAN_B()     mTouch_sleep_lastResult = (uint16_t)(ADRES | 0x0100) - mTouch_sleep_lastResult;
#else
    #define MTOUCH_STORE_SCAN_A()           mTouch_lastResult = (uint16_t) (ADRESH         << 8) + ADRESL;
    #define MTOUCH_STORE_SCAN_B()           mTouch_lastResult = (uint16_t)((ADRESH | 0x04) << 8) + ADRESL - mTouch_lastResult;
    
    #define MTOUCH_SLEEP_STORE_SCAN_A()     mTouch_sleep_lastResult = (uint16_t) (ADRESH         << 8) + ADRESL;
    #define MTOUCH_SLEEP_STORE_SCAN_B()     mTouch_sleep_lastResult = (uint16_t)((ADRESH | 0x04) << 8) + ADRESL - mTouch_sleep_lastResult;
#endif
//@}

/** @name   Timing Critical CVD Scanning Routine Macros
*
*   These macros implement the CVD scanning routine and basic acquistion-level filtering 
*   techniques. Editting these macros could reduce the noise immunity of the system.
*/
//@{


/**
* @def          MTOUCH_READ_SENSOR(indexSensor, indexRef)
* @param[in]    indexSensor     the index of the sensor to be scanned
* @param[in]    indexRef        the index of the sensor to be used as the reference
* @brief        Abstraction macro used to alternate between implementing CTMU and
*               CVD, based on the current configuration of the mTouch framework.
* @hideinitializer
*/
#define MTOUCH_READ_SENSOR(indexSensor, indexRef)           CVD_READ_SENSOR(indexSensor,indexRef)

/**
* @def          MTOUCH_SLEEP_READ_SENSOR(indexSensor, indexRef)
* @param[in]    indexSensor     the index of the sensor to be scanned
* @param[in]    indexRef        the index of the sensor to be used as the reference
* @brief        Abstraction macro used to alternate between implementing CTMU and
*               CVD, based on the current configuration of the mTouch framework.
* @hideinitializer
*/
#define MTOUCH_SLEEP_READ_SENSOR(indexSensor, indexRef)     CVD_SLEEP_READ_SENSOR(indexSensor,indexRef)


/**
* @def          CVD_READ_SENSOR(indexSensor, indexRef)
* @param[in]    indexSensor     the index of the sensor to be scanned
* @param[in]    indexRef        the index of the sensor to be used as the reference
* @brief        Utility macro called by MTOUCH_READ_SENSOR(). Do not use directly.
*
*   This is the full implementation routine for the CVD acquisition. The algorithm steps
*   will changed based on the processor core being used, but the general steps should
*   remain the same:
*
* @section      ReadSensorExpanded      Reference-Sensor CVD Scanning Algorithm Steps:
*   <b>1. Scan A - Sensor High, Reference Low</b>
*   @li To begin, ADCON0 is set so that the mux will point to the reference sensor.
*       The reference sensor is already output low. 
*   @li The reference sensor is then set low (just in case) and the sensor to be scanned
*       is set high.
*   @li The new ADCON0 value which will point to the scanning sensor is loaded into
*       the working register and a short delay is implemented to allow the internal hold
*       capacitor enough time to discharge.
*   @li The sensor's port is set as an input and ADCON0 is immediately loaded with
*       the new mux value.
*   @li After a short delay to allow the charges to settle, the Go/Done bit is set to
*       begin the ADC conversion. The sensor port is then set back to an output to
*       minimize noise.
*
* <b>2. Execute Math while ADC Performs Conversion</b>
*   @li The CVD_DECIMATE(), MTOUCH_SENSOR_SMALLER(), and CVD_DECIMATE_END() macros are
*       now used to perform calculations based on the previous ISR's results. The result
*       of Scan A is also stored into the mTouch_lastResult variable to later have Scan B's 
*       result subtracted from it.
*
* <b>3. Scan B - Sensor Low, Reference High</b><br>
*       The exact process as Scan A is repeated, but with the voltage levels of the
*       sensor and the reference reversed.
*   @li ADCON0 mux is set to the reference sensor.
*   @li Reference sensor is set to output high, sensor is output low.
*   @li After a short delay, sensor is set to an input.
*   @li ADCON0 mux is set to the sensor.
*   @li After a short settling delay, the Go/Done bit is set.
*   @li Sensor is set to an output.
*
* <b>4. The ISR then exits and stores the ADC result at the start of the next ISR.</b>
*
* @section ReadSensorExpanded1Sensor Single-Sensor CVD Scanning Algorithm Steps:
*   Since we are now going to be using the same sensor to charge the internal ADC
*   capacitor and perform our scan, the steps are going to change slightly. The
*   overall, algorithmic process is the same as above, however. First we scan with
*   the internal hold capacitor as a high reference, then we perform math on the 
*   previous ISR's result, and then we scan in the opposite direction - internal 
*   hold capacitor is low, sensor is high.
*   @li Sensor is output low to begin
*   @li ADCON0 is set to point to the sensor and the Go/Done bit is set. This
*       provides a very small window that allows the internal hold capacitor to
*       gain some charge from the external sensor. Since the Go/Done bit is set at
*       the same time, the internal hold capacitor will be disconnected from the 
*       sensor.
*   @li The ADCON0 register is then set to point to an isolated, unimplemented
*       ADC channel option and the Go/Done bit is cleared. This stops the conversion
*       that just begin and isolates the now-charged hold capacitor from the sensor.
*   @li The sensor is then set output low to remove all charge.
*   @li The sensor is set to an input and ADCON0 is set to point to the sensor.
*   @li After some settling time, the ADCON0 register then points to an isolated
*       sensor option and begins the conversion.
*
* @hideinitializer
*/


/**
* @def          CVD_RS_START_CRITICAL()
* @brief        Starts the mTouch critical scanning portion of code
* @hideinitializer
*/
/**
* @def          CVD_RS_END_CRITICAL(index)
* @param[in]    index           the index of the sensor to be scanned
* @brief        Ends the mTouch critical scanning portion of code. If an interrupt occured mid-scan, repeat the scan.
* @hideinitializer
*/
#if defined(MTOUCH_ACTIVE_RUNS_AS_MAINLOOP)
    #define CVD_RS_START_CRITICAL()                     mTouch_state.isrServiced = 0
    #define CVD_RS_END_CRITICAL(index)                          \
        if (mTouch_state.isrServiced)                           \
        {                                                       \
            mTouch_state.isRepeatScan = 1;                      \
            asm("ljmp " ___mkstr(__paste(sensor_,index)));      \
        }
    // asm("ljmp " ___mkstr(__paste(sensor_,index)));           TRANSLATES TO "ljmp sensor_0" (or similar)
#else
    #define CVD_RS_START_CRITICAL()
    #define CVD_RS_END_CRITICAL(index)
#endif


/**
* @def          CVD_S_RS_START_CRITICAL()
* @brief        Starts the mTouch critical scanning portion of code
* @hideinitializer
*/
/**
* @def          CVD_S_RS_END_CRITICAL(index)
* @param[in]    index           the index of the sensor to be scanned
* @brief        Ends the mTouch critical scanning portion of code. If an interrupt occured mid-scan, repeat the scan.
* @hideinitializer
*/
#if defined(MTOUCH_SLEEP_SCAN_INTERRUPTABLE)
    #define CVD_S_RS_START_CRITICAL()                   mTouch_state.isrServiced = 0
    #define CVD_S_RS_END_CRITICAL(index)                            \
        if (mTouch_state.isrServiced)                               \
        {                                                           \
            mTouch_state.isRepeatScan = 1;                          \
            asm("ljmp " ___mkstr(__paste(sleep_sensor_,index)));    \
        }
    // asm("ljmp " ___mkstr(__paste(sleep_sensor_,index)));     TRANSLATES TO "ljmp sleep_sensor_0" (or similar)
#else
    #define CVD_S_RS_START_CRITICAL()
    #define CVD_S_RS_END_CRITICAL(index)
#endif
    
//@}


/** @name       Polishing/Finishing Macros
*
*   These macros perform scaling and final operations before setting the 
*   #mTouchCVD_dataReady flag and allowing the main loop application to process
*   the new data.
*/
//@{

/**
* @def          STORE_SENSOR_SCALE(index, shift)
* @brief        Do not use directly. Expanded utility macro called by STORE_SENSOR(index). 
* @param[in]    index   the index of the sensor to be scaled and stored
* @param[in]    shift   the number of times to shift the accumulator value to the right before storing its value in the mTouch_sensorData array
* @hideinitializer
*/
#define STORE_SENSOR_SCALE(index, shift)                                                        \
    do {                                                                                        \
        data    = &mTouch_acqData[index];                                                       \
        mTouch_sensorData[index-1] = (uint16_t)(mTouch_GetAccumulator(data) >> shift);          \
        mTouch_ClearAccumulator(data);                                                          \
    } while(0)
    
/**
* @def          STORE_LAST_SENSOR_SCALE(index, shift)
* @brief        Do not use directly. Expanded utility macro called by STORE_SENSOR(index). 
* @param[in]    index   the index of the sensor to be scaled and stored
* @param[in]    shift   the number of times to shift the accumulator value to the right before storing its value in the mTouch_sensorData array
* @hideinitializer
*/
#define STORE_LAST_SENSOR_SCALE(index, shift)                                                               \
    do {                                                                                                    \
        data    = &mTouch_acqData[index];                                                                   \
        mTouch_sensorData[MTOUCH_NUMBER_SENSORS-1] = (uint16_t)(mTouch_GetAccumulator(data) >> shift);      \
        mTouch_ClearAccumulator(data);                                                                      \
    } while(0)
    
/**
* @def          STORE_SINGLE_SENSOR_SCALE(index, shift)
* @brief        Do not use directly. Expanded utility macro called by STORE_SENSOR(index) if there is only one sensor in the mTouch application. 
* @param[in]    index   the index of the sensor to be scaled and stored
* @param[in]    shift   the number of times to shift the accumulator value to the right before storing its value in the mTouch_sensorData array
* @hideinitializer
*/
#define STORE_SINGLE_SENSOR_SCALE(index, shift)                                         \
    do {                                                                                \
        mTouch_sensorData[0] = (uint16_t)(mTouch_GetAccumulator(data) >> shift);        \
        mTouch_ClearAccumulator(data);                                                  \
    } while(0)

/**
* @def          STORE_SENSOR(index)
* @ingroup      Acquisition
* @param[in]    index   the index of the sensor to be scaled and stored
* @brief        Scales the accumulated result of the CVD acquisition and stores it for main-application access
*
*   This macro uses the MTOUCH_SCALING configuration option to determine which scaling option to choose.
* 
* @hideinitializer
*/
#if (MTOUCH_NUMBER_SENSORS > 1)
    #if (MTOUCH_SCALING >= 0) && (MTOUCH_SCALING <= 8)
        #define STORE_SENSOR(index)             STORE_SENSOR_SCALE(index, MTOUCH_SCALING_VALUE) 
        #define STORE_LAST_SENSOR(index)        STORE_LAST_SENSOR_SCALE(index, MTOUCH_SCALING_VALUE) 
    #else
        #error MTOUCH_SCALING must be set to a value between 0 and 8.
    #endif
#else
    #if (MTOUCH_SCALING >= 0) && (MTOUCH_SCALING <= 8)
        #define STORE_SENSOR(index)             STORE_SINGLE_SENSOR_SCALE(index, MTOUCH_SCALING_VALUE) 
    #else
        #error MTOUCH_SCALING must be set to a value between 0 and 8.
    #endif
#endif

/**
* @def          EXIT_SENSOR(index)
* @ingroup      Acquisition
* @param[in]    index   the index variable to increment
* @brief        Increments the given index variable and exits the ISR
* @hideinitializer
*/
#if (MTOUCH_ADC_CONTROL == MTOUCH_ALWAYS_CONTROLS_ADC)
    #define EXIT_SENSOR(index)                                  \
        do {                                                    \
            index++;                                            \
            asm("goto END_MTOUCH_SCAN");    /* Exit the ISR */  \
        } while(0)
#else
    #define EXIT_SENSOR(index)                                  \
        do {                                                    \
            index++;                                            \
            WAIT_FOR_GODONE_BIT();                              \
            MTOUCH_STORE_SCAN_B();                              \
            asm("goto END_MTOUCH_SCAN");    /* Exit the ISR */  \
        } while(0)
#endif

/**
* @def          EXIT_LAST_SENSOR(index)
* @ingroup      Acquisition
* @param[in]    index   the index variable to reset to 0
* @brief        Performs several important functions after all sensors have been scanned
*
*   Resets the index variable to 0 for the next scan, decrements the oversampling
*   counter to determine if it is time to complete the sample and store the result,
*   and then exits the ISR.
*
* @hideinitializer
*/
/**
* @def          EXIT_LAST_SENSOR_DECINDEX(index) 
* @param[in]    index   the index variable to reset to 0
* @brief        Do not use directly. Utility macro used by EXIT_LAST_SENSOR() to reset the index variable.
* @hideinitializer
*/
/**
* @def          EXIT_LAST_SENSOR_MAIN()
* @brief        Do not use directly. Utility macro used by EXIT_LAST_SENSOR() to decrement the oversampling counter. 
* @hideinitializer
*/
#if (MTOUCH_ADC_CONTROL == MTOUCH_ALWAYS_CONTROLS_ADC)
    #if (MTOUCH_NUMBER_SENSORS > 1)
        #define EXIT_LAST_SENSOR(index)     EXIT_LAST_SENSOR_DECINDEX(index);   \
                                            EXIT_LAST_SENSOR_MAIN()
    #else
        #define EXIT_LAST_SENSOR(index)     EXIT_LAST_SENSOR_MAIN()             // Only one sensor, so only updates the sample counter
    #endif
#else
    #if (MTOUCH_NUMBER_SENSORS > 1)
        #define EXIT_LAST_SENSOR(index)     EXIT_LAST_SENSOR_DECINDEX(index);   \
                                            WAIT_FOR_GODONE_BIT();              \
                                            MTOUCH_STORE_SCAN_B();              \
                                            EXIT_LAST_SENSOR_MAIN()
    #else
        #define EXIT_LAST_SENSOR(index)     WAIT_FOR_GODONE_BIT();      \
                                            MTOUCH_STORE_SCAN_B();      \
                                            EXIT_LAST_SENSOR_MAIN()                                     
    #endif
#endif

#define EXIT_LAST_SENSOR_MAIN()                                                                                                                 \
do {                                                                                                                                            \
    if (--mTouch_stateVars.active.sampleCounter != 0) {                 /* Decrement the sample counter and check if it's equal to 0        */  \
        asm("goto END_MTOUCH_SCAN");                                    /* If not equal to 0, exit the ISR                                  */  \
    }                                                                                                                                           \
    mTouch_stateVars.active.sampleCounter = MTOUCH_SAMPLES_PER_SCAN;    /* If equal to 0, reset the sample counter continue with storage    */  \
} while(0)
    
#define EXIT_LAST_SENSOR_DECINDEX(index)        index = 0;

#define MTOUCH_PERFORM_SCAN(index)              MTOUCH_READ_SENSOR(index, __paste(MTOUCH_INDEXMINUSONE_,index))

#define MTOUCH_SLEEP_PERFORM_SCAN(index)        MTOUCH_SLEEP_READ_SENSOR(index, __paste(MTOUCH_SLEEP_INDEXMINUSONE_,index))

#define MTOUCH_EXIT_SCAN_FUNCTION(index)        __3paste(MTOUCH_EXIT_SENSOR,index,())

#define MTOUCH_INC_INDEX_AND_STORE_RESULT()                         \
                mTouch_currentSensor++;                             \
                data    = &mTouch_acqData[mTouch_currentSensor];    \
                result  = mTouch_GetResult(data);                   \
                WAIT_FOR_GODONE_BIT();                              \
                MTOUCH_STORE_SCAN_B();
                
#define MTOUCH_SLEEP_INCINDEX_STORE_RESULT(index)   data = &mTouch_sleep_acqData[index];    \
                                                    result  = mTouch_GetResult(data);       \
                                                    WAIT_FOR_GODONE_BIT();                  \
                                                    MTOUCH_SLEEP_STORE_SCAN_B();

/**
* @def          SET_DATA_READY_FLAG()
* @ingroup      Acquisition
* @brief        Sets the dataReady flag to signal the main application of a new reading
* @hideinitializer
*/
#define SET_DATA_READY_FLAG()               mTouch_state.dataReady = 1
//@}


/**
* @def          GENERATE_JUMP_TABLE()
* @ingroup      Acquisition
* @brief        Generates a jump table in ASM used by the acquisition ISR. Do not use directly.
* @hideinitializer
*/
/**
* @def          GENERATE_STORAGE_FUNCTION()
* @ingroup      Acquisition
* @brief        Generates the storage macro calls for the acquisition ISR. Do not use directly.
* @hideinitializer
*/
#if (MTOUCH_NUMBER_SENSORS == 1)

    #define GENERATE_JUMP_TABLE()       \
        do {                            \
            asm("global   sensor_0");   \
            asm("goto     sensor_0");   \
        } while (0)
     
    #define GENERATE_STORAGE_FUNCTION()             STORE_SENSOR(0);   
    
#else

    #define CVD_ASM_JUMP_LABEL_1()      case 1:     asm("goto sensor_1");
    #define CVD_ASM_JUMP_LABEL_2()      case 2:     asm("goto sensor_2");
    #define CVD_ASM_JUMP_LABEL_3()      case 3:     asm("goto sensor_3");
    #define CVD_ASM_JUMP_LABEL_4()      case 4:     asm("goto sensor_4");
    #define CVD_ASM_JUMP_LABEL_5()      case 5:     asm("goto sensor_5");
    #define CVD_ASM_JUMP_LABEL_6()      case 6:     asm("goto sensor_6");
    #define CVD_ASM_JUMP_LABEL_7()      case 7:     asm("goto sensor_7");
    #define CVD_ASM_JUMP_LABEL_8()      case 8:     asm("goto sensor_8");
    #define CVD_ASM_JUMP_LABEL_9()      case 9:     asm("goto sensor_9");
    #define CVD_ASM_JUMP_LABEL_10()     case 10:    asm("goto sensor_10");
    #define CVD_ASM_JUMP_LABEL_11()     case 11:    asm("goto sensor_11");
    #define CVD_ASM_JUMP_LABEL_12()     case 12:    asm("goto sensor_12");
    #define CVD_ASM_JUMP_LABEL_13()     case 13:    asm("goto sensor_13");
    #define CVD_ASM_JUMP_LABEL_14()     case 14:    asm("goto sensor_14");
    #define CVD_ASM_JUMP_LABEL_15()     case 15:    asm("goto sensor_15");
    #define CVD_ASM_JUMP_LABEL_16()     case 16:    asm("goto sensor_16");
    #define CVD_ASM_JUMP_LABEL_17()     case 17:    asm("goto sensor_17");
    #define CVD_ASM_JUMP_LABEL_18()     case 18:    asm("goto sensor_18");
    #define CVD_ASM_JUMP_LABEL_19()     case 19:    asm("goto sensor_19");
    #define CVD_ASM_JUMP_LABEL_20()     case 20:    asm("goto sensor_20");
    #define CVD_ASM_JUMP_LABEL_21()     case 21:    asm("goto sensor_21");
    #define CVD_ASM_JUMP_LABEL_22()     case 22:    asm("goto sensor_22");
    #define CVD_ASM_JUMP_LABEL_23()     case 23:    asm("goto sensor_23");
    #define CVD_ASM_JUMP_LABEL_24()     case 24:    asm("goto sensor_24");
    #define CVD_ASM_JUMP_LABEL_25()     case 25:    asm("goto sensor_25");
    #define CVD_ASM_JUMP_LABEL_26()     case 26:    asm("goto sensor_26");
    #define CVD_ASM_JUMP_LABEL_27()     case 27:    asm("goto sensor_27");
    #define CVD_ASM_JUMP_LABEL_28()     case 28:    asm("goto sensor_28");
    #define CVD_ASM_JUMP_LABEL_29()     case 29:    asm("goto sensor_29");

    #define CVD_STORE_LABEL_0()         STORE_LAST_SENSOR(0);
    #define CVD_STORE_LABEL_1()         STORE_SENSOR(1);
    #define CVD_STORE_LABEL_2()         STORE_SENSOR(2);
    #define CVD_STORE_LABEL_3()         STORE_SENSOR(3);
    #define CVD_STORE_LABEL_4()         STORE_SENSOR(4);
    #define CVD_STORE_LABEL_5()         STORE_SENSOR(5);
    #define CVD_STORE_LABEL_6()         STORE_SENSOR(6);
    #define CVD_STORE_LABEL_7()         STORE_SENSOR(7);
    #define CVD_STORE_LABEL_8()         STORE_SENSOR(8);
    #define CVD_STORE_LABEL_9()         STORE_SENSOR(9);
    #define CVD_STORE_LABEL_10()        STORE_SENSOR(10);
    #define CVD_STORE_LABEL_11()        STORE_SENSOR(11);
    #define CVD_STORE_LABEL_12()        STORE_SENSOR(12);
    #define CVD_STORE_LABEL_13()        STORE_SENSOR(13);
    #define CVD_STORE_LABEL_14()        STORE_SENSOR(14);
    #define CVD_STORE_LABEL_15()        STORE_SENSOR(15);
    #define CVD_STORE_LABEL_16()        STORE_SENSOR(16);
    #define CVD_STORE_LABEL_17()        STORE_SENSOR(17);
    #define CVD_STORE_LABEL_18()        STORE_SENSOR(18);
    #define CVD_STORE_LABEL_19()        STORE_SENSOR(19);
    #define CVD_STORE_LABEL_20()        STORE_SENSOR(20);
    #define CVD_STORE_LABEL_21()        STORE_SENSOR(21);
    #define CVD_STORE_LABEL_22()        STORE_SENSOR(22);
    #define CVD_STORE_LABEL_23()        STORE_SENSOR(23);
    #define CVD_STORE_LABEL_24()        STORE_SENSOR(24);
    #define CVD_STORE_LABEL_25()        STORE_SENSOR(25);
    #define CVD_STORE_LABEL_26()        STORE_SENSOR(26);
    #define CVD_STORE_LABEL_27()        STORE_SENSOR(27);
    #define CVD_STORE_LABEL_28()        STORE_SENSOR(28);
    #define CVD_STORE_LABEL_29()        STORE_SENSOR(29);

    #if (MTOUCH_NUMBER_SENSORS > 30)
        #error This version of the framework macro library unable to support more than 30 sensors at this time.
    #endif
    
    #if MTOUCH_NUMBER_SENSORS < 30
        #undef  CVD_ASM_JUMP_LABEL_29()
        #define CVD_ASM_JUMP_LABEL_29()
        #undef  CVD_STORE_LABEL_29()
        #define CVD_STORE_LABEL_29()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 29
        #undef  CVD_ASM_JUMP_LABEL_28()
        #define CVD_ASM_JUMP_LABEL_28()
        #undef  CVD_STORE_LABEL_28()
        #define CVD_STORE_LABEL_28() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 28
        #undef  CVD_ASM_JUMP_LABEL_27()
        #define CVD_ASM_JUMP_LABEL_27()
        #undef  CVD_STORE_LABEL_27()
        #define CVD_STORE_LABEL_27()  
    #endif
    #if MTOUCH_NUMBER_SENSORS < 27
        #undef  CVD_ASM_JUMP_LABEL_26()
        #define CVD_ASM_JUMP_LABEL_26()
        #undef  CVD_STORE_LABEL_26()
        #define CVD_STORE_LABEL_26() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 26
        #undef  CVD_ASM_JUMP_LABEL_25()
        #define CVD_ASM_JUMP_LABEL_25()
        #undef  CVD_STORE_LABEL_25()
        #define CVD_STORE_LABEL_25() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 25 
        #undef  CVD_ASM_JUMP_LABEL_24()
        #define CVD_ASM_JUMP_LABEL_24()
        #undef  CVD_STORE_LABEL_24()
        #define CVD_STORE_LABEL_24()  
    #endif
    #if MTOUCH_NUMBER_SENSORS < 24
        #undef  CVD_ASM_JUMP_LABEL_23()
        #define CVD_ASM_JUMP_LABEL_23()
        #undef  CVD_STORE_LABEL_23()
        #define CVD_STORE_LABEL_23()   
    #endif
    #if MTOUCH_NUMBER_SENSORS < 23
        #undef  CVD_ASM_JUMP_LABEL_22()
        #define CVD_ASM_JUMP_LABEL_22()
        #undef  CVD_STORE_LABEL_22()
        #define CVD_STORE_LABEL_22()   
    #endif
    #if MTOUCH_NUMBER_SENSORS < 22
        #undef  CVD_ASM_JUMP_LABEL_21()
        #define CVD_ASM_JUMP_LABEL_21()
        #undef  CVD_STORE_LABEL_21()
        #define CVD_STORE_LABEL_21()   
    #endif
    #if MTOUCH_NUMBER_SENSORS < 21
        #undef  CVD_ASM_JUMP_LABEL_20()
        #define CVD_ASM_JUMP_LABEL_20()
        #undef  CVD_STORE_LABEL_20()
        #define CVD_STORE_LABEL_20()  
    #endif
    #if MTOUCH_NUMBER_SENSORS < 20
        #undef  CVD_ASM_JUMP_LABEL_19()
        #define CVD_ASM_JUMP_LABEL_19()
        #undef  CVD_STORE_LABEL_19()
        #define CVD_STORE_LABEL_19()    
    #endif
    #if MTOUCH_NUMBER_SENSORS < 19
        #undef  CVD_ASM_JUMP_LABEL_18()
        #define CVD_ASM_JUMP_LABEL_18()
        #undef  CVD_STORE_LABEL_18()
        #define CVD_STORE_LABEL_18()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 18
        #undef  CVD_ASM_JUMP_LABEL_17()
        #define CVD_ASM_JUMP_LABEL_17()
        #undef  CVD_STORE_LABEL_17()
        #define CVD_STORE_LABEL_17()   
    #endif
    #if MTOUCH_NUMBER_SENSORS < 17
        #undef  CVD_ASM_JUMP_LABEL_16()
        #define CVD_ASM_JUMP_LABEL_16()
        #undef  CVD_STORE_LABEL_16()
        #define CVD_STORE_LABEL_16()  
    #endif
    #if MTOUCH_NUMBER_SENSORS < 16
        #undef  CVD_ASM_JUMP_LABEL_15()
        #define CVD_ASM_JUMP_LABEL_15()
        #undef  CVD_STORE_LABEL_15()
        #define CVD_STORE_LABEL_15()  
    #endif
    #if MTOUCH_NUMBER_SENSORS < 15
        #undef  CVD_ASM_JUMP_LABEL_14()
        #define CVD_ASM_JUMP_LABEL_14()
        #undef  CVD_STORE_LABEL_14()
        #define CVD_STORE_LABEL_14()   
    #endif
    #if MTOUCH_NUMBER_SENSORS < 14
        #undef  CVD_ASM_JUMP_LABEL_13()
        #define CVD_ASM_JUMP_LABEL_13()
        #undef  CVD_STORE_LABEL_13()
        #define CVD_STORE_LABEL_13()  
    #endif
    #if MTOUCH_NUMBER_SENSORS < 13
        #undef  CVD_ASM_JUMP_LABEL_12()
        #define CVD_ASM_JUMP_LABEL_12()
        #undef  CVD_STORE_LABEL_12()
        #define CVD_STORE_LABEL_12() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 12
        #undef  CVD_ASM_JUMP_LABEL_11()
        #define CVD_ASM_JUMP_LABEL_11()
        #undef  CVD_STORE_LABEL_11()
        #define CVD_STORE_LABEL_11() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 11
        #undef  CVD_ASM_JUMP_LABEL_10()
        #define CVD_ASM_JUMP_LABEL_10()
        #undef  CVD_STORE_LABEL_10()
        #define CVD_STORE_LABEL_10()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 10
        #undef  CVD_ASM_JUMP_LABEL_9()
        #define CVD_ASM_JUMP_LABEL_9()
        #undef  CVD_STORE_LABEL_9()
        #define CVD_STORE_LABEL_9() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 9
        #undef  CVD_ASM_JUMP_LABEL_8()
        #define CVD_ASM_JUMP_LABEL_8()
        #undef  CVD_STORE_LABEL_8()
        #define CVD_STORE_LABEL_8() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 8
        #undef  CVD_ASM_JUMP_LABEL_7()
        #define CVD_ASM_JUMP_LABEL_7()
        #undef  CVD_STORE_LABEL_7()
        #define CVD_STORE_LABEL_7()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 7
        #undef  CVD_ASM_JUMP_LABEL_6()
        #define CVD_ASM_JUMP_LABEL_6()
        #undef  CVD_STORE_LABEL_6()
        #define CVD_STORE_LABEL_6()   
    #endif
    #if MTOUCH_NUMBER_SENSORS < 6
        #undef  CVD_ASM_JUMP_LABEL_5()
        #define CVD_ASM_JUMP_LABEL_5()
        #undef  CVD_STORE_LABEL_5()
        #define CVD_STORE_LABEL_5()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 5
        #undef  CVD_ASM_JUMP_LABEL_4()
        #define CVD_ASM_JUMP_LABEL_4()
        #undef  CVD_STORE_LABEL_4()
        #define CVD_STORE_LABEL_4() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 4
        #undef  CVD_ASM_JUMP_LABEL_3()
        #define CVD_ASM_JUMP_LABEL_3()
        #undef  CVD_STORE_LABEL_3()
        #define CVD_STORE_LABEL_3()    
    #endif
    #if MTOUCH_NUMBER_SENSORS < 3
        #undef  CVD_ASM_JUMP_LABEL_2()
        #define CVD_ASM_JUMP_LABEL_2()
        #undef  CVD_STORE_LABEL_2()
        #define CVD_STORE_LABEL_2() 
    #endif
    #if MTOUCH_NUMBER_SENSORS < 2
        #undef  CVD_ASM_JUMP_LABEL_1()
        #define CVD_ASM_JUMP_LABEL_1()
        #undef  CVD_STORE_LABEL_1()
        #define CVD_STORE_LABEL_1() 
    #endif

    #define GENERATE_JUMP_TABLE()                   \
        do {                                        \
            MTOUCH_ASM_MACRO_GLOBAL_SCAN_LABELS();  \
            switch (mTouch_currentSensor)           \
            {                                       \
                case 0: asm("goto sensor_0");       \
                CVD_ASM_JUMP_LABEL_1();             \
                CVD_ASM_JUMP_LABEL_2();             \
                CVD_ASM_JUMP_LABEL_3();             \
                CVD_ASM_JUMP_LABEL_4();             \
                CVD_ASM_JUMP_LABEL_5();             \
                CVD_ASM_JUMP_LABEL_6();             \
                CVD_ASM_JUMP_LABEL_7();             \
                CVD_ASM_JUMP_LABEL_8();             \
                CVD_ASM_JUMP_LABEL_9();             \
                CVD_ASM_JUMP_LABEL_10();            \
                CVD_ASM_JUMP_LABEL_11();            \
                CVD_ASM_JUMP_LABEL_12();            \
                CVD_ASM_JUMP_LABEL_13();            \
                CVD_ASM_JUMP_LABEL_14();            \
                CVD_ASM_JUMP_LABEL_15();            \
                CVD_ASM_JUMP_LABEL_16();            \
                CVD_ASM_JUMP_LABEL_17();            \
                CVD_ASM_JUMP_LABEL_18();            \
                CVD_ASM_JUMP_LABEL_19();            \
                CVD_ASM_JUMP_LABEL_20();            \
                CVD_ASM_JUMP_LABEL_21();            \
                CVD_ASM_JUMP_LABEL_22();            \
                CVD_ASM_JUMP_LABEL_23();            \
                CVD_ASM_JUMP_LABEL_24();            \
                CVD_ASM_JUMP_LABEL_25();            \
                CVD_ASM_JUMP_LABEL_26();            \
                CVD_ASM_JUMP_LABEL_27();            \
                CVD_ASM_JUMP_LABEL_28();            \
                CVD_ASM_JUMP_LABEL_29();            \
                default: break;                     \
            }                                       \
        } while (0)


    #define GENERATE_STORAGE_FUNCTION()         \
        do {                                    \
            CVD_STORE_LABEL_0();                \
            CVD_STORE_LABEL_1();                \
            CVD_STORE_LABEL_2();                \
            CVD_STORE_LABEL_3();                \
            CVD_STORE_LABEL_4();                \
            CVD_STORE_LABEL_5();                \
            CVD_STORE_LABEL_6();                \
            CVD_STORE_LABEL_7();                \
            CVD_STORE_LABEL_8();                \
            CVD_STORE_LABEL_9();                \
            CVD_STORE_LABEL_10();               \
            CVD_STORE_LABEL_11();               \
            CVD_STORE_LABEL_12();               \
            CVD_STORE_LABEL_13();               \
            CVD_STORE_LABEL_14();               \
            CVD_STORE_LABEL_15();               \
            CVD_STORE_LABEL_16();               \
            CVD_STORE_LABEL_17();               \
            CVD_STORE_LABEL_18();               \
            CVD_STORE_LABEL_19();               \
            CVD_STORE_LABEL_20();               \
            CVD_STORE_LABEL_21();               \
            CVD_STORE_LABEL_22();               \
            CVD_STORE_LABEL_23();               \
            CVD_STORE_LABEL_24();               \
            CVD_STORE_LABEL_25();               \
            CVD_STORE_LABEL_26();               \
            CVD_STORE_LABEL_27();               \
            CVD_STORE_LABEL_28();               \
            CVD_STORE_LABEL_29();               \
        } while (0)
        
#endif

