/**
 * @brief A bit-bang UART routine for the PICtail device which has an IrDA transciever
 *  and encoder/decoder
 *
 *  The encoder/decoder chip requires a 16x clock for correct operation
 *  (i.e baud rate * 16 = Fclk) => 9600*16 = 153.6KHz
 *
 *  MODEL #: TFDU6300 (transciever)
 *  MODEL #: MCP2122  (encoder/decoder)
 *
 *  @file OnePIC_8bit_demo_irda.c
 *  @defgroup irda IrDA PICtail
 *  @ingroup eight_bit_demo
 *  @{
 * 
 *
 */
#include <htc.h>   //PIC specific hardware
#include <stdio.h> //Used for printf and sprintf

#include "bsp.h"   //XTAL Frequ
#include "OnePIC_8bit_demo_irda.h" //prototypes

/**
 * @brief bit-bang IR byte
 * @details  lets just use a baud rate of 9600 since this is bit-banged! A higher baudrate may be achievable,
 *        no guarentee that the data will be accurate
 * @param c char byte to send
 *
 * @note writing delay loops directly in 'C' is a very bad idea! The actual delay will depend
 * on the accuracy of the compiler. Use a predefined macro instead or use in-line assembly!
 */
void irda_putch(char c) {
    unsigned char i;

    IRDA_TX = 0; ///start bit is always a 0
    for (i = NUM_DATA_BITS + 1; i != 0; i--) {
        __delay_us(BIT_UDLY);
        if (c & 1)
            IRDA_TX = 1;
        if (!(c & 1))
            IRDA_TX = 0;
        c = (c >> 1) | 0x80;
    }
    IRDA_TX = 1; ///stop bit always a 1

}

/**
 * @brief receives one byte
 * @details  lets just use a baud rate of 9600 since this is bit-banged! A higher baudrate may be achievable,
 *        no guarentee that the data will be accurate
 * @ingroup ISR
 * @return char received byte
 *
 * @note writing delay loops directly in 'C' is a very bad idea! The actual delay will depend
 * on the accuracy of the compiler. Use a predefined macro instead or use in-line assembly!
 */
char irda_getch(void) {
    char c = 0; //returned character received
    uint8_t i;

    //Worst case is that start bit is caught 20us from inital start, depending on rate of sampling start bit
    //Want to sample in the middle( 104us/2)

    __delay_us(5);
    if (IRDA_RX)
        return 0; // twas just noise
    for(i=NUM_DATA_BITS+1;i!=0;i--){
        c = (c >> 1) | (IRDA_RX << 7); //LSB is transmitted first
        __delay_us(BIT_UDLY);
    }
    return c;   //return the char
}

/**
 * @brief echos back whatever received
 * @return char received byte
 */
char irda_getche(void) {
    char c;

    c = irda_getch();
    irda_putch(c);
    return c;
}
/** @}*/