/**
 *
 * @file OnePIC_8bit_demo_main.c
 * @brief Main application for the IrDA demo on the 8-bit PIC16LF1939
 * @defgroup eight_bit_demo 8bit Demo
 * @ingroup eight_bit
 * @{
 * 
 */
#include <htc.h>   //PIC specific hardware defintions
#include <string.h> //Used for sprintf
#include <stdio.h> //Used for sprintf

#include "OnePIC_int.h"
#include "OnePIC_timer.h"
#include "OnePIC_lcd.h"
#include "OnePIC_led.h"
#include "OnePIC_switch.h"
#include "Compiler.h"
#include "mTouchCVD.h"
#include "TimeDelay.h"

//Custom Demo Routines
#include "OnePIC_8bit_demo_isr.h"
#include "OnePIC_8bit_demo_mtouch.h"
#include "OnePIC_8bit_demo_main.h"
#include "OnePIC_8bit_demo_mcp2200.h"
#include "OnePIC_8bit_demo_irda.h"

/**
 *
 * @struct IR_DEMO_RX
 * @ingroup global
 * @struct IR_DEMO_FLAG
 * @ingroup global
 * 
 */
IR_DEMO_FLAG rx_tx_flag; /**< Keep track of rx/tx flags */
IR_DEMO_RX rx_byte;  /**< store all rx/tx bytes in here */




/**
 * @brief IR main loop that sends and receives bytes
 */
void demo_main_loop(void) {
    uint8_t status_led = 0; // status_led used as a delay for the status led @note this is hardcode and dependent on Fosc */
    uint8_t blink_tx = 0; // blink_tx blink tx status LED this many times */
    uint8_t blink_rx = 0; // blink_rx blink rx status LED this many times */
    uint16_t counter = 0; // counter hardcoded delay routine @note this is depeendent on Fosc */


    irda_demo_init();

    while (1) {
        if (counter++ == 100) { //hardcoded delay loop to blink TX/RX status LEDs
            counter = 0;
            if (blink_tx != 0) {
                gpLED_1 ^= 1; //blink RX LED
                blink_tx--;
            }
            if (blink_rx != 0) {
                gpLED_2 ^= 1; //blink TX LED
                blink_rx--;
            }
            if (blink_tx == 0)
                gpLED_1 = 0; //Reset
            if (blink_rx == 0)
                gpLED_2 = 0; //Reset

            if (status_led++ == 90) {
                gpLED_6 = 1; //set the "I'm Alive" LED indicator
            }
        }
        if (status_led == 95) { //only allow this LED to be on for a short while
            status_led = 0; //clear the "I'm Alive" LED indicator
            gpLED_6 = 0;
        }

        if (rx_tx_flag.TOUCH_TX_FLAG) { //was a button pressed?
            touch_service();
            blink_tx = 5;

        }
        if (rx_tx_flag.MCP2200_RX_FLAG) { //was something received from USB?
            mcp2200_service();
            blink_tx = 5;
            blink_rx = 5;
        }
        if (rx_tx_flag.IRDA_RX_FLAG) { //was someting received from the PICtail?
            ir_service();
            blink_tx = 5;
            blink_rx = 5;
        }
        Demo_mTouchPoll(); //keep on polling the buttons
    }
}

/**
 * @brief Service the TOUCH_TX_FLAG
 * - Print the received byte to the LCD
 * - echo back the received byte
 * - send it the IR PICtail
 * - clear flag
 * @note Up, Down, Left, Right, and Enter will print U, D, L, R, E respefuly on the LCD
 */
void touch_service() {
    mcp2200_tx(rx_byte.touch_tx); //send the button press byte to the PC
    irda_tx(rx_byte.touch_tx);   //send the button press byte to the IR link
    if (rx_byte.rx_tx_old_byte != rx_byte.touch_tx) { //Only enter here to refresh the LCD....don't want to update unless new byte sent
        EDF_INT_Disable(); //disable interrupts before updating or else risk failing the write
        sprintf(LCDBuffer[0], "PC-RX:  IR-RX: ");
        sprintf(LCDBuffer[1], "PC-TX:%c IR-TX:%c", rx_byte.touch_tx, rx_byte.touch_tx);
        rx_byte.rx_tx_old_byte = rx_byte.touch_tx; //save the current byte for checking next time through
        LCDCommand(CLEAR_DISPLAY);
        LCDUpdate(); //write both LCD buffers
        EDF_INT_Enable(); //Enable again now that it is safe
    }
    rx_tx_flag.TOUCH_TX_FLAG = false; //Don't enter here again unless a touch button is pressed again
}

/**
 * @brief Service the MCP2200_RX_FLAG
 * - Print the received byte to the LCD
 * - echo back the received byte
 * - send it the IR PICtail
 * - clear flag
 */
void mcp2200_service() {

    mcp2200_tx(rx_byte.mcp2200_rx); //echo received byte back
    irda_tx(rx_byte.mcp2200_rx); //send received byte to IR channel
    if (rx_byte.rx_tx_old_byte != rx_byte.mcp2200_rx) { //Only enter here to refresh the LCD....don't want to update unless new byte sent
        sprintf(LCDBuffer[0], "PC-RX:%c IR-RX: ", rx_byte.mcp2200_rx); //send and receive
        sprintf(LCDBuffer[1], "PC-TX:%c IR-TX:%c", rx_byte.mcp2200_rx, rx_byte.mcp2200_rx);
        rx_byte.rx_tx_old_byte = rx_byte.mcp2200_rx;
        EDF_INT_Disable();
        LCDCommand(CLEAR_DISPLAY);
        LCDUpdate(); //update the LCD
        EDF_INT_Enable();
    }
    rx_tx_flag.MCP2200_RX_FLAG = false; //don't enter here agin until new data received
}

/**
 * @brief Service the IRDA_RX_FLAG
 * - Print the received byte to the LCD
 * - send the received byte to the MCP2200
 * - clear flag
 */
void ir_service() {

    mcp2200_tx(rx_byte.irda_rx);
    if (rx_byte.rx_tx_old_byte != rx_byte.irda_rx) {
        sprintf(LCDBuffer[0], "PC-RX:  IR-RX:%c", rx_byte.irda_rx);
        sprintf(LCDBuffer[1], "PC-TX:%c IR-TX: ", rx_byte.irda_rx);
        rx_byte.rx_tx_old_byte = rx_byte.irda_rx;
        EDF_INT_Disable();
        LCDCommand(CLEAR_DISPLAY);
        LCDUpdate();
        EDF_INT_Enable();
    }
    rx_tx_flag.IRDA_RX_FLAG = false; ///don't enter here agin until new data received
}

/**
 * @brief Initilizes the IR link, specifically:
 * - 16X Clock for the MCP2122
 * - TMR4 for UART start bit from the IR transciever
 * - Display splash screen
 */
void irda_demo_init(void) {

    //0x20 is ASCII for blank space
    rx_byte.irda_rx = 0x20;
    rx_byte.mcp2200_rx = 0x20;
    rx_byte.touch_tx = 0x20;

    EDF_pot2bar(0); //disable the POT
    mcp2200_init(); //Initilize the onboard hardware UART to communicate with MCP2200
    CCPTMRS0 = 0b00001000; //CCP2 => TMR6 || CCP4 => TMR4

    //SETUP 16x clock for MCP2122
    EDF_TMR_ClockSourceSelect(TMR_6, TMR_SOURCE_INS_CLK);
    EDF_TMR_PrescalerSet(TMR_6, TMR246_PRESCALE_1_4);
    EDF_INT_SourceFlagClear(INT_TMR_6); //Don't enable the interrupt flag - let the module take care of it
    EDF_TMR_CounterSet(TMR_6,12); //Must be 16 * (baud rate).
    CCPR2L = 6;
    CCP2CON = 0b00101100;


    //use TIMR4 for IRDA_RX polling for the START bit
    EDF_TMR_PrescalerSet(TMR_4, TMR246_PRESCALE_1_1);
    EDF_INT_SourceEnable(INT_TMR_4); //enter the ISR when this rolls over to check IrDA RX pin for start bit (LOW when starting transmission)
    EDF_INT_SourceFlagClear(INT_TMR_4);
    EDF_TMR_CounterSet(TMR_4,50); //@todo document this better

    EDF_INT_Disable();
    LCDCommand(CLEAR_DISPLAY);
    sprintf(LCDBuffer[0], " IR Demo Active");
    LCDUpdate();
    EDF_INT_Enable();

    IRDA_16X_CLK_TRIS = 0; //Clear clock output
    IRDA_TX = 1; //IrDA devices transmit when TX is LOW..so set HIGH to avoid this
    EDF_TMR_Start(TMR_4); //Start the start bit polling
    EDF_TMR_Start(TMR_6); //Start the PWM module for the MCP2200 CLK
}

/**
 * @brief bit-bang a byte to the IR PICtail
 * @param c char byte to send to PICtail
 */
void irda_tx(char c) {
    EDF_INT_Disable();
    irda_putch(c);
    EDF_INT_Enable();
}
/**
 * @brief send a byte to the UART module back to the listening device
 * @param c char byte to send to UART
 */
void mcp2200_tx(char c) {
    EDF_INT_Disable();
    putch(c); //echo it back to PC just for fun
    EDF_INT_Enable();

}
/** @}*/
