/**
 * @file OnePIC_i2c.c
 * @author Larry Gass
 * @date 2010-10-11
 * @brief Performs all I2C Master related tasks such as:
 *  - init
 *  - tx
 *  - rx
 * 
 * @defgroup i2c I2C
 * @ingroup base 
 * @{
 *
 */

#include "Compiler.h"
#include "bsp.h"
#include "TimeDelay.h"
#include "OnePIC_i2c.h"

/**
 * @var bus1
 * @brief global variables that are used to init the i2c bus
 */
struct i2cBus bus1 = {
	I2C_IDLE,
	0,// no stopLock
	0,// offset 0
	(void *)0,
	&bus1.req
};
	
/**
 *
 * @param bus i2c statemachine
 * @return the state of the I2C of Idle
 */
int i2cIdle(struct i2cBus *bus)
{
	return bus->mstate == I2C_IDLE;
}
#if defined __PIC24F__
#define __I2C1_ISR    __attribute__((interrupt, shadow, auto_psv)) /**<@brief  */
/**
 * @brief service i2c interrupt for PIC24
 */
void __I2C1_ISR _MI2C1Interrupt(void)
#elif defined __PICC__
/**
 * @brief service the i2c interrupt for PIC16
 */
void ISR_i2c(void)
#elif defined(__PIC32MX__)
/**
 * @brief service the i2c interrupt for PIC32
 */
void ISR_i2c(void)
#endif
{
	struct trb *req = bus1.req;

   	I2C_INTERRUPT_FLAG = 0;		// Clear Interrupt Flag
	if (req)
	{
	    // Clear flag
		if (I2C_BCL_MASTER_BUS_COLLISION)
		{
			bus1.mstate = I2C_STARTING;	// try again from scratch
			bus1.bufOffset = 0;
			I2C_BCL_MASTER_BUS_COLLISION = 0;
			I2C_SEN_START_CONDITION_ENABLE = 1;
		}
		else
		{
    		Nop();
			switch (bus1.mstate)
			{
				case I2C_IDLE:
					bus1.mstate = I2C_ERRORS;
					break;
				case I2C_STARTING:
					bus1.bufOffset = 0;
					if (req->txLen)
					{
						bus1.mstate = I2C_XMITING;
						I2C_TRANSMIT_REGISTER = (req->addr << 1);
					}
					else if (req->rxLen)
					{
						bus1.mstate = I2C_RECEIVEADDR;
						I2C_TRANSMIT_REGISTER = (req->addr << 1) | 1;
					}
					else
					{
						bus1.mstate = I2C_ERRORS;
					}
					break;
				case I2C_RESTARTING:
					bus1.mstate = I2C_RECEIVEADDR;
					I2C_TRANSMIT_REGISTER = (req->addr << 1) | 1;
					break;
				case I2C_XMITING:
					req->status =  I2C_ACK_STATUS;
					if (bus1.bufOffset < req->txLen)
					{
						I2C_TRANSMIT_REGISTER = (req->txBuf[bus1.bufOffset++]);
					}
					else if (req->rxLen)
					{
						bus1.mstate = I2C_RESTARTING;
						I2C_REPEATED_START_ENABLE = 1;
					}
					else
					{
						bus1.mstate = I2C_STOPPING;
						Delay10us(5);		// WARNING! : Delay in interrupt handler
						I2C_STOP_CONDITION_ENABLE = 1;
					}
					break;
				case I2C_RECEIVEADDR:
					req->status = I2C_ACK_STATUS;
					if (req->status)	// NACKed
					{
						bus1.mstate = I2C_STOPPING;
						I2C_STOP_CONDITION_ENABLE = 1;
					}
					else	// ACKed
					{
						bus1.mstate = I2C_RECEIVING;
						bus1.bufOffset = 0;
						I2C_RECIEVE_ENABLE = 1;
					}
					break;
				case I2C_RECEIVING:
					bus1.mstate = I2C_ACKING;
					req->rxBuf[bus1.bufOffset++] = I2C_RECIEVE_REGISTER;
					I2C_ACK_DATA = (bus1.bufOffset < req->rxLen) ? 0 : 1;
				    I2C_ACK_SEQ_ENABLE = 1;
					break;
				case I2C_ACKING:
					if (bus1.bufOffset < req->rxLen)
					{
						bus1.mstate = I2C_RECEIVING;
						I2C_RECIEVE_ENABLE = 1;
					}
					else
					{
						bus1.mstate = I2C_STOPPING;
						I2C_STOP_CONDITION_ENABLE = 1;
					}
					break;
				case I2C_STOPPING:
					if ((req->status & 1) && req->retries)
					{
						--req->retries;
						bus1.mstate     = I2C_STARTING;
						I2C_SEN_START_CONDITION_ENABLE = 1;
					}
					else
					{
						req->status |= 0x80;	// set done bit
						if (bus1.stopLock)
						{
							bus1.mstate = I2C_STOP_IN_ENQ;
						}
						else if (req->next)
						{
							bus1.mstate     = I2C_STARTING;
							bus1.req        = req->next;
							I2C_SEN_START_CONDITION_ENABLE = 1;
						}
						else
						{
							bus1.mstate = I2C_IDLE;
						}
					}
					break;
				default:
					break;
		
			}
		}
	}

}




/**
 * @brief Initializes the I2C hardware and data structures
 * @param bus i2c bus status
 * @param req i2c bus properties
 */
void trbEnQ(struct i2cBus *bus, struct trb *req)
{
	req->status = 0;
        struct trb *reqTemp;
	if (bus->mstate == I2C_IDLE)
	{
		bus->mstate     = I2C_STARTING;
		bus->req        = req;
		bus->tail       = &req->next;
		I2C_SEN_START_CONDITION_ENABLE = 1;
	}
	else
	{
		bus->stopLock   = 1;
		*bus->tail = req;			// add to queue
		bus->tail = &req->next;		// advance the tail pointer
		bus->stopLock = 0;
		if (bus->mstate == I2C_STOP_IN_ENQ)	// pretend we're in handler
		{
                    reqTemp = bus->req->next;
                    bus->mstate     = I2C_STARTING;
                    bus->req        = reqTemp;
//                        bus->req = 2;
                    I2C_SEN_START_CONDITION_ENABLE = 1;
		}
	}
}
/**
 * @brief init the I2C according to each architecture
 */
void I2CInit (void)
{
	I2C_INTERRUPT_FLAG = 0;	// clear interrupt
	I2C_INTERRUPT_ENABLE = 1;	// enable I2C1 master interrupts
        //I2C master :: clock = Fosc / (4 * (SSPADD + 1))E
#ifdef __PICC__
        I2C_MODULE_SETUP_MASTER_3 = 1;
        I2C_MODULE_SETUP_MASTER_2 = 0;
        I2C_MODULE_SETUP_MASTER_1 = 0;
        I2C_MODULE_SETUP_MASTER_0 = 0;
#endif
#ifdef __PIC32MX__
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
    INTEnableInterrupts();
    INTSetVectorPriority(INT_I2C_1_VECTOR, INT_PRIORITY_LEVEL_4);
    INTClearFlag(INT_I2C1);
    INTClearFlag(INT_I2C1B);
    INTClearFlag(INT_I2C1M);
    INTEnable(INT_I2C1, INT_ENABLED);
    INTEnable(INT_I2C1B, INT_ENABLED);
    INTEnable(INT_I2C1M, INT_ENABLED);
#endif // __PIC32mx__

	I2C_BRG = 0x27;//GetPeripheralClock() / I2CFREQ - 2;
	I2C_MODULE_ENABLE = 1;	// enable I2C1
	I2C_DISABLE_SLEW_CONTROL = 1;
}
/** @}*/
