/**
 * @file OnePIC_lcd.c
 * @brief Delegates all LCD routines. The LCD is written to in 4 bit mode via 4 pins on each micro.
 * @defgroup lcd LCD
 * @ingroup base
 * @{
 * @note LCD Model #: TSB1G7000 based on NT7603 controller
 */


#include <stdio.h>
#include "OnePIC_lcd.h"
#include "TimeDelay.h"
#include "bsp.h"
#include "OnePIC_int.h" //for disabling interrupts when writing to LCD


char LCDBuffer[LCD_LINES][LCD_LINE_LENGTH];

/**
 * @brief toggles enable pin for LCD sample pins
 */
void LCDStrobeEnable(void)
{
    LCD_ENABLE = 1;
    Delay10us(5);
    //DelayMs(5);
    LCD_ENABLE = 0;
    Delay10us(5);
    //DelayMs(5);
}

/**
 * @brief bit-bang lower nibble
 * @param data byte to write
 */
void LCDLowNibPut(uint8_t data)
{
    LCD_RS = 0;
    LCD_ENABLE = 1;
    LCD_D3 = (data & 0x08) >> 3;
    LCD_D2 = (data & 0x04) >> 2;
    LCD_D1 = (data & 0x02) >> 1;
    LCD_D0 = (data & 0x01);
    DelayMs(1);
    LCD_ENABLE = 0;
    DelayMs(1);
}	 

/**
 * @brief bit-bang upper nibble to LCD
 * @param data byte to write
 */
void LCDDataPut(unsigned char data)
{

    LCD_D3 = (data & 0x80) >> 7;
    LCD_D2 = (data & 0x40) >> 6;
    LCD_D1 = (data & 0x20) >> 5;
    LCD_D0 = (data & 0x10) >> 4;
    LCDStrobeEnable();
    DelayMs(2);


    LCD_D3 = (data & 0x08) >> 3;
    LCD_D2 = (data & 0x04) >> 2;
    LCD_D1 = (data & 0x02) >> 1;
    LCD_D0 = (data & 0x01);
    Delay10us(100);
    LCDStrobeEnable();
    DelayMs(2);
    

}

/**
 * @brief writing to the internals of the LCD
 * @param command LCD command
 *
 * @note see the NT7603 datasheet for commands and timing specs
 */
void LCDCommand(unsigned char command)
{
    LCD_RS = 0;
    LCDDataPut(command);
}

/**
 * @brief write a single data byte to the LCD
 * @param character character to write
 */
void LCDPut(unsigned char character)
{
    ////@todo For some reason, the 8bit IR demo must call GIE directly and not EDF_INT_DISABLE or else refreshing
    ////      the LCD takes approx 500ms for the second line to occur. A weird annomaly. --JJB
#if defined (__PICC__)
    GIE = 0;
#else
    EDF_INT_Disable();
#endif
    LCD_RS = 1; // Data -- not instruction
    LCDDataPut(character);

#if defined(__PICC__)
    GIE = 1;
#else
    EDF_INT_Enable();
#endif
}	

/**
 * @brief Initilize the LCD
 *  - 4-bit mode
 *  - two lines
 *  - cursor OFF
 *  - blinking OFF
 *  - clear screen
 */
void LCDInit(void)
{
    LCD_D0_TRIS = 0;
    LCD_D1_TRIS = 0;
    LCD_D2_TRIS = 0;
    LCD_D3_TRIS	= 0;

    LCD_RS_TRIS = 0;
    LCD_RS 		= 0;	

    LCD_ENABLE_TRIS = 0;
    LCD_ENABLE 	    = 0;


    LCDLowNibPut(0x03);
    LCDLowNibPut(0x03);
    LCDLowNibPut(0x03);
    LCDLowNibPut(0x02);
    LCDLowNibPut(0x02);
    LCDLowNibPut(0x0C);
    LCDLowNibPut(0x00);
    LCDLowNibPut(0x08);
    LCDLowNibPut(0x00);
    LCDLowNibPut(0x01);
    LCDLowNibPut(0x00);
    LCDLowNibPut(0x06);


    LCDCommand( FUNCTION_SET | FOUR_BITS | TWO_LINES | FIVEXEIGHT );
    LCDCommand( DISPLAY_CONTROL | DISPLAY_ON | CURSOR_OFF | BLINKING_OFF );
    LCDCommand( ENTRY_MODE_SET | INCREMENT | SHIFT_OFF);


    LCDCommand(CLEAR_DISPLAY);
    LCDCommand(RETURN_HOME);

}

/**
 * @brief Update the LCD using LCDBuffer[0] and LCDBuffer[1]
 * @pre load the LCD buffers before updating
 *
 * @note will always RETURN_HOME before writing
 */
void LCDUpdate(void)
{

    uint8_t cursPos;
    //uint8_t i,j;

    LCDCommand(RETURN_HOME);

    for( cursPos = 0; cursPos < LCD_LINE_LENGTH; cursPos++ )
    {
        if(LCDBuffer[0][cursPos] == '\0'){break;}
        LCDPut(LCDBuffer[0][cursPos]);

    }
    LCDCommand(SET_DDRAM_ADDR|40);
    for( cursPos = 0; cursPos < LCD_LINE_LENGTH; cursPos++ )
    {
        if(LCDBuffer[1][cursPos] == '\0'){break;}
        LCDPut(LCDBuffer[1][cursPos]);

    }
    /*
    for( i = 0; i < LCD_LINES; i++)
        for( j = 0; j < LCD_LINE_LENGTH; j++)
            LCDBuffer[i][j] = '\0';*/


} 

/** @}*/

