/*****************************************************************************
 *
 * Real Time Clock Calender
 *
 *****************************************************************************
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Chris Valenti		05-26-05	...	
 * Ross Fosler			06-06-2005	Several changes
 * Anton Alkhimenok     10-21-2005  Get/Set functions
 * Anton Alkhimenok     02-07-2008  PIC32 support
 *****************************************************************************/
/**
 * @brief performs all RTCC tasks 
 * @file OnePIC_rtcc.c
 *
 * @author Chris Valenti
 * @date 05-26-05
 *
 * @author Ross Fosler
 * @date 06-06-2005
 *
 * @author Anton Alkhimenok
 * @date 10-21-2005
 *
 * @author Anton Alkhimenok
 * @date 02-07-2008
 * 
 * @defgroup rtcc RTCC
 * @ingroup base
 * @{
 * 
 *
 */
#include "OnePIC_rtcc.h"
#include "Compiler.h"
#include "OnePIC_i2c.h"
#include <string.h>

/*
 * @brief These structures contain the time and date.
 * RTCCProcessEvents updates them. _time_chk is used as input/output for
 * get/set operations.
 */


RTCC _time; /**<@brief contains the time*/
RTCC _time_chk; /**<@brief used as input/output for get/set operations */
RTCC _pwr_off; /**<@brief used to save time when power off*/
RTCC _pwr_on; /**<@brief used to reload time when powered on*/

unsigned char temp[10]; /**<@brief temporary buffer*/
/*****************************************************************************
 * RTCCProcessEvents updates these.
 *****************************************************************************/
char _day_time_str[16] = "                "; /**<@brief Contain date/time string -> Sat 10:01:15 */
char _date_str[16] = "                "; /**<@brief Contians date string -> Sep 30, 2005 */
// XXX 10/4/2011 8:39:40 AM: Changed original _time_str to _day_time_str
char _time_str[9] = "         "; /**<@brief   containg time string    12:32:49 */
// .
unsigned char _rtcc_flag; /**<@brief The flag stops updating time and date and used for get/set operations*/
unsigned char rtccBuf[32]; /**<@brief contains all RTCC bytes*/
unsigned char rtccWriteBuf[10]; /**<@brief used for write operations*/

/**
 * @var rtcStart
 * @brief Sets "start" and VbatEn
 */
struct trb rtcStart = {
    0x6f, // Addr
    5, // 5 write bytes
    0, // 0 bytes to read
    (unsigned char*) "\000\200\000\000\010", // Set the Start bit
    rtccBuf,
    0, // no retries
    0, // status is 0
    (void *) 0 // no next pointer
};

/**
 * @var rtcClearVbat
 * @brief clears the RTcc
 */
struct trb rtcClearVbat = {
    0x6f, // Addr
    2, // 2 write bytes
    0, // 0 bytes to read
    (unsigned char*) "\003\011", // Clear Vbat bit
    rtccBuf,
    0, // no retries
    0, // status is 0
    (void *) 0 // no next pointer
};

/**
 * @var rtcRead
 * @brief Reads 1 byte at address 0x20 (seconds)
 */
struct trb rtcRead = {
    0x6f, // Addr
    1, // 1 write byte
    0x20, // 0x20 bytes to read
    rtccWriteBuf, // Register address
    rtccBuf,
    0, // no retries
    0, // status is 0
    (void *) 0 // no next pointer
};

/**
 * @var rtcWrite
 * @brief write to RTCC
 */
struct trb rtcWrite = {
    0x6f, // Addr
    1, // 1 write byte
    0, // bytes to read
    rtccWriteBuf,
    rtccBuf,
    0, // no retries
    0, // status is 0
    (void *) 0 // no next pointer
};

/**
 * @var rtcSaveInRAM
 * @brief also write to its RAM
 */
struct trb rtcSaveInRAM = {
    0x6f, // Addr
    4, // 4 write byte
    0, // bytes to read
    rtccWriteBuf,
    rtccBuf,
    0, // no retries
    0, // status is 0
    (void *) 0 // no next pointer
};


/**
 * @brief read the RTCC current Time
 * @param t RTCC structure that holds RTCC register copy
 * @pre RTCCInit must be called before.
 */
void RTCCReadTime(RTCC *t) {
    static unsigned char prevSec = 0;
    rtccWriteBuf[0] = 0; // register address
    rtcRead.rxBuf = (unsigned char *) t;
    rtcRead.rxLen = sizeof (*t);
    trbEnQ(&bus1, &rtcRead);
    while ((rtcRead.status & 0x80) == 0) // wait until done
        ;
    if (t->sec != prevSec) {
        prevSec = t->sec;
        rtccWriteBuf[0] = 0x22;
        rtccWriteBuf[1] = ((unsigned char *) t)[0];
        rtccWriteBuf[2] = ((unsigned char *) t)[1];
        rtccWriteBuf[3] = ((unsigned char *) t)[2];
        trbEnQ(&bus1, &rtcSaveInRAM);
        while ((rtcSaveInRAM.status & 0x80) == 0) // wait until done
            ;
    }
}

/**
 * @brief read the RTCC current Time
 * @param t RTCC structure that holds RTCC register copy
 */
void RTCCReadOffTime(RTCC *t) {
    // Unused warning: static unsigned char prevSec=0;
    rtccWriteBuf[0] = 0x22; // register address
    rtcRead.rxBuf = (unsigned char *) t;
    rtcRead.rxLen = sizeof (*t);
    trbEnQ(&bus1, &rtcRead);
    while ((rtcRead.status & 0x80) == 0) // wait until done
        ;
}


/**
 * @brief Write the RTCC current Time
 * @pre RTCCInit must be called before.
 * @param t RTCC structure that holds RTCC register copy
 * 
 */
void RTCCWriteTime(RTCC *t) {
    rtccWriteBuf[0] = 0; // register address
    memcpy(rtccWriteBuf + 1, t, sizeof (*t));
    rtcWrite.txLen = sizeof (*t) + 1;
    trbEnQ(&bus1, &rtcWrite);
    while ((rtcWrite.status & 0x80) == 0) // wait until done
        ;
}

/**
 * @brief grabs the current time from the RTCC and translate
 * it into strings.
 *
 * It updates time and date strings  _day_time_str, _date_str,
 * and _time, _time_chk structures.
 *
 * @pre RTCCInit must be called before.
 *
 * 
 */
void RTCCProcessEvents(void) {


    // Process time object only if time is not being set
    while (!_rtcc_flag) {
        // Grab the time
        RTCCReadTime(&_time);

        // Grab the time again
        RTCCReadTime(&_time_chk);

        // Verify there is no roll-over
        if (!memcmp(&_time, &_time_chk, sizeof (_time))) {
            switch (_time.mth) {
                default:

                case 0x01:
                    _date_str[0] = 'J';
                    _date_str[1] = 'a';
                    _date_str[2] = 'n';
                    break;

                case 0x02:
                    _date_str[0] = 'F';
                    _date_str[1] = 'e';
                    _date_str[2] = 'b';
                    break;

                case 0x03:
                    _date_str[0] = 'M';
                    _date_str[1] = 'a';
                    _date_str[2] = 'r';
                    break;

                case 0x04:
                    _date_str[0] = 'A';
                    _date_str[1] = 'p';
                    _date_str[2] = 'r';
                    break;

                case 0x05:
                    _date_str[0] = 'M';
                    _date_str[1] = 'a';
                    _date_str[2] = 'y';
                    break;

                case 0x06:
                    _date_str[0] = 'J';
                    _date_str[1] = 'u';
                    _date_str[2] = 'n';
                    break;

                case 0x07:
                    _date_str[0] = 'J';
                    _date_str[1] = 'u';
                    _date_str[2] = 'l';
                    break;

                case 0x08:
                    _date_str[0] = 'A';
                    _date_str[1] = 'u';
                    _date_str[2] = 'g';
                    break;

                case 0x09:
                    _date_str[0] = 'S';
                    _date_str[1] = 'e';
                    _date_str[2] = 'p';
                    break;

                case 0x10:
                    _date_str[0] = 'O';
                    _date_str[1] = 'c';
                    _date_str[2] = 't';
                    break;

                case 0x11:
                    _date_str[0] = 'N';
                    _date_str[1] = 'o';
                    _date_str[2] = 'v';
                    break;

                case 0x12:
                    _date_str[0] = 'D';
                    _date_str[1] = 'e';
                    _date_str[2] = 'c';
                    break;
            }

            _date_str[3] = ' ';
            _date_str[6] = ',';
            _date_str[7] = ' ';
            _date_str[8] = '2';
            _date_str[9] = '0';

            _date_str[4] = (_time.day >> 4) + '0';
            _date_str[5] = (_time.day & 0xF) + '0';

            _date_str[10] = (_time.yr >> 4) + '0';
            _date_str[11] = (_time.yr & 0xF) + '0';

            switch (_time.wkd) {
                default:

                case 0x00:
                    _day_time_str[0] = 'S';
                    _day_time_str[1] = 'u';
                    _day_time_str[2] = 'n';
                    break;

                case 0x01:
                    _day_time_str[0] = 'M';
                    _day_time_str[1] = 'o';
                    _day_time_str[2] = 'n';
                    break;

                case 0x02:
                    _day_time_str[0] = 'T';
                    _day_time_str[1] = 'u';
                    _day_time_str[2] = 'e';
                    break;

                case 0x03:
                    _day_time_str[0] = 'W';
                    _day_time_str[1] = 'e';
                    _day_time_str[2] = 'd';
                    break;

                case 0x04:
                    _day_time_str[0] = 'T';
                    _day_time_str[1] = 'h';
                    _day_time_str[2] = 'u';
                    break;

                case 0x05:
                    _day_time_str[0] = 'F';
                    _day_time_str[1] = 'r';
                    _day_time_str[2] = 'i';
                    break;

                case 0x06:
                    _day_time_str[0] = 'S';
                    _day_time_str[1] = 'a';
                    _day_time_str[2] = 't';
                    break;
            }

            _day_time_str[3] = ' ';
            _day_time_str[6] = ':';
            _day_time_str[9] = ':';

            _day_time_str[4] = (_time.hr >> 4) + '0';
            _day_time_str[5] = (_time.hr & 0xF) + '0';

            _day_time_str[7] = (_time.min >> 4) + '0';
            _day_time_str[8] = (_time.min & 0xF) + '0';

            _day_time_str[10] = (_time.sec >> 4) + '0';
            _day_time_str[11] = (_time.sec & 0xF) + '0';

            _time_str[0] = _day_time_str[4];
            _time_str[1] = _day_time_str[5];
            _time_str[2] = _day_time_str[6];
            _time_str[3] = _day_time_str[7];
            _time_str[4] = _day_time_str[8];
            _time_str[5] = _day_time_str[9];
            _time_str[6] = _day_time_str[10];
            _time_str[7] = _day_time_str[11];
            _time_str[8] = '\0';


            break;
        }
    }
}


/**
 * @brief Enable the oscillator for the RTCC
 * @note call this first before any other RTCC related function
 */
void RTCCInit(void) {

    RTCCReadOffTime(&_pwr_off);
    RTCCReadTime(&_pwr_on);
    RTCCReadTime(&_time);
    _time.vbatEn = 0;
    if (_time.start == 0 || _time.vbatEn == 0) {
        _time.start = 0; // Stop Oscillator
        RTCCWriteTime(&_time);

        // EXTOSC ON
        /*		rtccWriteBuf[0] = 0x07;	// register address
                rtccWriteBuf[1] = 0x4B;
                rtcWrite.txLen = 2;
                trbEnQ(&bus1, &rtcWrite);
                while ((rtcWrite.status & 0x80) == 0)	// wait until done
         */
        _time.time12 = 0;
        _time.start = 1;
        _time.vbatEn = 1;
        _time.sec = 0x00;
        _time.min = 0x00;
        _time.hr = 0x12;
        _time.wkd = 3;
        _time.day = 0x14;
        _time.mth = 0x09;
        _time.yr = 0x11;
        RTCCWriteTime(&_time);

        rtccWriteBuf[0] = 0x07; // register address
        rtcRead.rxBuf = temp;
        rtcRead.rxLen = 1;
        trbEnQ(&bus1, &rtcRead);
        while ((rtcRead.status & 0x80) == 0) // wait until done
            ;
    }
}

/**
 * @brief Restart the clock
 */
void RTCCClear() {
    _time.start = 0; // Stop Oscillator
    RTCCWriteTime(&_time);
    RTCCInit();
}

/**
 * @brief uploads time and date from _time_chk into clock.
 * @see RTCC::_time_chk
 * @note Input: _time_chk - structure containing time and date.
 */
void RTCCSet(void) {
    // Set the time
    RTCCWriteTime(&_time_chk);
    _rtcc_flag = 0; // Release the lock on the time
}


/**
 * @brief allows a writing into the clock registers.
 */
void RTCCUnlock(void) {
#ifdef __PIC32MX__
    SYSKEY = 0xaa996655; // write first unlock key to SYSKEY
    SYSKEY = 0x556699aa; // write second unlock key to SYSKEY
    RTCCONSET = 0x8; // set RTCWREN in RTCCONSET
#endif
    // XXX 10/5/2011 4:42:26 PM: Added ifdef for PIC24/16
#ifdef __PIC24FJ256GA106
    asm volatile("disi	#5");
    asm volatile("mov	#0x55, w7");
    asm volatile("mov	w7, _NVMKEY");
    asm volatile("mov	#0xAA, w8");
    asm volatile("mov	w8, _NVMKEY");

    //	asm volatile("bset	_NVMCON, #15");
    asm volatile("bset	_RCFGCAL, #13");
    asm volatile("nop");
    asm volatile("nop");
#endif

#ifdef _16F1939
    // XXX 10/6/2011 8:35:28 AM: not sure what needs to happen for the PIC16F, the following code was here commented out already.
    // XXX 12/15/2011 10:15:28 AM: this writes to the EEPROM - shown below is the required sequence
    //	EECON2 = 0x55;
    //	EECON2 = 0xAA;
    //	RCFGCALbits.RTCWREN = 1;
#endif
}



/**
 * @brief verifies setting seconds range, translates it into
 * BCD format and writes into _time_chk structure. To write the structure into
 * clock RTCCSet must be called.
 * 
 * @param Sec Seconds binary value.
 * @return Checked BCD value in _time_chk structure.
 */
void RTCCSetBinSec(unsigned char Sec) {
    if (Sec == 0xff)
        Sec = 59;
    if (Sec == 60)
        Sec = 0;
    mRTCCSetSec(mRTCCBin2Dec(Sec));
}

/**
 * @brief verifies a setting minutes range, translates it into
 * BCD format and writes into _time_chk structure. To write the structure into
 * clock RTCCSet must be called.
 * 
 * @param Min Minutes binary value.
 * @return Checked BCD value in _time_chk structure
 */
void RTCCSetBinMin(unsigned char Min) {
    if (Min == 0xff)
        Min = 59;
    if (Min == 60)
        Min = 0;
    mRTCCSetMin(mRTCCBin2Dec(Min));
}

/**
 * @brief verifies a setting hours range, translates it into
 * BCD format and writes into _time_chk structure. To write the structure into
 * clock RTCCSet must be called.
 * 
 * @param Hour Hours binary value.
 * @return Checked BCD value in _time_chk structure.
 */
void RTCCSetBinHour(unsigned char Hour) {
    if (Hour == 0xff)
        Hour = 23;
    if (Hour == 24)
        Hour = 0;
    mRTCCSetHour(mRTCCBin2Dec(Hour));
}


/**
 * @brief reads day, month and year from _time_chk and
 * calculates week day. Than It writes result into _time_chk. To write
 * the structure into clock RTCCSet must be called.
 *
 * @pre Valid values of day, month and year must be presented in
 * _time_chk structure.
 * @note input:  _time_chk with valid values of day, month and year.
 * @return Zero based week day in _time_chk structure.
 * 
 */

void RTCCCalculateWeekDay(void) {
    const char MonthOffset[] =

            //jan feb mar apr may jun jul aug sep oct nov dec
    {0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5};
    unsigned Year;
    unsigned Month;
    unsigned Day;
    unsigned Offset;

    // calculate week day
    Year = mRTCCGetBinYear();
    Month = mRTCCGetBinMonth();
    Day = mRTCCGetBinDay();

    // 2000s century offset = 6 +
    // every year 365%7 = 1 day shift +
    // every leap year adds 1 day
    Offset = 6 + Year + Year / 4;

    // Add month offset from table
    Offset += MonthOffset[Month - 1];

    // Add day
    Offset += Day;

    // If it's a leap year and before March there's no additional day yet
    if ((Year % 4) == 0)
        if (Month < 3)
            Offset -= 1;

    // Week day is
    Offset %= 7;

    mRTCCSetWkDay(Offset);
}


/**
 * @brief verifies a setting day range, translates it into
 * BCD format and writes into _time_chk structure. To write the structure into
 * clock RTCCSet must be called.
 *
 * @param Day Day binary value.
 * @return Checked BCD value in _time_chk structure.
 */
void RTCCSetBinDay(unsigned char Day) {
    const char MonthDaymax[] =

            //jan feb mar apr may jun jul aug sep oct nov dec
    {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    unsigned Daymax;
    unsigned Month;
    unsigned Year;

    Month = mRTCCGetBinMonth();
    Year = mRTCCGetBinYear();

    Daymax = MonthDaymax[Month - 1];

    // February has one day more for a leap year
    if (Month == 2)
        if ((Year % 4) == 0)
            Daymax++;

    if (Day == 0)
        Day = Daymax;
    if (Day > Daymax)
        Day = 1;
    mRTCCSetDay(mRTCCBin2Dec(Day));
}

/**
 * @brief verifies a setting month range, translates it into
 * BCD format and writes into _time_chk structure. To write the structure into
 * clock RTCCSet must be called.
 * 
 * @param Month Month binary value.
 * @return Checked BCD value in _time_chk structure.
 */
void RTCCSetBinMonth(unsigned char Month) {
    if (Month < 1)
        Month = 12;
    if (Month > 12)
        Month = 1;
    mRTCCSetMonth(mRTCCBin2Dec(Month));
}


/**
 * @brief verifies a setting year range, translates it into
 * BCD format and writes into _time_chk structure. To write the structure into
 * clock RTCCSet must be called.
 * @param Year Year binary value.
 * @return Checked BCD value in _time_chk structure.
 */
void RTCCSetBinYear(unsigned char Year) {
    if (Year == 0xff)
        Year = 99;
    if (Year == 100)
        Year = 0;
    mRTCCSetYear(mRTCCBin2Dec(Year));

    // Recheck day. Leap year influences to Feb 28/29.
    RTCCSetBinDay(mRTCCGetBinDay());
}
/** @}*/
/*****************************************************************************
 * EOF
 *****************************************************************************/
