
/**
 * @file OnePIC_i2c.h
 * @author Larry Gass
 * @date 2010-10-11
 * @ingroup base
 * @{
 *
 */

/****************************************************************************
  Section: PIC Configuration
  ***************************************************************************/
#ifndef _EDF_BUS_H
#define _EDF_BUS_H

/**
 * @enum i2cState
 * @brief Contains all I2C states that the state machine can be in
 * @var i2cState::I2C_IDLE
 * i2c no activity
 * @var i2cState::I2C_STARTING
 * sent start bit
 * @var i2cState::I2C_XMITING
 * transmitting
 * @var i2cState::I2C_RESTARTING
 * restarting comms
 * @var i2cState::I2C_RECEIVING
 * receiving a byte
 * @var i2cState::I2C_ACKING
 * acknowledging to other devices
 * @var i2cState::I2C_STOPPING
 * stopping communicatin
 * @var i2cState::I2C_STOP_IN_ENQ
 * stop in progress
 * @var i2cState::I2C_ERRORS
 * error flag (did slave device ack?)
 *
 */
enum i2cState
{
	I2C_IDLE, 
	I2C_STARTING,
	I2C_XMITING,
	I2C_RECEIVEADDR,
	I2C_RESTARTING,
	I2C_RECEIVING,
	I2C_ACKING,
	I2C_STOPPING,
	I2C_STOP_IN_ENQ,
	I2C_ERRORS
};

/**
 * @struct trb
 * @brief Contains I2C attributes such as:
 *  - address
 *  - # of bits per byte in rx/tx
 *  - tx/rx buffer
 *  - # of retries
 *  - status of i2c
 * @var trb::addr
 * slave address
 * @var trb::txLen
 * num of bits in transmission
 * @var trb::rxLen
 * num of bits in reception
 * @var trb::txBuf
 * pointer to the transmitted byte
 * @var trb::rxBuf
 * pointer to the received byte
 * @var trb::retries
 * num of allowed retries
 * @var trb::status
 * bit 7 - done, bit 0 - NAK
 * @var trb::next
 * pointer to next byte
 *
 * @note talk to larry
 */
struct trb
{
	unsigned char addr;
	unsigned char txLen;
	unsigned char rxLen;
	const unsigned char *txBuf;
	unsigned char *rxBuf;
	unsigned char retries;
	volatile unsigned char status;	
	struct trb *next;
};

/**
 * @struct i2cBus
 * @brief Holds the statemachine properties
 * @var i2cBus::mstate
 * the current state of the bus
 * @var i2cBus::stopLock
 * wether or not to hold the line busy
 * @var i2cBus::bufOffset
 * num of bits offset in the single byte of the tx buffer
 * @var i2cBus::req
 * pointer back to the properties
 * @var i2cBus::tail
 * ?
 * 
 */
struct i2cBus
{
	enum i2cState mstate;
	unsigned char stopLock;
	unsigned char bufOffset;
	struct trb *req;
	struct trb **tail;
};

int i2cIdle(struct i2cBus *bus);
void trbEnQ(struct i2cBus *bus, struct trb *req);
void I2CInit (void);
#if defined(__PIC32MX__) || defined(__PICC__)
void ISR_i2c(void);
#endif
extern struct i2cBus bus1;
#endif

/** @}*/
