/*****************************************************************************
 *
 * Real Time Clock Calender
 *
 *****************************************************************************
 * FileName:        rtcc.h
 * Dependencies:    
 * Processor:       PIC24, PIC32
 * Compiler:       	C30 xx.xx or higher, C32
 * Linker:          MPLINK 03.20.01 or higher, MPLINK32
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Chris Valenti		05-26-05	...	
 * Ross Fosler			06-06-2005	Several changes
 * Anton Alkhimenok     10-21-2005  Get/Set functions
 * Anton Alkhimenok     02-07-2008  PIC32 support
 *****************************************************************************/

/**
 * @file OnePIC_rtcc.h
 * 
 * @ingroup base
 * @{
 */

// Setup clock
extern void RTCCInit(void);
// Must be called periodically to refresh time and date strings
extern void RTCCProcessEvents(void);
// Write changed time and date to clock registers
extern void RTCCSet(void);


/**
 * @struct tagRTCC OnePIC_rtcc.h OnePIC_rtcc
 * @brief Union to access rtcc registers
 *
 * @var tagRTCC::sec
 * seconds
 * @var tagRTCC::start
 * start flag
 * @var tagRTCC::min
 * minutes
 * @var tagRTCC::res1
 * first digit
 * @var tagRTCC::hr
 * hour
 * @var tagRTCC::time12
 * noon
 * @var tagRTCC::res2
 * second digit
 * @var tagRTCC::wkd
 * weekday
 * @var tagRTCC::vbatEn
 * enable external battery
 * @var tagRTCC::vbat
 * battery
 * @var tagRTCC::oscOn
 * enable external oscillator
 * @var tagRTCC::res3
 * 3rd digit
 * @var tagRTCC::day
 * day
 * @var tagRTCC::res4
 * 4th digit
 * @var tagRTCC::mth
 * month
 * @var tagRTCC::leapyear
 * leapyear
 * @var tagRTCC::res5
 * 5th digit
 * @var tagRTCC::yr
 * year
 *
 * @ingroup thisgroup
 *
 */
typedef struct tagRTCC{
    unsigned char sec : 7;
    unsigned char start : 1;
    unsigned char min : 7;
    unsigned char res1 : 1;
    unsigned char hr : 6;
    unsigned char time12 : 1;
    unsigned char res2 : 1;
    unsigned char wkd : 3;
    unsigned char vbatEn : 1;
    unsigned char vbat : 1;
    unsigned char oscOn : 1;
    unsigned char res3 : 2;
    unsigned char day : 6;
    unsigned char res4 : 2;
    unsigned char mth : 5;
    unsigned char leapyear : 1;
    unsigned char res5 : 2;
    unsigned char yr;
} RTCC;

// Gets time and date
extern RTCC _time;
extern RTCC _pwr_on;
extern RTCC _pwr_off;

// Gets time and date second time to detect overflow
extern RTCC _time_chk;

// Prevents updating of _time and _time_chk
extern unsigned char _rtcc_flag;

// _time_str and _date_str contain current time and date strings
extern char _day_time_str[16]; // Sat 10:01:15
extern char _date_str[16]; // Sep 30, 2005
extern char _time_str[9];
// Macros and functions to get/verify and set time and date
extern void RTCCSetBinSec(unsigned char Sec);
extern void RTCCSetBinMin(unsigned char Min);
extern void RTCCSetBinHour(unsigned char Hour);
extern void RTCCSetBinDay(unsigned char Day);
extern void RTCCSetBinMonth(unsigned char Month);

// Set a new year value and correct Feb 29 for none leap year
extern void RTCCSetBinYear(unsigned char Year);

extern void RTCCReadTime(RTCC *t);

// Reads current year, month, day, calculates and set week day
extern void RTCCCalculateWeekDay(void);


#define mRTCCInit() RTCCInit() /**<@brief Sets the RTCC to 00:00:00 */
#define mRTCCSet()  RTCCSet() /**<@brief  Sets the RTCC to 00:00:00 */
#define mRTCCGetSec()   _time_chk.sec /**<@brief returns the second */
#define mRTCCGetMin()   _time_chk.min /**<@brief returns the minute*/
#define mRTCCGetHour()  _time_chk.hr /**<@brief returns the hour*/
#define mRTCCGetWkDay() (_time_chk.wkd-1) /**<@brief returns the current weekday*/
#define mRTCCGetDay()   _time_chk.day /**<@brief returns the day*/
#define mRTCCGetMonth() _time_chk.mth /**<@brief returns the month */
#define mRTCCGetYear()  _time_chk.yr /**<@brief returns the year */

/**@brief sets the second*/
#define mRTCCSetSec(__rtccdat) \
    _time_chk.sec = __rtccdat; \
    _rtcc_flag = 1;
/**@brief set minute*/
#define mRTCCSetMin(__rtccdat) \
    _time_chk.min = __rtccdat; \
    _rtcc_flag = 1;
/**@brief set hour*/
#define mRTCCSetHour(__rtccdat) \
    _time_chk.hr = __rtccdat;   \
    _rtcc_flag = 1;
/**@brief set weekday*/
#define mRTCCSetWkDay(__rtccdat) \
    _time_chk.wkd = __rtccdat+1;   \
    _rtcc_flag = 1;
/**@brief set day*/
#define mRTCCSetDay(__rtccdat) \
    _time_chk.day = __rtccdat; \
    _rtcc_flag = 1;
/**@brief set month*/
#define mRTCCSetMonth(__rtccdat) \
    _time_chk.mth = __rtccdat;   \
    _rtcc_flag = 1;
/**@brief set year*/
#define mRTCCSetYear(__rtccdat) \
    _time_chk.yr = __rtccdat;   \
    _rtcc_flag = 1;

#define mRTCCDec2Bin(Dec)   (10 * (Dec >> 4) + (Dec & 0x0f)) /**@brief quick & dirty decimal to binary conversion. Use this before printing to LCD */
#define mRTCCBin2Dec(Bin)   (((Bin / 10) << 4) | (Bin % 10)) /**@brief quick & dirty binary to decimal conversion */
#define mRTCCGetBinSec()    mRTCCDec2Bin(mRTCCGetSec()) /**@brief get the binary equivalent of the second*/
#define mRTCCGetBinMin()    mRTCCDec2Bin(mRTCCGetMin()) /**@brief get the binary equivalent of the minute */
#define mRTCCGetBinHour()   mRTCCDec2Bin(mRTCCGetHour()) /**@brief get the binary equivalent of the hour */
#define mRTCCGetBinWkDay()  mRTCCDec2Bin(mRTCCGetWkDay()) /**@brief get the binary equivalent of the weekday */
#define mRTCCGetBinDay()    mRTCCDec2Bin(mRTCCGetDay()) /**@brief get the binary equivalent of the day */
#define mRTCCGetBinMonth()  mRTCCDec2Bin(mRTCCGetMonth()) /**@brief get the binary equivalent of the month */
#define mRTCCGetBinYear()   mRTCCDec2Bin(mRTCCGetYear()) /**@brief get the binary equivalent of the year */

/*****************************************************************************
 * EOF
 *****************************************************************************/
/** @}*/
